#' #' Causal Graph G
#' #' @details For each \code{Y}, run \code{Vario_subset_search} to get all ingoing edges \code{X->Y (PI)}, where Pi, X is admissible
#' #' create a DAG by removing ancestral edges using \code{Vario_G}
#' #' @field adj_admissible from Vario
#' #' @field adj from VarioG
#' #' @field edges_admissible from Vario
#' #' @field edges_pruned from VarioG
#' #' @examples Classes overview:
#' #' cls_graph (all targets Y)
#' #' > cls_subset_search (one target Y, all subsets X)
#' #' > cls_pi_search (one subset, all partitions)
#' #' > cls_pi (one partition, all coefficients)
#' #' > cls_phi (one coefficient) 
#' cls_Graph = setRefClass("cls_Graph",
#'                          fields = list(
#'                            adj_admissible = "matrix",
#'                            adj_admissible_confident = "matrix",
#'                            adj = "matrix",
#'                            edges_admissible = "list",
#'                            edges_admissible_confident = "list",
#'                            edges = "list",
#'                            variables = "character"
#'                          ))
#' 
#' cls_Graph$methods(
#'   init = function(variables, prune){
#'     variables <<- variables
#'     prune <<- prune
#'     edges_in  = lapply(variables, function (y) return(list()))
#'     edges_out = lapply(variables, function (y) return(list()))
#'     nodes = length(variables)
#'     adj <<- matrix(0, nodes, nodes)
#'   })
#' 
#'   #  Adds a result of Vario search for one variable Y -----
#' cls_Graph$methods(
#'   add_result = function(res){
#'     add_ranking(res$RESULT$Ranking, res$x_named[res$y_target])
#'   },
#'   add_ranking = function(Ranking, ynm){
#'     for(i in 1:length(Ranking)){
#'       entry = Ranking[[i]]
#'       entry_nm = names(Ranking)[i]
#' 
#'       # Up to last relevant edge
#'       one_group = (length(unique(entry$E_part)) == 1)
#'       max_group = (length(unique(entry$E_part))==length(entry$E_part))
#'       if(one_group | max_group)
#'         break
#' 
#'       # Names
#'       xid = which(variables==entry$xnm)
#'       yid = which(variables==ynm)
#'       ID = paste0("(", paste0(xid, paste(",", paste0(yid, ")"))))
#'       nm = paste(paste(ID, renameErkAkt(entry$xnm ), paste("->", paste(renameErkAkt(ynm), paste("(",paste(paste(entry$E_part, collapse=""), ")"))))))
#' 
#'       #Entries
#'       one_intervention = (length(unique(entry$E_part))==2 & length(which(entry$E_part==2)==1))
#'       if(prune & !one_intervention)
#'       {
#'         false_positives[[nm]] <<- entry$score_norm
#'         next
#'       }
#'       edges[[nm]] <<- entry$score_norm
#'       edges_in[[yid]][[nm]] = entry$score_norm
#'       edges_out[[xid]][[nm]]  = entry$score_norm
#'       adj[xid, yid] <<- 1
#'     }
#'   })
#' 
#'   #  After results have been added, match with ground truth ------
#' cls_Graph$methods(
#'   accuracies_Sachs = function(graph){
#'     Gvario  = graph$adj
#'     Gtruth = Vario:::consensus_graph_Yall(graph$variables)
#'     Gsum = Gtruth + Gvario
#'     Gdiff = Gvario - Gtruth
#' 
#'     tp = sum(apply(Gsum, 2, function(row) return(length(which(row==2)))))
#'     fp = sum(apply(Gdiff, 2, function(row) return(length(which(row==1)))))
#'     fn = sum(apply(Gdiff, 2, function(row) return(length(which(row==-1)))))
#' 
#'     return(list(tp=tp, fp=fp, fn=fn,
#'                 fone=(tp / (tp + 0.5*(fp + fn))),
#'                 precision=tp/(tp+fp),
#'                 recall=tp/(tp+fn),
#'                 sid=structIntervDist(Gtruth, Gvario)))
#'   })
#' 
