#' 
#' #' Class for one function f:X->Y and its regression coefficients 
#' #' @details Has results for one subset \code{\{X1,...Xm\}} and one function \code{Y=f(X1,...Xm; varphi_1,...varphi_m)}. 
#' #' Search goes over all coefficients (varphi_i) to find a partition/clustering by similarity (Pi). Result is a Ranking of candidates Pi.
#' #' @field Data_E1_En,Dstd_E1_En Data,Data standardized; list: \code{E -> (X1,..Xm, Y)}
#' #' @field Cf_X_E1_En regression coefficients; list: \code{X_i ->} mat: \code{E_j, varphi_ik} at ik
#' #' @field lconf,rconf,pval as Cf_X_E1_En with resp. conf.interval,pval.
#' #' @field Pi_histories,C_histories subclasses, information about a single partition/coefficient
#' #' @field Pi_ranking main result
#' #' @field Pi_ranking_raw,Pi_ranking_nogroup,Pi_ranking_nscaled,Pi_ranking_correctone test versions
#' #' @field htest_invariant different hypothesis tests on whether varphi are invariant
#' #' @examples Classes overview:
#' #' cls_subset_search (one target Y, all subsets X)
#' #' > cls_pi_search (one subset, all partitions) 
#' #' > cls_pi (one partition, all coefficients) 
#' #' > cls_phi (one coefficient) 
#' cls_phi = setRefClass(
#'   "cls_pi_search",
#'   fields = list(
#'     Data_E1_En = "list", 
#'     Dstd_E1_En = "list", 
#'     Cf_X_E1_En = "list", 
#'     lconf = "list", 
#'     rconf = "list",
#'     pval = "list", 
#'     Rejects_X_E1_En = "list", 
#'     Fun_id = "character", 
#'     X_n = "numeric",
#'     X_C_n = "numeric",
#'     E_n = "numeric", 
#'     Pi_histories = "environment", 
#'     C_histories = "environment", 
#'     #
#'     # Pi search result
#'     Pi_ranking = "list",
#'     #including conf based on confint sampling frpm pi one ->inconf cases set to 1 (or better singleton groups?)
#'     # Versions
#'     Pi_ranking_raw = "list",
#'     #not including any correction for one group or confidence (kmax accurate, k=1 never guessed)
#'     Pi_ranking_nogroup = "list",
#'     #confidence based on confint f pi kmax. problem: should rather generate samples randomly (as we do for k=1) than equidist (as for kmax)
#'     Pi_ranking_nscaled = "list",
#'     Pi_ranking_correctone = "list",
#'     #hypoth testing for inv. case.
#'     #
#'     #Pi_ranking_lconf="list",
#'     Pi_kbest = "list",
#'     scores_kbest = "numeric", 
#'     # special case one group
#'     Pi_one = "list", 
#'     # score confidence
#'     confints = "list", 
#'     C_conf = "list", 
#'     # invariance
#'     htest_invariant = "list",
#'     invariant_conservative = "logical",
#'     invariant_locally="logical",
#'     #Regression
#'     regress_E1_En = "list",
#'     greedy_k = "logical",
#'     meta_computing = "logical",
#'     done = "logical",
#'     done_kbest = "logical",
#'     speedy = "logical",
#'     verbose = "logical",
#'     #residuals
#'     Pi_resids ="list",
#'     version_mdl = "logical",
#'     version_simp = "logical"
#'   )
#' )
