
#' Partition Pi and its MDL score
#' @examples Classes overview:
#' cls_subset_search (one target Y, all subsets X) 
#' > cls_pi_search (one subset, all partitions) 
#' > cls_pi (one partition, all coefficients) 
#' > cls_varphi (one coefficient) 
#' @export
cls_Pi = setRefClass("cls_Pi",
                       fields = list(
                         E_part="numeric",  
                         K="numeric",  
                         #
                         # features X,coefficients C
                         X_n="numeric",  
                         X_C_n="numeric", 
                         sse="numeric",
                         # The final scores used:
                         scores_X_C="list",
                         score_sum="numeric",
                         #scaled score:
                         scoresB_X_C="list",
                         scoreB_sum="numeric",
                         # confidence bounds (not used):
                         lscores_X_C="list",
                         lscore_sum="numeric", 
                         rscores_X_C="list",
                         rscore_sum="numeric", 
                         inter_dist="numeric",  
                         # not scaled (not used)
                         scoresE_X_C="list",  #rename
                         lscoresE_X_C="list", 
                         rscoresE_X_C="list", 
                         scoreE_sum="numeric",
                         lscoreE_sum="numeric",
                         rscoreE_sum="numeric",
                         conf_sum="numeric", 
                         done = "logical",
                         version_mdl = "logical",
                         version_simp = "logical", 
                         version_mdl_nscaled = "logical",
                         slopes_1_1_each_k = "numeric", #debug
                         mses_1_1_each_k= "numeric",
                         precs = "numeric" 
                         )) 

cls_Pi$methods(
  init = function(E_part, X_n, X_C_n, version_mdl, version_simp, precomputing_mdl = FALSE){
    
   # cat("\n mdl:",version_mdl,"\nsimp:",version_simp)
    X_n <<- X_n
    X_C_n <<- X_C_n
    E_part <<- E_part
    K <<- length(unique(E_part))
    init_scores = lapply(1:X_n, function(xi)
      return(rep(1000, X_C_n[xi]))) 
    scores_X_C <<- init_scores
    scoresB_X_C <<- init_scores
    scoresE_X_C <<- init_scores
    lscores_X_C <<- init_scores
    lscoresE_X_C <<- init_scores
    rscores_X_C <<- init_scores
    rscoresE_X_C <<- init_scores
    
    score_sum <<- Inf 
    scoreE_sum <<- Inf
    scoreB_sum <<- Inf
    lscore_sum <<- 0 
    lscoreE_sum <<- 0
    rscore_sum <<- 0 
    rscoreE_sum <<- 0
    done <<- FALSE 
    version_mdl<<- version_mdl
    version_simp<<- version_simp
    
    E_n = length(E_part)
    
    
    #Simp:
    if(version_simp)
    precs <<- logg(E_n) / 2  
    #BINS = 8
    # precs <<- 2^(-logg(length(E_part)/BINS)/2)
 
    
    
    version_mdl_nscaled <<- FALSE    
    slopes_1_1_each_k <<-rep(0,K)
    mses_1_1_each_k <<-rep(0,K)
  }, 
  
  #Add one (Xi, Cj)-pair
  edit = function(C_history, C_part){
    xi = C_history$xi
    N = C_history$E_n 
    E_ord = C_history$E_ord
    ci = C_history$ci
    # Order check
    E_part_q = C_part[E_ord] 
    E_part_q = sort_classes_lex(E_part_q)
    sapply(1:N, function(i)
           assert(E_part[i]) == E_part_q[[i]]) 
    
    C_cf = C_history$C_cf
    C_cf_nscaled = C_history$C_cf_nscaled
    E_cf = C_history$E_cf
    k = length(unique(C_part)) 
    E_means = infer_means(E_part,E_cf) 
    
    
    C_lcf = C_history$C_lcf
    E_lcf = C_history$E_lcf
    C_lcf_nscaled = sort(E_lcf)
    E_lmeans = infer_means(E_part,E_lcf) 
    
    C_rcf = C_history$C_rcf
    E_rcf = C_history$E_rcf
    C_rcf_nscaled = sort(E_rcf)
    E_rmeans = infer_means(E_part,E_rcf) 
    
    #---- Coef errors -------
    # Adapt precision ----
    sample = c()
    BINS =  8
    binned_interval = sapply(1:BINS, function(i) return((100*i)/BINS))
    j = 0
    for (cf in C_cf){
      j  = j+1
      df = abs(binned_interval - cf)
      i= which(df==min(df))
      if(length(i)>1) i = i[1]
      sample[j] = binned_interval[i]
    }
    
sample = sapply(sample, function(i) return(i/10))
 sample = C_cf
    
    lmns = infer_means (C_part, C_lcf)
    lmns_nscaled = infer_means (C_part, C_lcf_nscaled)    
    rmns = infer_means (C_part, C_rcf)
    rmns_nscaled = infer_means (C_part, C_rcf_nscaled)
    mns_nscaled = infer_means (C_part, C_cf_nscaled)
    
    mns = infer_means (C_part, sample) 
    
    assert(length(mns)==length(unique(C_part))) 
     
    mse_k = logg(sum_assignment_mat(k, dispersion(k, mns, sample), ceil=FALSE)) 
  
    lmse_k = logg(sum_assignment_mat(k, dispersion(k, lmns, C_lcf)))
    rmse_k = logg(sum_assignment_mat(k, dispersion(k, rmns, C_rcf)))
     
    #---- Coef er not scaled -------
 
    mse_k_E = logg(mses_part(C_cf_nscaled, C_part))
    lmse_k_E = logg(mses_part(C_lcf_nscaled, C_part))
    rmse_k_E = logg(mses_part(C_rcf_nscaled, C_part))
     
    mse_1_E = C_history$mse_1_E
    lmse_1_E = C_history$lmse_1_E
    rmse_1_E = C_history$rmse_1_E
    
    
    s_k_positive_E =(mse_k_E - mse_1_E)/(k-1) + mse_1_E
    s_k_E = (mse_k_E - mse_1_E)/(k-1)  
    ls_k_positive_E =(lmse_k_E - lmse_1_E)/(k-1) + lmse_1_E
    ls_k_E = (lmse_k_E - lmse_1_E)/(k-1) 
    rs_k_positive_E =(rmse_k_E - rmse_1_E)/(k-1) + rmse_1_E
    rs_k_E = (rmse_k_E - rmse_1_E)/(k-1) 
    
  
    #---- Score -------
    mse_1 = C_history$mse_1
    s_k_positive =( mse_k - mse_1)/(k-1) + mse_1
    s_k = ( mse_k - mse_1)/(k-1) 
    mse = mse_k
    
    # ---- MDL version -------
    
    sig = 1 
    suppressWarnings(rm("pi"))
    
    L_loglik = -(-(N/2)*logg(2*pi*sig^2) - 1/(2*sig^2) * mse_k) 
    L_loglik_1 = -(-(N/2)*logg(2*pi*sig^2) - 1/(2*sig^2) * mse_1)
 
    #for debug
    if(ci == 1 & xi ==1)  mses_1_1_each_k[1]<<- mse_1
    if(ci == 1 & xi ==1)  mses_1_1_each_k[k]<<- mse_k

    if(ci == 1 & xi ==1 ) {   
      slopes_1_1_each_k[k] <<-L_loglik  
      slopes_1_1_each_k[1] <<-L_loglik_1
    }
    # Other options
    # p=2*N-1 
    #L_pi = logg(choose(N-1,N-k-1)) + logg(N) 

    p= 2
    L_alpha = logg(p) * max(k-1,1) 
    L_pi = N * logg(N) + logg(N) 
    L_const = 0  
    L_model = L_alpha + L_pi +  L_loglik + L_const 
 
    #L_alpha = sum(sapply(mns[-1], function(cf)  
    #  return(logN(cf/100))))  
    #L_pi = N * logg(N) + logg(N) 
    #L_model = L_alpha + L_pi + L_loglik + L_const

     L_alpha = precs* k
     L_pi =  N * logg(N) + logg(N)
     L_model = L_alpha + L_pi + L_loglik + L_const
 
    
    lmse_1 = C_history$lmse_1
    rmse_1 = C_history$rmse_1    
    ls_k = ( lmse_k - lmse_1)/(k-1) 
    rs_k = ( rmse_k - rmse_1)/(k-1) 
    
    #---- Inter group distancce -------
    #scale btw min, split into K parts and max, 2 parts
   interdist_scaled = function(mns, C_part, C_cf){
    sm = 0
    for(i in 1:(length(mns)-1)){
      # TODO actual elems
      sm = sm + abs(mns[i+1]-mns[i]) #best case
    }
     
    sm = sm/(length(mns)-1)
    return(sm)
    
    mx = abs(max(C_cf)-min(C_cf))
    mn  = abs(max(C_cf)-min(C_cf))/(length(C_cf)-1)
  
    if(sm>=mx) return (1)
    if(sm<=mn) return (0)
    return((sm-mn)/(mx-mn))
   
    }
    inter_dist <<- interdist_scaled(mns, C_part, C_cf)  
    
    
    #---- Entries -------
    #The score that the search alg is based on:
    if(version_mdl)
      scores_X_C[[xi]][ci] <<- L_model
    else 
      scores_X_C[[xi]][ci] <<- s_k
    
    #for reference:
    scoresB_X_C[[xi]][ci] <<- s_k 
    scoresE_X_C[[xi]][ci] <<- s_k_E
    
    
    lscores_X_C[[xi]][ci] <<- ls_k
    lscoresE_X_C[[xi]][ci] <<- ls_k_E
    
    
    rscores_X_C[[xi]][ci] <<- rs_k
    rscoresE_X_C[[xi]][ci] <<- rs_k_E
    

    
    dne = TRUE
    ssum = 0
    ssum_E = 0
    for(scores_X in scores_X_C)
      dne = dne & !(FALSE %in% sapply(scores_X, function(s) return(s<1000)))
    if(dne){
      for(scores_X in scores_X_C)
        ssum = ssum + sum(scores_X)
      score_sum <<- ssum
      
      for(scores_X in scoresB_X_C)    scoreB_sum <<- scoreB_sum + sum(scores_X)
      for(scores_X in scoresE_X_C)    scoreE_sum <<- scoreE_sum + sum(scores_X)
      for(scores_X in lscores_X_C)    lscore_sum <<- lscore_sum + sum(scores_X)
      for(scores_X in rscores_X_C)    rscore_sum <<- rscore_sum + sum(scores_X)
      for(scores_X in lscoresE_X_C)    lscoreE_sum <<- lscoreE_sum + sum(scores_X)
      for(scores_X in rscoresE_X_C)    rscoreE_sum <<- rscoreE_sum + sum(scores_X)
    }
     
    done <<- dne
  }
)
   

# Partition Utils ------


#Given C_part, return E_part (to search for in history_F)
reord_C_E_part = function(C_part, C_history){
  
  E_ord = C_history$E_ord
  E_part = C_part[E_ord] 
  E_part = sort_classes_lex(E_part)

  k = length(unique(C_part)) 
  assert (k == length(unique(E_part)))
  return(E_part)
}
reord_E_C_part = function(E_part, C_history){
  
  C_ord = C_history$C_ord
  C_part = E_part[C_ord] 
  C_part = sort_classes_lex(C_part)
  
  E_check = reord_C_E_part(C_part, C_history)
  sapply(1:length(E_check), function(i)
    assert(E_check[i]==E_part[i], paste(out_br(clld(E_check,d=0)), out_br(clld(E_part,d=0)))))
  k = length(unique(C_part)) 
  assert (k == length(unique(E_part)))
  return(C_part)
}
 

sort_classes_lex = function (part){
  
  cls_maxseen = 0
  for (i in 1:length(part)){
    cls = part[i]
    if(cls>cls_maxseen) {#next group; skip smaller(=alr seen) groups 
      #cls should be cls_maxseen+1
      cls_next = cls_maxseen+1
      
      if(cls!=cls_next){
        replace_indices = which(part==cls)
        conflict_classes = which(part==cls_next)  #swap: even if cls still to large, will be fixed later
        part[replace_indices] = 0#cls_next
        part[conflict_classes] = cls
        part[replace_indices] = cls_next
      }
      cls_maxseen  = cls_next
    }
  }
  assert(cls_maxseen==length(unique(part)))
  cls_cur = 0
  
  for (i in 1:length(part)){
    cls = part[i]
    if(cls> cls_cur){
      assert(cls==cls_cur+1)#no jumps
      cls_cur = cls_cur + 1
    }
  }
  
  return(part)
}
