# wrap 
resultVario = setRefClass("resultVario", 
                          fields = list(  
                            Data_E1_En = "list",  
                            runs_vario_pi = "list",
                            subsets_id = "list",  
                            subsets_nm = "list",  
                            covariates = "character",
                            target = "character",
                            invariant = "numeric",
                            invariant_locally = "character",
                            invariant_nms = "character",
                            invariant_sig = "numeric",
                            insignificant = "list",
                            Ranking = "list",
                            R1 = "list",
                            R2= "list"))

resultVario$methods(
  init = function(X_history){ 
    Data_E1_En <<- X_history$Data_E1_En
    subsets_nm <<- X_history$X_subset_nms
    subsets_id <<- X_history$X_subsets
    covariates <<- names(X_history$covariates_nm)
    target <<- X_history$target_nm
    Ranking <<- X_history$X_Pi_ranking
    R1 <<- X_history$Ranking
    R2<<- X_history$XS_Pi_ranking
    #Runs of Vario Pi on each subset:
    runs_vario_pi <<- X_history$F_histories
    invariant <<- X_history$X_invariant
    invariant_nms <<- covariates[X_history$X_invariant]
    invariant_locally <<- covariates[X_history$X_invariant_locally]
    invariant_sig <<- X_history$X_invariant_i
    insignificant <<- X_history$X_insig
  }
                          
 
                                                 
)
 
cls_subset_search = setRefClass("cls_subset_search",
                        fields = list( 
                          Data_E1_En = "list", #samples E_i=(X1..Xm,Y)
                          #
                          X_n = "numeric",
                          X_max_n = "numeric",
                          X_ids = "numeric", #index 1:X_n -> feature name/id
                          X_subsets = "list", #subset id (order of seen)-> array of Xids in set
                          X_subsets_regr="list", #subset id(order of seen)->regr info, here SSE for individual contexts
                          X_seen = "list", #subsets seen, sanity
                          X_subset_nms = "list",  
                          F_histories="list", #store  for all subsets XS for debug    
                          #
                          # Invariance and Insignificance
                          X_invariant="numeric",
                          X_invariant_locally="numeric",
                          X_invariant_i="numeric",
                          X_invariant_locally_cands="list", 
                          X_invariant_cands="list", # any subset leading to invariant coefficients (similar  to subset rankings)
                          X_invariant_significant_cands="list", # any subset leading to invariant coefficients, prune those x that are insignificant
                          X_insig="list",
                          #
                          #Subset rankings (1)
                          XS_Pi_ranking="list",
                          XS_Pione_ranking="list", # TODO rename
                          XS_Pionei_ranking="list", # corrected f significance
                          #
                          # Variable rankings (2) (average score over subsets they appear in; soft version of "averaging"(intersecting) the appearance in sets )
                          X_Pi_ranking="list",
                          X_Pi_ranking_i="list",
                          #
                          # Final ranking (3)
                          Ranking="list", 
                          Ranking_final="list",  
                          # Ranking versions: inconf/insignif
                          Ranking_inconf_pre="list", # score confidence percoefficient per Xi
                          Ranking_i ="list", # score confidence sum vs. total score sum (per Xi)
                          #
                          debug_Dsubs = "list",
                          config="list", #input args for search e.g. precomputed
                          #
                          # Subset Ranking versions (only subsets, no soft intersection)
                          XS_Pi_ranking_raw="list",#not including any correction for one group or confidence (kmax accurate, k=1 never guessed)
                          XS_Pi_ranking_nogroup="list",#confidence based on confint f pi kmax. problem: should rather generate samples randomly (as we do for k=1) than equidist (as for kmax)
                          XS_Pi_ranking_nscaled="list", 
                          #Prediction  (avg. lm of majority group)
                          pred_lm="lm",
                          pred_cfs="numeric",
                          pred_data="list",
                          # Variables
                          target_nm="character",
                          covariates_nm="list"
                          )) 


cls_subset_search$methods(  
  init = function(X_n, X_max_n, Data_E1_En, config, target_nm, covariates_nm){
    target_nm <<- target_nm
    covariates_nm <<- covariates_nm
    X_n <<- X_n
    X_max_n <<- X_max_n
    X_ids <<- 1:X_n
    Data_E1_En <<- Data_E1_En
    config <<- config  
    
    
    X_invariant_cands <<- lapply(1:X_max_n, function(xi) 
      return(list()))
    X_invariant_significant_cands <<- lapply(1:X_max_n, function(xi) 
      return(list()))
    X_invariant_locally_cands <<- lapply(1:X_max_n, function(xi) 
      return(list()))
  },
  
  edit = function(X_subset_id, 
                  X_subset_name, it){
    
    # Subset X1, ... Xm -----
    xnm =  paste0("{", paste0(paste(X_subset_name,collapse=", "), "}"))
   # if(config$verbose) 
     #### cat(paste("    ", paste0(it, "."), xnm), "\n") 
     
    xid = length(X_subsets)+1
    X_subsets[[xid]] <<- X_subset_id 
    X_subset_nms[[xid]] <<-X_subset_name
    
    D_sub = subset_collection(Data_E1_En, 
                              X_subset_id)
    names(D_sub) = names(Data_E1_En) 
    D_sub = rename_alphabetically(D_sub) # to have E1: x1, x2,...y even for Xsubset=(x3, x5)
    
    debug_Dsubs[[xnm]] <<- D_sub
  
    
    # Run Vario (pi search) ------
    deg = 1
    X_degrees = lapply(1:length(X_subset_id), function(xi) return(deg))
    Fhist = Vario_Pi_search(D_sub, X_degrees, config)
    
    #TODO
    #L_loglik_D1_Dn = c()
    # regr_D1_Dn = Fhist$regress_E1_En
    #for (i in 1:length(regr_D1_Dn)){
      #     rsq =regr_D1_Dn[[i]]$rsq
      #     dN = nrow(Fhist$Data_E1_En[[i]])
      #     sig = rsq / (dN-1)
      #     L_loglik = -(-(dN/2)*logg(2*pi*sig^2) - 1/(2*sig^2) * rsq) 
      ## cat("\n", dN, rsq, sig, L_loglik)
    #L_loglik_D1_Dn[[i]] = L_loglik
      # }
   
    #Add entries ----
    F_histories[[xnm]] <<- Fhist #at index XS_n also
    #X_subset_regr[[xnm]] <<- L_loglik_D1_Dn
    XS_n = length(X_subset_id)
    
    # Invariance & Insignificance ------
    # Add ANY potentially invariant variables in pairs of environments, icp style
    #if(Fhist$invariant_locally)  
    #  X_invariant_locally_cands[[XS_n]][[length(X_invariant_locally_cands[[XS_n]])+1]] <<- X_subset_id
    #if(Fhist$invariant_conservative)  {
    #  X_invariant_cands[[XS_n]][[length(X_invariant_cands[[XS_n]])+1]] <<- X_subset_id
 
      
      
      #Significance in addition
      # X_subset_significant_id = c()
      # for(xi in X_subset_id){
      #   id_in_subset=which(X_subset_id==xi)
      #   
      #   mt = Fhist$pval[[id_in_subset]]
      #   
      #   insig =  !(FALSE %in% sapply(1:nrow(mt) , function (rw) 
      #     return (!FALSE %in% sapply(mt[rw,], function(pval_in_Ei) return(pval_in_Ei  > 0.05)))))  #TODO other levels.
      #   
      #   if(!insig) X_subset_significant_id = c(X_subset_significant_id, xi) 
      # }
      # X_invariant_significant_cands[[XS_n]][[length(X_invariant_significant_cands[[XS_n]])+1]] <<- X_subset_significant_id
      # 
    #}
    
    Xn = length(Fhist$Cf_X_E1_En)
    score_sum = Fhist$Pi_ranking[[1]]$score_sum / Xn
    pi = Fhist$Pi_ranking[[1]]$E_part
    XS_Pi_ranking[[xnm]] <<-list(xid=xid, 
                                 pi=pi, 
                                 k=length(unique(pi)), 
                                 score_sum=score_sum, 
                                 xsid=X_subsets[[xid]], 
                                 xnm=X_subset_nms[[xid]],
                                 inv=Fhist$htest_invariant)
    
    
    # Other versions - remove
    XS_Pi_ranking_raw[[xnm]]<<- list(xid=xid, xsid =  X_subsets[[xid]], xnm =  X_subset_nms[[xid]],  inv=Fhist$htest_invariant, score_sum= Fhist$Pi_ranking_raw[[1]]$score_sum/Xn, 
                                     pi = Fhist$Pi_ranking_raw[[1]]$E_part, k= length(unique( Fhist$Pi_ranking_raw[[1]]$E_part)))
    XS_Pi_ranking_nogroup[[xnm]] <<- list(xid=xid, xsid =  X_subsets[[xid]], xnm =  X_subset_nms[[xid]],   inv=Fhist$htest_invariant, score_sum= Fhist$Pi_ranking_nogroup[[1]]$score_sum/Xn, pi = Fhist$Pi_ranking_nogroup[[1]]$E_part, k= length(unique( Fhist$Pi_ranking_nogroup[[1]]$E_part)))
    XS_Pi_ranking_nscaled[[xnm]] <<- list(xid=xid, xsid =  X_subsets[[xid]], xnm =  X_subset_nms[[xid]],   inv=Fhist$htest_invariant, scoreE_sum= Fhist$Pi_ranking_nscaled[[1]]$scoreE_sum/Xn, pi = Fhist$Pi_ranking_nscaled[[1]]$E_part, k= length(unique( Fhist$Pi_ranking_nscaled[[1]]$E_part)))
 #   }
  },
  
  finish = function(){
    #assert(length(F_histories)=..)
    # Invariant Variables in any subset
    X_inv_set=c()
    X_inv_set_loc=c()
    X_inv_set_i=c() # the same omitting insig
    
    for(xi in 1:X_n){
    keep = !(FALSE %in% sapply(1:length(X_invariant_cands), function(k) {
      sets_k = X_invariant_cands[[k]]
      return(TRUE %in% sapply(sets_k, function (set) return(xi %in% set) )) #xi is only in set if invariant in regression over this set
    }))
    if(keep) X_inv_set = c(X_inv_set,xi)
    keep_loc = !(FALSE %in% sapply(1:length(X_invariant_locally_cands), function(k) {
      sets_k = X_invariant_locally_cands[[k]]
      return(TRUE %in% sapply(sets_k, function (set) return(xi %in% set) )) #xi is only in set if invariant in regression over this set
    }))
    if(keep) X_inv_set_loc = c(X_inv_set_loc,xi)
    
    keep_sig = !(FALSE %in% sapply(1:length(X_invariant_significant_cands), function(k) {
      sets_k = X_invariant_significant_cands[[k]]
      return(TRUE %in% sapply(sets_k, function (set) return(xi %in% set) )) #xi is only in set if invariant in regression AND not insig in all E
    }))
    if(keep_sig) X_inv_set_i = c(X_inv_set_i,xi)
    
    }
    if(length(X_inv_set)) X_invariant <<- X_inv_set
    if(length(X_inv_set_loc)) X_invariant_locally <<- X_inv_set_loc
    if(length(X_inv_set_i)) X_invariant_i <<- X_inv_set_i
    
    #Filter out non invariant variables 
      for (i in 1:length(XS_Pi_ranking)){ 
        X_subset = XS_Pi_ranking[[i]] 
        invariant = !(FALSE %in% sapply(X_subset$xsid, function (xi) return(xi %in% X_invariant)))
        invariant_i = !(FALSE %in% sapply(X_subset$xsid, function (xi) return(xi %in% X_invariant_i)))
       
       
        t2 =  length(intersect(X_subsets[[X_subset$xid]], X_invariant))==length(X_subsets[[X_subset$xid]])
          assert(invariant==t2)
          XS_Pi_ranking[[i]]$invariant<<-invariant 
          XS_Pi_ranking[[i]]$invariant_i<<-invariant_i
      }
    
    XS_Pione_ranking <<- list.filter(XS_Pi_ranking, invariant)
    XS_Pionei_ranking <<- list.filter(XS_Pi_ranking, invariant_i)
     
     
     
    XS_Pi_ranking <<- list.sort(as.list(XS_Pi_ranking), score_sum) 
    XS_Pione_ranking <<- list.sort(as.list(XS_Pione_ranking), score_sum) 
    XS_Pi_ranking_raw <<- list.sort(as.list(XS_Pi_ranking_raw), score_sum) 
    XS_Pi_ranking_nscaled <<- list.sort(as.list(XS_Pi_ranking_nscaled), scoreE_sum)
    XS_Pi_ranking_nogroup <<- list.sort(as.list(XS_Pi_ranking_nogroup), score_sum) 
    
   # XS_Pi_ranking_correctone <<- list.sort(as.list(XS_Pi_ranking_correctone), score_sum) 
    
    
    post()
    combine()
    
    if(!config$version_mdl) return()
    
   # cmp_sse()
  }, 
 post = function() {
    # create variable ranking
    # Good for us: all partitions relevant for X1..Xn (and smaller subsets) alr exist in the resp. Fhist$Pi_rankings.
   
   X_Pi_ranking <<- list()
   X_Pi_ranking_i <<- list() # corrected for regression insig.
   
    for(Xi in 1:X_n){
      # intersect subsets for Pi -> each Pi that is in all subsets with X is a cand. for score sum (simply go through all Pi in {Xi} ranking. or smallest later??)
      xid = Xi  # TODO relies on search order, smallest first. TODO cleaner

      xseen = 0 # subsets incl x
      xsid = X_subsets[[xid]] 
      xnm = X_subset_nms[[xid]] 
 
      Fhist = F_histories[[xsid]]
      Pi_cands = Fhist$Pi_ranking
      
      
    ###  cat("Checking ",xid, ". ",xnm, "\n", sep="")
      
      # Partitons Pi for variable
      for(Pi_cand in Pi_cands){
        cand = TRUE
        cand_insig = TRUE
        score_sum = 0
        str = ""
        E_part = Pi_cand$E_part
        partnm = paste(E_part, collapse="") 
        
        # Subsets including the variable - check whether they all include Pi
        for(XS_id in 1:length(X_subsets)){
          XS_ids = X_subsets[[XS_id]] 
          if(xid %in% XS_ids){ # includes the singleton set 
            xseen = xseen+1 
            Pi_cands_2 = ls(F_histories[[XS_id]]$Pi_ranking)
            if(!(partnm %in% Pi_cands_2)) {
              
              cand=FALSE
              cand_insig=FALSE
            }
             
            else{
              id_in_subset = which(XS_ids==xid)
              # Can still prune the cadidate for ranking_i if in all environments, all coefficients (for the variable) insig.
              mt = F_histories[[XS_id]]$pval[[id_in_subset]]
              
                insig =  !(FALSE %in% sapply(1:nrow(mt) , function (rw) 
                        return (!FALSE %in% sapply(mt[rw,], function(pval_in_Ei) return(pval_in_Ei  > 0.05)))))  #TODO other levels.
              if(insig) cand_insig=FALSE
                
                #record the scores f debug --discarded in case cand FALSE at some point.
              score = F_histories[[XS_id]]$Pi_ranking[[partnm]]$score_sum / length(XS_ids)
              scores_xi = F_histories[[XS_id]]$Pi_histories[[partnm]]$scores_X_C[[id_in_subset]] #todo only one cf

              score_xi = sum(scores_xi)
              
              if(length(unique(E_part))==1) { # The artificial one-group partition stores the inconf. scores. 
               score_xi =  score
              }
              #else 
               # assert(score_xi==score)
              score_sum = score_sum + score_xi
              str = paste0(str, paste0(", { x", paste0(paste0(XS_ids, collapse = ", x"), paste0(" } ", clld(score_xi, d=2)))))
             # cat(" +", score)
            }
          } 
          if(!cand) next
        }
        
        score_norm = score_sum / subset_no(X_max_n, length(covariates_nm))
        
        if(cand)  X_Pi_ranking[[paste(partnm, xnm, sep="_")]] <<- list(xid=xid , xnm=xnm, score_sum = score_sum, score_norm = score_norm,
                                                                         E_part=E_part, record=str)
        #cat("test")
        if(cand_insig)  X_Pi_ranking_i[[paste(partnm, xnm, sep="_")]] <<- list(xid=xid , xnm=xnm, score_sum = score_sum, score_norm = score_norm,
                                                                                 E_part=E_part, record=str)
      }
      
      X_seen[[xid]]<<-xseen
    }
   X_Pi_ranking <<- list.sort(as.list(X_Pi_ranking), score_sum) 
   X_Pi_ranking_i <<- list.sort(as.list(X_Pi_ranking_i), score_sum) 
    xseen = X_seen[[1]]
   ### ?x= sapply(X_seen, function(s) assert(s==xseen))
  },
  
 
# TODO sparse combine (or alr post!): if for (xi in 1:length(cellxh3$F_histories)) if  (TRUE %in% cellxh3$F_histories[[xi]]$Rejects_X_E1_En[[1]]) print(xi)
# or the same f insig coefs, disconsider from soft intersection and pi - intersection.
 # Combine features by partition they share. TODO pi seen along the way.
 combine = function(){
   
  
   
   
   
   for(pii in 1:length(X_Pi_ranking)){
     pi = X_Pi_ranking[[pii]]
     partnm = paste(pi$E_part, collapse="")
     k = length(unique(pi$E_part)) 
     
     has_score = (length(unique(pi$E_part))>1)
   if(has_score) 
     thresh = config$conf_precomputed$conf_nogroups[1,k-1]
     else thresh = pi$score_norm
     
     score = pi$score_norm 
      
      if(partnm %in%  ls(Ranking)){
        # print("ok") 
         pi2 = Ranking[[partnm]]
       #  Ranking[[partnm]]$thresh <<- c(pi2$thresh, thresh_X) 
         Ranking[[partnm]]$scores <<- c(pi2$scores,score) # normalize by size later
       Ranking[[partnm]]$xsid  <<-c(pi2$xsid,  pi$xid)
       Ranking[[partnm]]$xnm  <<- c(pi2$xnm,  pi$xnm)
       Ranking[[partnm]]$E_part  <<- pi$E_part
     }
     else    
       
       Ranking[[partnm]] <<- list(xnm = pi$xnm, xsid = pi$xid, scores = score, E_part= pi$E_part)#, thresh = thresh_X)
     
 
          if(score<thresh) {
        if(partnm %in%  ls(Ranking_i)) {
           Ranking_i[[partnm]]$scores <<- c(pi2$scores, score) # normalize by size later
           Ranking_i[[partnm]]$xsid  <<-c(pi2$xsid,  pi$xid)
           Ranking_i[[partnm]]$xnm  <<- c(pi2$xnm,  pi$xnm) 
           Ranking_i[[partnm]]$E_part  <<- pi$E_part

        }
        else 
          Ranking_i[[partnm]] <<- list(xnm = pi$xnm, xsid = pi$xid, scores = score, E_part= pi$E_part)
        }

   }
   if(length(Ranking_i))
      for(pii in 1:length(Ranking_i)){
     pi = Ranking_i[[pii]]
     mems = length(pi$scores)
     Ranking_i[[pii]]$score_norm <<- sum(pi$scores) / mems 
     Ranking_i[[pii]]$score_sum <<- sum(pi$scores) 
   }
   
   for(pii in 1:length(Ranking)) {
     Ranking[[pii]]$score_norm <<- sum(Ranking[[pii]]$scores) / length(Ranking[[pii]]$scores)
     Ranking[[pii]]$score_sum <<- sum(Ranking[[pii]]$scores) 
     Ranking[[pii]]$len <<- length(Ranking[[pii]]$scores) 
     Ranking[[pii]]$k <<- length(unique(Ranking[[pii]]$E_part)) 
     
   }
   
   Ranking <<- list.sort(as.list(Ranking), score_norm) 
   
   # Special case no groups: allow at place 1 (means no Pi-invariance), ow. remove or it will dominate by likelihood
   Ranking_final[[1]] <<- Ranking[[1]]
   for(i in 2:length(Ranking)){
     k = length(unique(Ranking[[i]]$E_part))
     N = length(Ranking[[i]]$E_part)
     if(!(k==N | k==1))
      Ranking_final[[length(Ranking_final)+1]] <<- Ranking[[i]]
   }
     
     
   #  finish_predict()
 } ,
cmp_sse = function(){
  
   if(!config$version_mdl) return()

  xsid=0#length(X_subsets)
  for(k in 1:length(Ranking_final)){
    xnm = Ranking_final[[k]]$xsid
    for(j in 1:length( X_subsets)) {
    if(!(FALSE %in% sapply(xnm, function(nm)( nm %in% X_subsets[[j]])))
       & !(FALSE %in% sapply(X_subsets[[j]], function(nm) (nm %in% xnm))))
      xsid = j
    }  
    
   cat("\n", xsid, xnm)
    
    # Find Likelihood terms from regression result of subset S
    regr = F_histories[[xsid]]$regress_E1_En
    dta =  F_histories[[xsid]]$Data_E1_En
    L_loglik = 0
    
    for (i in 1:length(regr)){
      rsq =regr[[i]]$rsq
      dN = nrow(dta[[i]])
      sig = rsq / (dN-1)
      L_loglik_c = -(-(dN/2)*logg(2*pi*sig^2) - 1/(2*sig^2) * rsq) 
      #if(vb) cat("\n\t", dN, rsq, sig, L_loglik_c)
      L_loglik = L_loglik + L_loglik_c
    }
    Ranking_final[[k]]$L_loglik <<- L_loglik
  #  if(!config$version_mdl) 
    Ranking_final[[k]]$score_norm <<- Ranking_final[[k]]$score_norm + L_loglik
    
    
  }
  
    Ranking_final <<-  list.sort(as.list(Ranking_final), score_norm)
},

#TODO
finish_predict= function(){
  
  best_pi = Ranking_final[[1]]$E_part
  best_pa = Ranking_final[[1]]$xids
  
  maj_group =as.numeric(names(table(best_pi))[1])
  
  maj_members = which(best_pi==maj_group)
  maj_data = Data_E1_En[maj_members] 
  names(maj_data) = paste("D", 1:length(maj_data), sep="")
  maj_data = subset_collection(maj_data, best_pa)
  for(di in 1:length(maj_data)) colnames(maj_data[[di]]) = 
    c(paste("x", 1:(length(maj_data[[di]])-1), sep=""), "y")
  
  
  maj_regression_local = regression_E1_En(maj_data,  
                                          rep(1,length(maj_data[[1]])-1),
                                          skip_invariance=TRUE) 
  
  maj_regression_avg = maj_regression_local$regress_E1_En[[1]]$m
  
  avg_cfs = sapply(maj_regression_local$cf_mats$cfs, function(cfs_xi) 
    return(mean(cfs_xi)))
  maj_regression_avg$coefficients = avg_cfs
  
  pred_lm <<- maj_regression_avg
  pred_cfs <<- avg_cfs
  pred_data <<- maj_data
}
 )
