
# Flow cytometry Dataset ----------

load_Sachs_data = function(){
  D1 = read_excel("~/Vario/data_sachs/1. cd3cd28.xls")
  D2 = read_excel("~/Vario/data_sachs/2. cd3cd28icam2.xls")
  D3 = read_excel("~/Vario/data_sachs/3. cd3cd28+aktinhib.xls")
  D4 = read_excel("~/Vario/data_sachs/4. cd3cd28+g0076.xls")
  D5 = read_excel("~/Vario/data_sachs/5. cd3cd28+psitect.xls")
  D6 = read_excel("~/Vario/data_sachs/6. cd3cd28+u0126.xls")
  D7 = read_excel("~/Vario/data_sachs/7. cd3cd28+ly.xls")
  D8 = read_excel("~/Vario/data_sachs/8. pma.xls")
  D9 = read_excel("~/Vario/data_sachs/9. b2camp.xls")
  
  D1_Dn = list(D1, D2, D3, D4, D5, D6, D7, D8) 
  #D9 not used in Related Works.
  
  names (D1_Dn) = sapply(1:length(D1_Dn), function(i) 
    return(paste0("E", i)))
  return(D1_Dn) 
}



target_Vario_Sachs = function(target=6,#target Y, #6 is Erk
                              X_max_n=4,#max. subsets considered, usually always 4
                              vb = TRUE,
                              prune_contexts = NULL,#optional: use a subset of the eight contexts
                              version_mdl = TRUE,
                              version_simp=TRUE,
                              greedy_k=FALSE){ 
  

   Dcell = load_Sachs_data()
   x_named = names(Dcell[[1]])
   
   #change to capital letters to match other environments:
   names(Dcell$E8)[4]="PIP2"
   names(Dcell$E8)[5]="PIP3"
   
   # Remove environment datasets if needed
   if(length(prune_contexts))
     Dcell = Dcell[-prune_contexts]
   names(Dcell)= sapply(1:length(Dcell), function(i) 
     return(paste0("E", i))) #stupid naming convention
   
  # All variables but Y
  xs = (1:length(Dcell[[1]]))[-target]
  D = transform_xls_data(Dcell, xs, c(target))
  D1_Dn = D$D_data
  x_nms = D$X_names[which(D$X_names != "y")] #rmv y name
  
  E_n = length(D1_Dn)
  X_n = length(x_nms)

  # Run Vario
  RESULT = Vario_subset_search(D1_Dn, x_nms, #names(x_nms), 
                               X_max_n=X_max_n,
                               vb=vb, version_mdl=version_mdl,version_simp=version_simp,
                               greedy_k=greedy_k)
  
   
  return(list(RESULT=RESULT,
              #for reference:
              y_target=target,
              x_named=x_named,
              x_renamed=x_nms)) # to reproduce order of variables after y was removed
  
}

# other: used? --------


score_exp_k =function(cfs){
  test = init_history_C(1,1, as_mat(cfs),as_mat(cfs),as_mat(cfs ))
  
  pitest = new("cls_Pi")
  pitest$init(cfs, 1,c(1), FALSE, FALSE)
  pitest$edit(test, cfs)
  return(pitest$score_sum)
}

scale_score = function(score, best_score_sum, inconf_score_sum){
  a = best_score_sum
  b = inconf_score_sum
  if(score>b) return(0)
  if(score<=a) return(1)
  score = -score
  a = -a
  b = -b
  
  score= score -b
  a = a-b
  return(score/(a))
}
 

print_inv  = function(cell_res){
  for(n in cell_res$X_invariant_cands) {
        cat(paste0(paste0("|X", i),"| \n"))
     i=i+1
        for(id in n){
         cat("     { ")
         for (i in id)
       cat(cell_res$X_subset_nms[[i]], " ")
            
        cat("} \n")
     }
     
    }
  
}
