# Preprocessing: Extract edges, if needed Sort edges, before ancestral edge removal

edgeanalysis_sort = function(edges_per_Y, 
                             edges_per_node,
                             xi_names,
                             sort_by_gain=TRUE,
                             consider_nogroup=TRUE,
                             vb =FALSE){
  
  ##edges_per_node = lapply(results_per_Y, function(res) return(res$Ranking))
  
  #Extract full adjacency matrix -- all conf. edges found in subset search
  #adj = matrix(0, 11, 11)
  adj_bin = matrix(0, 11, 11)
  #anc_adj = matrix(0, 11, 11)
  edges_ordered = list()
  
  for (node_i in 1:11){ 
    nm_i = xi_names[node_i] 
    #result_i = as.list(edges_per_Y[[node_i]]$X_Pi_ranking)
    result_i = edges_per_node[[node_i]]
    
    for (edge_i in 1:length(result_i)){
      edge  = result_i[[edge_i]]
      
      pi = edge$E_part
      #score = edge$score_norm 
      
      #k = length(unique(pi)) 
      #conf = mean(confints_8$conf_pione[,(k-1)])
      if(consider_nogroup){
        if(length(unique(pi))==1) next  
        if(length(unique(pi))==length(pi)) next 
      }
      else {
        if(length(unique(pi))==1)  break 
        if(length(unique(pi))==length(pi))  break
        if(length(unique(pi))>2) next
      }
      #if(consider_conf & (score>conf)) next
      
      #if(match_partitions & !(length(unique(pi))==2)) next
      nm_j = edge$xnm
      node_j = which(xi_names==nm_j)
      #node_to_ingoing[[nm_i]][[nm_j]] =  edge
      
      edge$nm_i = nm_i
      edge$nm_j = nm_j
      edge$node_i = node_i
      edge$node_j = node_j
      edge$pruned = FALSE
      nm = paste0(names(result_i)[edge_i], "_", nm_i) 
      edges_ordered[[nm]] = edge
      
      #score = edge$score_norm 
      
      xi_nm = edge$xnm
      xi = which(xi_names==xi_nm)  
      if(vb)  cat("\n\tEdge: ", xi_names[xi]," ", xi, paste0(pi, collapse=""))
      #if(adj[xi, node_i]==0) #do not overwrite edges with lower scores 
      #  adj[xi, node_i]= score
      adj_bin[xi, node_i] = 1 
    }
  }
  edges_ordered = list.sort(edges_ordered, score_norm)
  
  
  for(i in 1:length(edges_ordered)){
    res = edgeanalysis_MDL_gain(xi_names, edges_ordered, edges_per_Y, 
                         edges_ordered[[i]], edges_ordered[[i]]$node_j, edges_ordered[[i]]$node_i)
    # cat("\n",xi_names[edges_ordered[[i]]$node_i],"->",xi_names[edges_ordered[[i]]$node_i], "gain", gain)
    
    #Gain of using J as a parent in addition to another (the best existing, irresp. of J) subset
    edges_ordered[[i]]$gain = res$gain
    #edges_ordered[[i]]$cost_S_i = res$cost_S_i
    #edges_ordered[[i]]$cost_Sj_i = res$cost_Sj_i
  }
  if(!sort_by_gain) 
    return(list(edges_ordered=edges_ordered, adj_bin=adj_bin))
  
  edges_ordered = list.sort(edges_ordered, (gain))
  
  return(list(edges_ordered=edges_ordered, adj_bin=adj_bin))
}



edgeanalysis_prune = function(edges_per_Y,
                              edges_ordered, adj_bin, 
                              xi_names, 
                              consider_confounded=TRUE,
                              match_partitions=TRUE,
                              consider_pruned=TRUE,
                              consider_gain=FALSE,
                              consider_3anc=FALSE, vb=FALSE){
  
  keepers = list() 
  discard = list()
  
  
  #pruned_adj = matrix(0, 11, 11)
  anc_adj = matrix(0, 11, 11)
  
  #Analyse edges
  adj_aug = consensus_augmented_graph_Sachs(xi_names)
  adj_small = consensus_small_graph_Sachs(xi_names)
 
  i=0
  for(adj_true in list(adj_small, adj_aug)){
    i=i+1
    cat("\t")
    
    diff = adj_bin - adj_true
    sm = adj_bin + adj_true 
    FP_per_Y = apply(diff, 2, function(i) return(which(i==1)))
    FN_per_Y = apply(diff, 2, function(i) return(which(i==-1)))
    TP_per_Y = apply(sm, 2, function(i) return(which(i==2)))
    
    FP_named = apply(diff, 2, function(i) return(xi_names[which(i==1)]))
    FN_named = apply(diff, 2, function(i) return(xi_names[which(i==-1)]))
    TP_named = apply(sm, 2, function(i) return(xi_names[which(i==2)]))
    if(length(TP_named)) tp = sum(sapply(TP_named, function(x) return(length(x))))
    else tp = 0
    if(length(FP_named)) fp = sum(sapply(FP_named, function(x) return(length(x))))#- sum(apply(adj_ancestral_noncausal,1:2, sum))
    else fp = 0
    if(length(FN_named)) fn = sum(sapply(FN_named, function(x) return(length(x))))
    else fn = 0
    #a = sum(apply(adj_ancestral_noncausal,1:2, sum))
    f1 = tp/(tp + (1/2)* (fp + fn))
    #f1 = tp/(tp + (1/2)* ((fp+a) + fn))
    if(i==1) sm = "sm"
    else sm = "aug"
    cat("\n\tPre",sm,":", tp, fp, paste0("(", fp, "+ ",")"), paste0("(",fn,")"),  paste0( "= --", ", *", fmtd(f1,d=2)))
    
  }  
  cat("\n")
  #dont look up edges here while pruning, unless for the node itself
  #adj_pruned = adj 
  
  #Find pruned adjacency columns -- for each Yi, consider adj[,-Yi] as given, and for each adj[xj,yi]>0, prune if exists z, adj[xj,z]+adj[z,yi]/2 >adj[xj,yi] 
  # As z cannot be among
  if(vb)  cat("\n---------------------\nAncestral edges\n---------------------")
  FP_but_ANC = 0
  for (e in 1:length(edges_ordered)){
    edge_ji = edges_ordered[[e]]
    node_i = edge_ji$node_i
    node_j = edge_ji$node_j
    #exists_ancestral = FALSE 
    #exists_only_ancestral =FALSE
    true_ancestors = ""
    if(vb)  cat("\nChecking ", xi_names[node_j], "->", xi_names[node_i])
    edge_ji$exists_ancestral_small= FALSE 
    edge_ji$exists_ancestral_aug= FALSE
    
    edge_ji$exists_only_ancestral_small= FALSE 
    edge_ji$exists_only_ancestral_aug= FALSE
    edge_ji$exists_only_3ancestral_small= FALSE 
    edge_ji$exists_only_3ancestral_aug= FALSE
    
    for (nk in 1:11){
      if(nk==node_j | nk==node_i) next
      if( (adj_small[node_j,nk]==1 & adj_small[nk,node_i]==1)) {
        if(adj_small[node_j,node_i]==0 & adj_small[node_i,node_j]==0 ){
          edge_ji$exists_only_ancestral_small =TRUE
          anc_adj[node_i,node_j]= 1
        }
        true_ancestors = paste0(true_ancestors, xi_names[nk], ",")
        edge_ji$exists_ancestral_small= TRUE
        
        
      }
      if( (adj_aug[node_j,nk]==1 & adj_aug[nk,node_i]==1)) {
        if(adj_aug[node_j,node_i]==0 & adj_aug[node_i,node_j]==0){
          edge_ji$exists_only_ancestral_aug =TRUE
          anc_adj[node_i,node_j]= 1
        } 
        edge_ji$exists_ancestral_aug= TRUE
        
      }
      for (nr in 1:11){
        if(nr==node_j | nr==nk | nr==node_i) next
        if( (adj_big[node_j,nr]==1 &adj_big[nr,nk]==1 & adj_big[nk,node_i]==1) &
            adj_big[node_j,node_i]==0 & adj_big[node_i,node_j]==0 ) 
          edge_ji$exists_only_3ancestral_aug= TRUE
        if( (adj_small[node_j,nr]==1 &adj_small[nr,nk]==1 & adj_small[nk,node_i]==1) & 
            adj_small[node_j,node_i]==0 & adj_small[node_i,node_j]==0 ) 
          edge_ji$exists_only_3ancestral_small= TRUE
      }
      
      
    }
    edge_ji$causal_small = (adj_small[node_j,node_i]==1)
    edge_ji$causal_aug = (adj_big[node_j,node_i]==1)
    
    #exists_ancestral = exists_ancestral | (adj_true[node_j,node_k]==1 & adj_true[node_k,node_i]==1)
    edge_ji$noncausal_small = (adj_small[node_j,node_i]==0)
    edge_ji$exists_anti_small = (adj_small[node_i,node_j]==1)
    edge_ji$exists_anti_big = (adj_big[node_i,node_j]==1)
    
    
    # Go through Ancestral edges, J->K, J->K->I
    # If consider confounded: K->J, J<-K->I
    for (e2 in 1:length(edges_ordered)){ 
      edge_jk = edges_ordered[[e2]]
      node_jj = edge_jk$node_j
      node_k = edge_jk$node_i
      
      for(it in 1:2){
      if(it==2){ #confounded
        if(!consider_confounded) break
        node_jj = edge_jk$node_i
        node_k = edge_jk$node_j
      }
      
      if(node_jj != node_j | node_k == node_i  | node_k == node_j ) next
      
     if(consider_pruned) if(edge_jk$pruned | edge_ji$pruned) next
      
      #Find edge K->I
      for (e3 in 1:length(edges_ordered)){ 
        edge_ki = edges_ordered[[e3]]
        node_kk = edge_ki$node_j
        node_ii = edge_ki$node_i
        
        if(!(node_kk == node_k & node_ii == node_i)) next
        
        if(consider_pruned)  if(edge_jk$pruned | edge_ji$pruned | edge_ki$pruned) next
        
        if(vb)  cat("\n\tPath ", xi_names[node_j], "->", xi_names[node_k], "->", xi_names[node_i], 
                    ", Pis", paste0(edge_jk$E_part, collapse=""),"->" ,paste0(edge_ki$E_part, collapse=""),"->" , paste0(edge_ji$E_part, collapse=""))
        
        if(consider_gain){
        
        #Option1: don't prune, leave one or both of J->I, K->I:
        # find that common subset S, possibly containing K, that explains I best, and consider gain of adding J to it:
     
        
        res_ji = edgeanalysis_MDL_gain(xi_names, edges_ordered, edges_per_Y, 
                                       edge_ji, node_j = node_j, node_i = node_i)
        #Option2: prune path J->I:
        # find that common subset S, NOT containing J, that explains Y best, and consider gain of adding K to it:
        res_ki = edgeanalysis_MDL_gain(xi_names, edges_ordered, edges_per_Y, 
                                       edge_ki, node_j = node_k, node_i = node_i, node_not_allowed = node_j)
   
        
        c1 = res_ji$cost_S_i
        c2 = res_ki$cost_S_i
        
        c3 = res_ji$cost_Sj_i
        c4 = res_ki$cost_Sj_i
    #    cat("\nSet for J->I:", res_ji$set,", cost(S):", c1,", cost(S,j): ", c3,
     #       "\n Set for K->I:",  res_ki$set,", cost(S):", c2,", cost(S,k): ", c4)
 
        
        anc = c4 +((edge_jk$score_norm + edge_ki$score_norm)/2) #- res_ki$gain# 
        dir = c3 +edge_ji$score_norm #- res_ji$gain # 
        }
        else{
          anc = (edge_jk$score_norm + edge_ki$score_norm)/2
          dir =  edge_ji$score_norm
        }
          pi1 = edge_jk$E_part
          pi2 = edge_ki$E_part
          pi3 = edge_ji$E_part 
          
          
          pi12 = rep(1,length(pi1))
          cur = 1
          prev1 = 1
          prev2 = 1
          for(i in 1:length(pi2)){
            if(pi1[i]!=prev1 | pi2[i]!=prev2 ){
              cur = cur + 1
            } 
            pi12[i] = cur  
            prev1= pi1[i]
            prev2= pi2[i]
          }  
          consistent  = (check_finegrained(pi12,pi3)$finegrained) #| 
                          # check_finegrained(pi3,pi12)$finegrained) 
          
         # if(it==2) consistent = consistent & check_finegrained(pi2,pi1)$finegrained

        # if(i==2){
            # confounded: pi (K), i.e.k->J, pi1, must have a subset of interventions of k->I
         #   consistent = consistent & (check_finegrained(pi1,pi3)$finegrained | check_finegrained(pi3,pi1)$finegrained )
          #}
          
          
          
          
          if( (anc < dir)   #At least one ancestral path (via z) is found that is better (cheaper by score) than the direct one
              & (consistent | (!match_partitions)))  { 
            
            edge_ji$pruned = TRUE 
            if(vb)  cat("\n\t Removed:",    xi_names[node_j],  "->",xi_names[node_k] , "->", xi_names[node_i], "\t\t", anc,"<", dir, "\t CONS:", consistent)
            
           # pruned_adj[node_j,node_i] = anc-dir 
          }
          # #jrki 
          if(consider_3anc & (it!=2))  
            for (e4 in 1:length(edges_ordered))
              for (e5 in 1:length(edges_ordered)){
                edge_jr = edges_ordered[[e4]]
                edge_rk = edges_ordered[[e5]]
                
                node_jj = edge_jr$node_j
                node_r= edge_jr$node_i
                node_rr= edge_rk$node_j
                node_kk= edge_rk$node_i
                if(node_r == node_k | node_r == node_j | node_r==node_i | 
                   node_r != node_rr | node_k != node_kk | node_j != node_jj) next 
                
                # pi2 = edge_ki$E_part
                #  pi3 = edge_ji$E_part 
                pi0 = edge_jr$E_part 
                pi1 = edge_rk$E_part 
                
                pi012 = rep(1,length(pi1))
                cur = 1
                prev0 = 1
                prev1 = 1
                prev2 = 1
                for(i in 1:length(pi2)){
                  if(pi0[i]!=prev0 | pi1[i]!=prev1| pi2[i]!=prev2 ){
                    cur = cur + 1
                  } 
                  pi012[i] = cur  
                  prev0= pi0[i]
                  prev1= pi1[i]
                  prev2= pi2[i]
                }  
                

                consistent  = (check_finegrained(pi012,pi3)$finegrained ) 
                #| 
                 #                check_finegrained(pi3,pi012)$finegrained) 
                
                if(vb) cat(" \n\t | Path: ", xi_names[node_j], "->",  xi_names[node_r], "->", xi_names[node_k], "->", xi_names[node_i])
                 
                anc = (edge_jr$score_norm + edge_rk$score_norm + edge_ki$score_norm)/3
                dir = edge_ji$score_norm
                
                if(anc < dir  & (consistent | (!match_partitions)))  {   
                  consistent = "TODO"
                  edge_ji$pruned = TRUE 
                  if(vb)  cat("\n\t Removed:",    xi_names[node_j],  "->",xi_names[node_k] , "->", xi_names[node_i], "\t\t", anc,"<", dir, "\t CONS:", consistent)

              #    pruned_adj[node_j,node_i] = anc-dir
                }
              }
      }
    } 
  }
    #All edges J->K->I for node J are processed
    
    edge_ji$node_pa = node_j
    edge_ji$node_child = node_i 
    
    if(!edge_ji$pruned){ 
      if(vb) cat("\n\t Keeping:",  nm,   xi_names[node_j], "-> {",true_ancestors,
                 " } ->", xi_names[node_i]) 
    }
    
    if(vb) cat("\n---------------------\n")
    
    # if(consider_conf_afterwards) { 
    #   stop() #TODO
    #   pi = edge_ji$E_part
    #   k = length(unique(pi))
    #   conf = 0# 42.85396#mean(confints_8$conf_pione[,(k-1)])
    #   score= edge_ji$score_norm
    #   if(score>conf) edge_ji$pruned=  TRUE
    #   #     if(noncausal) fp = "++ noncausal\t"
    #   #     if(causal) fp = "-- causal\t" 
    #   #     if(vb)  cat("\n\t Removed:", fp, xi_names[node_j],  "->",xi_names[node_i], "by conf")
    #   #     pruned =TRUE
    #   #   }
    #   # }
    # } 
    edges_ordered[[e]] = edge_ji
    
    
  }
  for(e in edges_ordered){
    if(e$causal_aug | e$causal_small) nm = ifelse(e$pruned, "-c.", "+c.")
    else {
      if(e$exists_only_ancestral_small | e$exists_only_ancestral_aug) nm  =  ifelse(e$pruned,"+a.", "-a.")
      else{
        if(e$exists_only_3ancestral_small | e$exists_only_3ancestral_aug) nm  =  ifelse(e$pruned,"+ä.", "-ä.")
        else nm = ifelse(e$pruned,"+n.", "-n.")
      }
    } 
    nm = paste0(nm, xi_names[e$node_j], "->",xi_names[e$node_i])
   if(!e$pruned)  keepers[[nm]] = e
   else discard[[nm]] = e
  }
  EO=edgeanalysis_eval_edges(edges_ordered)
  return(list(edges_ordered=edges_ordered,
              EO=EO,
         keepers =keepers,
         discard = discard))
  
  
}

#edges_score_simp$EO
edgeanalysis_dectree  = function(edge_list, xi_names, edges_per_Y){
  
  adj = matrix(0,11,11)
  for(e in edge_list) 
    adj[e$node_j , e$node_i] = 1
  
  for (i in 1:length(edge_list)){
    e = edge_list[[i]]
    # pa = which(adj[,e$node_i]==1)
    
   # res  = edgeanalysis_MDL_gain_S(xi_names, edge_list, edges_per_Y, e$node_j, e$node_i, pa)
   res = edgeanalysis_MDL_gain(xi_names, edge_list, edges_per_Y, e, e$node_j, e$node_i)
    e$cost_S_i = res$cost_S_i
    e$cost_Sj_i = res$cost_Sj_i
    e$gain = res$gain
    #e$set = res$set
    edge_list[[i]] = e
  }
  edge_list = list.sort(edge_list, (gain))
  adj_true = consensus_augmented_graph_Sachs(xi_names)
  
  cat("\n---------------\nDECISION RATE\ni, x, acc, f1\n")
  for(cut in 1:length(edge_list)){
    pt = cut * 100/length(edge_list)
    
    consider = edge_list[1:cut]
    tp = 0
    fp = 0
    #if(cut==1) if(adj_true[consider[[1]]$node_j, consider[[1]]$node_i]==1) tp = tp + 1
    
    #else 
      for (i in 1:length(consider)){
           if(adj_true[consider[[i]]$node_j, consider[[i]]$node_i]==1) tp = tp + 1
           else fp = fp + 1
       }
    frac_tp = tp/cut
    fn = sum(true_adj)-tp
    f1 = tp/(tp + (1/2)* (fp + fn))
    
    cat("\n",i, pt,"\t", frac_tp, "\t\tF1:", f1,"\t\tTP:", tp, "FP:", fp )
  }
  cat("\n\n---------------\nGAIN PLOT\n")
  for(i in 1:length(edge_list)){
       cat("\n", i,  i*(100/length(edge_list)),  edge_list[[i]]$gain)
  }
  return (edge_list)
  
  
}

edgeanalysis_eval_edges = function(edges_ordered){
  
  
  found = matrix(0,11,11)
  #remove dups
  EO = list()
  for(e in edges_ordered){
    if(!e$pruned & found[e$node_j, e$node_i] == 0){ #keep the 1st edge that is found
      found[e$node_j, e$node_i] = 1
      
      i1 = "-n."
      if(e$causal_small) i1 = "+c."
      else{
        if(e$exists_only_ancestral_small) {
          if(e$exists_anti_small) i1 = "-x.a."
          i1 = "+a."
        }
        else if(e$exists_anti_small) i1 = "-x."
      }
      i2 = "-n"
      if(e$causal_aug) i2 = "+c"
      else{
        if(e$exists_only_ancestral_aug) {
          if(e$exists_anti_big) i2 = "-xa"
            else i2 = "+a"
        }
        else if(e$exists_anti_big) i2 = "-x"
      }
      nm = paste0(#i1,
                  i2, "_",
                  xi_names[e$node_j], "_" , paste0(e$E_part,collapse = ""), "_",
                  xi_names[e$node_i] )
      EO[[nm]] = e
    }
  }
  
  adj_small =consensus_small_graph_Sachs(xi_names)  #
  adj_big = consensus_augmented_graph_Sachs(xi_names) 
  
  adj_small_inv = -(adj_small-1)
  adj_aug_inv = -(adj_aug-1)
  #cat("\n\tSID sm: ", (suppressMessages(structIntervDist(adj_small, found)$sid/110)), #"/110", #structIntervDist(adj_small, adj_small_inv)$sid,
  #    "\n\tSID aug:", (suppressMessages(structIntervDist(adj_aug, found)$sid/110)), #"/110", #structIntervDist(adj_aug, adj_aug_inv)$sid,
  #    "\n")
  EO_small <<- EO
  adj_bin_pruned = matrix(0,11,11)
  
  vals_pruned = matrix(0,11,11)
  vals_causal = matrix(0,11,11)
  
  tp = matrix(0,11,11)
  fp = matrix(0,11,11)
  nc = matrix(0,11,11)
  fp_anc = matrix(0,11,11)
  
  
  for(e in EO) if(!e$pruned) adj_bin_pruned[e$node_j, e$node_i]=1

  
  for(e in EO) if((!e$pruned )& adj_small[e$node_j, e$node_i]==1) tp[e$node_j, e$node_i]=1
  for(e in EO) if((!e$pruned )& adj_small[e$node_i, e$node_j]==1 & adj_small[e$node_j, e$node_i]==0) nc[e$node_j, e$node_i]=1
  for(e in EO) if((!e$pruned)  &  adj_small[e$node_j, e$node_i]==0) fp[e$node_j, e$node_i]=1
  for(e in EO) if((!e$pruned)  &  adj_small[e$node_j, e$node_i]==0 &  adj_small[e$node_i, e$node_j]==0 & (e$exists_only_ancestral_small | e$exists_only_3ancestral_small)) fp_anc[e$node_j, e$node_i]=1
  
  tp = sum(tp)
  fp = sum(fp)
  fn = sum(adj_small) -tp
  nc = sum (nc)
  f1 = tp/(tp + (1/2)* (fp + fn))
  f1good = tp/(tp + (1/2)* ((fp-sum(fp_anc)) + fn))
  cat("\n\tF1 sm:",tp, fp,"(", sum(fp_anc), nc, ")", fn, "=",f1,"*" , f1good, "\n")
  
  tp = matrix(0,11,11)
  fp = matrix(0,11,11) 
  nc = matrix(0,11,11)
  fp_anc = matrix(0,11,11)
  
  for(e in EO) if(!e$pruned) adj_bin_pruned[e$node_j, e$node_i]=1
  
  for(e in EO) if((!e$pruned) & adj_big[e$node_i, e$node_j]==1 & adj_big[e$node_j, e$node_i]==0) nc[e$node_j, e$node_i]=1
  for(e in EO) if((!e$pruned) & adj_big[e$node_j, e$node_i]==1) tp[e$node_j, e$node_i]=1
  for(e in EO) if((!e$pruned)  &  adj_big[e$node_j, e$node_i]==0) fp[e$node_j, e$node_i]=1
  for(e in EO) if((!e$pruned)  &  adj_big[e$node_j, e$node_i]==0 & (e$exists_only_ancestral_aug | e$exists_only_3ancestral_aug)) 
    fp_anc[e$node_j, e$node_i]=1 
  
  
  tp = sum(tp)
  fp = sum(fp)
  nc = sum (nc)
  fn = sum(adj_big) -tp
  f1 = tp/(tp + (1/2)* (fp + fn))
  f1good = tp/(tp + (1/2)* ((fp-sum(fp_anc)) + fn))
  cat("\tF1 aug:",tp, fp, "(", sum(fp_anc),nc,  ")", fn, "=", f1,"*" , f1good, "\n")
  return(EO)
}

interv_analysis = function(edge_list, direct=FALSE, only_big_causal=FALSE, only_small_causal=FALSE){
  fp = 0
  tp = 0
  fn = 0
  
  
  adj_small =consensus_small_graph_Sachs(xi_names)  #
  adj_big = consensus_augmented_graph_Sachs(xi_names) 
  
  
  interv_loc = matrix(0,11,8)
  for (i in 1:length(edge_list)){
    edge = edge_list[[i]]
    node_i = edge$node_i
    node_j = edge$node_j
    if(only_big_causal & adj_big[node_j,node_i]==0) next
    if(only_small_causal & adj_small[node_j,node_i]==0) next
    
    E_part = edge$E_part
    
    interv_i = c()
    for (k in 1:length(E_part))
      if(E_part[k] != 1) {
        interv_i = c(interv_i, k) 
        interv_loc[node_i, k] = 1
      }
    
    edge$interv_i = interv_i
    edge$true_interv_i = union(consensus_direct_interventions_Y(node_i), consensus_indirect_interventions_Y(node_i))
    if(direct) edge$true_interv_i =  consensus_direct_interventions_Y(node_i) 
  
    edge$interv_tp = length(intersect( edge$interv_i , edge$true_interv_i ))
    edge$interv_fp =max(length( edge$interv_i)-length(intersect( edge$interv_i , edge$true_interv_i )),0)
    edge$interv_fn =max(length(edge$true_interv_i)-length(intersect( edge$interv_i , edge$true_interv_i )),0)

   edge_list[[i]] = edge
  }
  
  for(node_i in 1:11){
    for(k in 1:8) {
      tr =  (k %in% (union(consensus_direct_interventions_Y(node_i), consensus_indirect_interventions_Y(node_i))))
      if(direct) tr =  (k %in% consensus_direct_interventions_Y(node_i))
      pred_tr =  ( interv_loc[node_i,k]==1)
      if(tr){
        if(pred_tr) tp = tp +1
        else fn = fn + 1
      }
      else  if(pred_tr) fp = fp +1
    }
  } 
  f1 = tp/(tp + (1/2)* (fp + fn))
  cat("\n", tp, fp, fn, "f1", f1)
   
  return(edge_list)
  
}

interv_jci_direct = function() return(interv_show_competitors(TRUE, TRUE))
interv_jci_indirect = function() return(interv_show_competitors(TRUE, FALSE))
interv_em_direct = function() return(interv_show_competitors(FALSE, TRUE))
interv_em_indirect = function() return(interv_show_competitors(FALSE, FALSE))
interv_show_competitors = function( jci=TRUE , direct = FALSE){
  fp = 0
  tp = 0
  fn = 0
  for(node_i in 1:11){
    for(k in 1:8) {
      tr =  (k %in% (union(consensus_direct_interventions_Y(node_i), consensus_indirect_interventions_Y(node_i))))
      if(direct) tr =  (k %in% consensus_direct_interventions_Y(node_i))
      pred_tr =  (k %in% eaton_interventions_Y(node_i))
     if(jci)  pred_tr =  (k %in% jci_direct_interventions_Y(node_i))
      if(tr){
        if(pred_tr) tp = tp +1
        else fn = fn + 1
      }
      else  if(pred_tr) fp = fp +1
    }
  } 
  f1 = tp/(tp + (1/2)* (fp + fn))
  cat("\n", tp, fp, fn, "f1", f1)
}

# Help: Gain in likelihood -------

edgeanalysis_MDL_gain_S = function(xi_names, 
                                 edges_ordered, 
                                 Vario_results,
                                 #edge_ji, 
                                 node_j, node_i, nodes_S,
                                 vb=FALSE){
  
  
  
  cand_L = Inf
  cand_argmin = ""
  
  Vario_result_i = Vario_results[[node_i]] 
  #  Find MDLcost of admissible subsets S, S->I, S subseteq nodes_S
  # Choose the best subset S (irrespective of J)
  for(xsid in 1:length(Vario_result_i$X_subset_nms)){
    cands = Vario_result_i$X_subset_nms[[xsid]]
    if(length(intersect(xi_names[nodes_S], cands)) < 3 |
       length(cands)>3) next
      
     
    
    # Find Likelihood terms from regression result of subset S
    regr = Vario_result_i$F_histories[[xsid]]$regress_E1_En
    dta = Vario_result_i$F_histories[[xsid]]$Data_E1_En
    L_loglik = 0
    
    for (i in 1:length(regr)){
      rsq =regr[[i]]$rsq
      dN = nrow(dta[[i]])
      sig = rsq / (dN-1)
      L_loglik_c = -(-(dN/2)*logg(2*pi*sig^2) - 1/(2*sig^2) * rsq) 
      #if(vb) cat("\n\t", dN, rsq, sig, L_loglik_c)
      L_loglik = L_loglik + L_loglik_c
    }
    if(L_loglik <= cand_L){
      cand_L = L_loglik
      cand_argmin = cands
    }
    
  }
  if(vb) cat("\n\tChosen subset: {", paste(cand_argmin, collapse =","),"}", "\tScore:",  cand_L)
  
  # Augment the chosen subset by node nj
  #Find subsets containing node_j
  augmented_L  = Inf
  augmented_argmin =""
  for (xsid in 1:length(Vario_result_i$X_subsets)) {
    cand = Vario_result_i$X_subset_nms[[xsid]]
    if(length(cand)==1) next
    if(!(xi_names[node_j] %in% cand)) next
    if(length(intersect(cand_argmin, cand))!=(length(cand_argmin))) next
    if(length(cand_argmin)!=length(cand)-1) next
    
    #  Subsets_i = c(Subsets_i, xsid)
    Loglik = 0
    regr = Vario_result_i$F_histories[[xsid]]$regress_E1_En
    for (i in 1:length(regr)){
      rsq =regr[[i]]$rsq
      dN = nrow(Vario_result_i$F_histories[[xsid]]$Data_E1_En[[i]])
      sig = rsq / (dN-1)
      Loglik =Loglik -(-(dN/2)*logg(2*pi*sig^2) - 1/(2*sig^2) * rsq) 
    }
    if(Loglik <= augmented_L){
      augmented_L = Loglik
      augmented_argmin = cand
    }
    
  }
  if(vb) cat("\n\tAugmented set: {", augmented_argmin, "}\tScore:", augmented_L)
  
  
  if(augmented_L==Inf | cand_L == Inf) {
    if(vb) cat("\nInvalid")
    return(list(set="", score=0,score_aug=0, found=FALSE,gain=0))
  }
  diff = max(cand_L-augmented_L, 0)
  if(vb) cat("\nDiff:", diff)
  return(list(set=cand_argmin, cost_S_i=cand_L,cost_Sj_i=augmented_L, found=TRUE,gain=diff))
  
}

edgeanalysis_MDL_gain = function(xi_names, 
                         edges_ordered, 
                         Vario_results,
                         edge_ji, 
                         node_j, node_i,
                         node_not_allowed = "NONE", #if name given: subset S may not include this node
                         vb=FALSE){

  #
  # Find gain of adding the edge J->I vs. that of both edges J->K, K->I
  # - when the remaining graph is kept fixed
  # - i.e., use all ingoing nodes N_i but J, {N_i}->I, that Vario found
  # - and compare MDLcost({N_i}->I) to MDLcost({N_i, J}->I)
  if(vb) cat("\n\nEdge", xi_names[node_j], "->", xi_names[node_i])
  Vario_result_i = Vario_results[[node_i]] 
  edges_to_i = c()
  
  # Find edges {N_i}->I
  for (e_li in 1:length(edges_ordered)){
    edge_li = edges_ordered[[e_li]]
    node_ii = edge_li$node_i
    node_l = edge_li$node_j
    
    if((node_ii != node_i) |
       (node_l == node_j) |
       (node_l %in% edges_to_i)) next
    edges_to_i = c(edges_to_i, node_l)
  }
  
  if(length(edges_to_i)==0) return(list(set="", score=0,score_aug=0, found=FALSE,gain=0))
  #Find subset ID to look up info from regression on {Ni}->I and {Ni, J}->I
  nms_to_i = xi_names[edges_to_i] 
  if(vb) cat("\n\tIngoing Edges: (", nms_to_i, ")" )
  
   
  
  cand_L = Inf
  cand_argmin = ""
  
  #  Find MDLcost of admissible subsets S, S->I
  # Choose the best subset S (irrespective of J)
  for(xsid in 1:length(Vario_result_i$X_subset_nms)){
    cands = Vario_result_i$X_subset_nms[[xsid]]
    if(xi_names[node_j] %in% cands |
       node_not_allowed %in% cands |
       length(cands)>3) next
    

    #mind that subset sizes might be restricted, here to 4, thus select the minimal, up-to-size-3-subset that best explains I while covering all of edges-to-I 
 
    #if(length(intersect(nms_to_i, cands)) != min(length(nms_to_i),3)) next #3 of in edges but maybe noncausal nodes
    if(length(intersect(nms_to_i, cands))!= length(cands)) next

    
   ####if(vb) cat("\nAdmissible subset: {", paste(cands, collapse =","), "}", xsid)
    
    # Find Likelihood terms from regression result of subset S
     regr = Vario_result_i$F_histories[[xsid]]$regress_E1_En
     dta = Vario_result_i$F_histories[[xsid]]$Data_E1_En
     L_loglik = 0 
     
    for (i in 1:length(regr)){
      rsq =regr[[i]]$rsq
      dN = nrow(dta[[i]])
      sig = rsq / (dN-1)
      L_loglik_c = -(-(dN/2)*logg(2*pi*sig^2) - 1/(2*sig^2) * rsq) 
      #if(vb) cat("\n\t", dN, rsq, sig, L_loglik_c)
      L_loglik = L_loglik + L_loglik_c
    }
     if(L_loglik <= cand_L){
      cand_L = L_loglik
      cand_argmin = cands
    }
     
     }
     if(vb) cat("\n\tChosen subset: {", paste(cand_argmin, collapse =","),"}", "\tScore:",  cand_L)
  
     # Augment the chosen subset by node nj
     #Find subsets containing node_j
     augmented_L  = Inf
     augmented_argmin =""
     for (xsid in 1:length(Vario_result_i$X_subsets)) {
       cand = Vario_result_i$X_subset_nms[[xsid]]
       if(length(cand)==1) next
       if(!(xi_names[node_j] %in% cand)) next
       if(length(intersect(cand_argmin, cand))!=(length(cand_argmin))) next
      if(length(cand_argmin)!=length(cand)-1) next

     #  Subsets_i = c(Subsets_i, xsid)
       Loglik = 0
       regr = Vario_result_i$F_histories[[xsid]]$regress_E1_En
       for (i in 1:length(regr)){
         rsq =regr[[i]]$rsq
         dN = nrow(Vario_result_i$F_histories[[xsid]]$Data_E1_En[[i]])
         sig = rsq / (dN-1)
         Loglik =Loglik -(-(dN/2)*logg(2*pi*sig^2) - 1/(2*sig^2) * rsq) 
       }
       if(Loglik <= augmented_L){
         augmented_L = Loglik
         augmented_argmin = cand
       }
       
     }
     if(vb) cat("\n\tAugmented set: {", augmented_argmin, "}\tScore:", augmented_L)
     
     
     if(augmented_L==Inf | cand_L == Inf) {
       if(vb) cat("\nInvalid")
       return(list(set="", score=0,score_aug=0, found=FALSE,gain=0))
     }
     diff = max(cand_L-augmented_L, 0)
     if(vb) cat("\nDiff:", diff)
     return(list(set=cand_argmin, cost_S_i=cand_L,cost_Sj_i=augmented_L, found=TRUE,gain=diff))

}

# Help: Gain in likelihood compared for two parents J,K of I
# Since only J->I can be pruned here: allow subsets S including K for 

