# Ground truth and Related Work on Sachs ----
# yj order: praf, pmek, plcg, PIP2, PIP3, p44/42, pakts, PKA, PKC, p38, pjnk

# Graphs per target -----
consensus_small_graph_Y = function(yj, xis){ 
  if(yj==1) edges = c("PKA", "PKC")  
  if(yj==2) edges = c("praf", "PKA", "PKC")  
  if(yj==3) edges = c()   
  if(yj==4) edges = c("plcg", "PIP3")  
  if(yj==5) edges = c("plcg")  
  if(yj==6) edges = c("pmek", "PKA")  
  if(yj==7) edges = c("p44/42", "PKA", "PIP3")  
  if(yj==8) edges = c("PKC")  
  if(yj==9) edges = c("plcg", "PIP2")  
  if(yj==10) edges = c("PKC", "PKA")  
  if(yj==11) edges = c("PKC",  "PKA")  
  
  return(subgraph_to_Y(yj, xis, edges))
}


# Methods for causal inference from gene perturbation experiments and validation. Meinshausen et al.
# Consensus and newly reported edges
consensus_augmented_graph_Y = function(yj, xis,vb=FALSE){  
  if(yj==1) edges = c("pmek", "PKA", "PKC") 
  if(yj==2) edges = c("praf", "PKA", "PKC") 
  if(yj==3) edges = c("PIP2", "PIP3", "PKC")  
  if(yj==4) edges = c("plcg", "PIP3", "PKC")  
  if(yj==5) edges = c("plcg", "PIP2")  
  if(yj==6) edges = c("pmek", "PKA", "pakts473")  
  if(yj==7) edges = c("p44/42", "PKA", "PKC", "PIP3")  
  if(yj==8) edges = c("p44/42", "PKC")  
  if(yj==9) edges = c("plcg", "PIP2", "PKA") # possibly also p38 and jnk.
  if(yj==10) edges = c("PKC", "pjnk", "PKA")  
  if(yj==11) edges = c("PKC",  "PKA") # possibly p38.
  
 if(vb) cat ("\nConsensus Augmented",  "{", edges, "} -> ",xis[yj], "\n")
  return(subgraph_to_Y(yj, xis, edges))
}

# Invariant Causal Prediction. Peters et al.
icp_graph_Y = function(yj, xis, vb=FALSE){
  if(yj==1) edges = c( ) #praf
  if(yj==2) edges = c( ) #pmek
  if(yj==3) edges = c("PIP2") #plcg
  if(yj==4) edges = c("PIP3","plcg") #PIP2
  if(yj==5) edges = c ( ) #PIP3
  if(yj==6) edges = c("pakts473", "PKA") # p44/Erk
  if(yj==7) edges = c("p44/42") #Akt
  if(yj==8) edges = c( ) #PKA
  if(yj==9) edges = c( ) #PKC; p38 and jnk are likely.
  if(yj==10) edges = c( ) #  p38 
  if(yj==11) edges = c("PKC" ) #  pjnk, possibly p38
  
  if(vb) cat ("ICP Edges",  "{", edges, "} -> ",xis[yj], "\n")
  return(subgraph_to_Y(yj, xis, edges))
}

# Hidden ICP
hicp_graph_Y = function(yj, xis, vb=FALSE){
  if(yj==1) edges = c("pmek")  
  if(yj==2) edges = c("praf")  
  if(yj==3) edges = c( )  
  if(yj==4) edges = c("PIP3","plcg") 
  if(yj==5) edges = c ( ) 
  if(yj==6) edges = c("pakts473")  
  if(yj==7) edges = c("p44/42", "PKA" )  
  if(yj==8) edges = c( )  
  if(yj==9) edges = c("p38", "pjnk")  
  if(yj==10) edges = c("PKC", "pjnk")   
  if(yj==11) edges = c("PKC", "p38")  
  if(vb) cat ("HiddenICP",  "{", edges, "} -> ",xis[yj], "\n")
  return(subgraph_to_Y(yj, xis, edges))
}

# Joint Causal Inference from Multiple Contexts. Moji, Magliacane et al.
jci_graph_Y = function(yj, xis, vb = FALSE){
  if(yj==1) edges = c("pmek" ) 
  if(yj==4) edges = c("PIP3", "plcg" )  
  if(yj==6) edges = c("pakts")   
  if(yj==9) edges = c("p38"  )  
  if(yj==10) edges = c("pjnk") 
  if(yj==11) edges = c("p38")  
  if(yj==2 
     | yj==3 
     | yj==5 
     | yj==7 
     | yj==8) edges = c(  )   
  
  if(vb) cat ("JCI Edges",  "{", edges, "} -> ",xis[yj], "\n")
  return(subgraph_to_Y(yj, xis, edges))
}

# Graphs overall -----

consensus_small_graph_Sachs = function(x_nms){
  G = consensus_small_graph_Y(1, x_nms)
  
  for (y in 2:11){
    G = G + consensus_small_graph_Y(y, x_nms)
  }
  return(G)
}
consensus_augmented_graph_Sachs = function(x_nms){
  G = consensus_augmented_graph_Y(1, x_nms)
  
  for (y in 2:11){
    G = G + consensus_augmented_graph_Y(y, x_nms)
  }
  return(G)
}

consensus_ancestral_graph = function(adj, dontconsider_causal=TRUE, dontconsider_anticausal=TRUE){
  adj_anc = matrix(0, nrow(adj), ncol(adj))
  adj_anc3 = matrix(0, nrow(adj), ncol(adj))
  
  for (i in 1:11){
    for (j in 1:11){
      
      if (i==j)  
        next
      if(dontconsider_causal & adj[i,j]==1) next
      if(dontconsider_anticausal & adj[j,i]==1) next
      
      for (r in 1:11){
        if(r==i |r==j | (adj[i,r]==0 & adj[r,j]==0)) #either edge must exist st there can be an ancestral path over 2 or 3 variables
          next
        if (adj[i,r]==1 & adj[r,j]==1) adj_anc[i,j] = 1
      }
      if(adj_anc[i,j]==0) 
        for (r in 1:11){
         
          for (q in 1:11){
          if(q==i | q ==j | q==r )  
            next
          if ((adj[i,r]==1 & adj[r,q]==1 & adj[q,j]==1)
              | (adj[i,q]==1 & adj[q,r]==1 & adj[r,j]==1)) adj_anc3[i,j] = 1
        }
      }
        
    }
  }
  return(list(adj_anc=adj_anc, adj_anc3=adj_anc3))
}

#old
vario_graph_Sachs = function(x_nms){
  G = vario_graph_Y(1, x_nms)
  
  for (y in 2:11){
    G = G + vario_graph_Y(y, x_nms)
  }
  return(G)
}
hicp_graph_Sachs = function(x_nms){
  G = hicp_graph_Y(1, x_nms)
  
  for (y in 2:11){
    G = G + hicp_graph_Y(y, x_nms)
  }
  return(G)
}
icp_graph_Sachs = function(x_nms){
  G = icp_graph_Y(1, x_nms)
  
  for (y in 2:11){
    G = G + icp_graph_Y(y, x_nms)
  }
  return(G)
}
jci_graph_Sachs = function(x_nms){
  G = jci_graph_Y(1, x_nms)
  
  for (y in 2:11){
    G = G + jci_graph_Y(y, x_nms)
  }
  return(G)
}

subgraph_to_Y = function (yj, xis, 
                          edges_i_j, #named
                          edges_ids = NULL){ #ids, for sanity
  edges_i_j = sort(which(xis %in% edges_i_j))
  edges_ids = sort(edges_ids)
  
  if(length(edges_ids)) sapply(1:length(edges_ids), function(i)
    assert(edges_ids[i]==edges_i_j[i]))
  
  X = length(xis) 
  G = rep(0,X)
  for(i in 2:X)
    G = rbind(G, rep(0,11)) 
  for(xi in edges_i_j) G[xi,yj] = 1
  return(G)
}
# 
# consensus_graph_Vario = function(yj, xis,  edges_i_j, 
#                                  edges_ids ){ 
#   cat ("Vario Edges",  "{", edges_i_j, "} -> ",xis[yj], "\n")
#   return(consensus_graph(yj, xis, edges_i_j, edges_ids))
# }


# Interventions
# praf, pmek, plcg, PIP2, PIP3, p44/42, pakts, PKA, PKC, p38, pjnk

#2: ICAM-2
# 3: AKT INH
# 4:  G0076, PKC activity
# 5: Psitectorigenin, Pip2 abundance
# 6: U0126: Mek activity
# 7: LY294002: Pip2,3 mechanism change
# 8: PMA: PKC activity
# 9.: beta2CAMP: PKA activity
consensus_indirect_interventions_Y = function(yj){
  if(yj==1) edges = c(4,5,7,8  ) #praf
  if(yj==2) edges = c(4 ,5,7,8 ) #pmek
  if(yj==3) edges = c(5,7  ) #plcg
  if(yj==4) edges = c(5,7 ) #PIP2
  if(yj==5) edges = c (5,7 ) #PIP3
  if(yj==6) edges = c(4, 5,6 ,7,8  ) # p44/Erk 
  if(yj==7) edges = c(5, 7,8  ) #Akt
  if(yj==8) edges = c( ) #PKA
  if(yj==9) edges = c( 5,7) #PKC 
  if(yj==10) edges = c(4,5, 7,8  ) #  p38 
  if(yj==11) edges = c(4 ,5, 7,8) #  pjnk 
  return(edges)
}


consensus_direct_interventions_Y = function(yj){
  if(yj==1) edges = c(4,8   ) #praf
  if(yj==2) edges = c(4,8   ) #pmek
  if(yj==3) edges = c(  ) #plcg
  if(yj==4) edges = c(5,7 ) #PIP2
  if(yj==5) edges = c (7 ) #PIP3
  if(yj==6) edges = c(6 ,8  ) # p44/Erk 
  if(yj==7) edges = c(  8 ) #Akt
  if(yj==8) edges = c( ) #PKA
  if(yj==9) edges = c( ) #PKC 
  if(yj==10) edges = c(4 ,8  ) #  p38 
  if(yj==11) edges = c(4,8  ) #  pjnk 
  return(edges)
}


consensus_interventions_old_Y = function(yj){
  if(yj==1) edges = c(4,5,7 ) #praf
  if(yj==2) edges = c(4,5,7 ) #pmek
  if(yj==3) edges = c(5 ,7) #plcg
  if(yj==4) edges = c(3,5,7 ) #PIP2
  if(yj==5) edges = c (3,5,7 ) #PIP3
  if(yj==6) edges = c(4,5,6,7 ) # p44/Erkedges
  if(yj==7) edges = c(3,5,7 ) #Akt
  if(yj==8) edges = c( ) #PKA
  if(yj==9) edges = c(5,7) #PKC 
  if(yj==10) edges = c(3,4,5,7 ) #  p38 
  if(yj==11) edges = c(3,4 ,5,7) #  pjnk 
  return(edges)
}

eaton_indirect_interventions_Y = function(yj){
  if(yj==1) edges = c(4,5,6,8 ) #praf
  if(yj==2) edges = c(4,5,6,8 ) #pmek
  if(yj==3) edges = c(4,5 ) #plcg
  if(yj==4) edges = c(3,4,5 ) #PIP2
  if(yj==5) edges = c (3,4,5,8 ) #PIP3
  if(yj==6) edges = c( 4,6 ) # p44/Erk
  if(yj==7) edges = c(3,4 ,6,8) #Akt
  if(yj==8) edges = c(4,6 ,8) #PKA
  if(yj==9) edges = c(4,5,6,8 ) #PKC 
  if(yj==10) edges = c(3,4,5,6,8 ) #  p38 
  if(yj==11) edges = c(3,4 ,5,6,8) #  pjnk
  return(edges)
}


eaton_direct_interventions_Y = function(yj){
  if(yj==1) edges = c( 6,8 ) #praf
  if(yj==2) edges = c(4, 6 ) #pmek
  if(yj==3) edges = c(4,5 ) #plcg
  if(yj==4) edges = c(3, 5 ) #PIP2
  if(yj==5) edges = c (3,8 ) #PIP3
  if(yj==6) edges = c( 4,6 ) # p44/Erk
  if(yj==7) edges = c(3, 6) #Akt
  if(yj==8) edges = c(4 ,8) #PKA
  if(yj==9) edges = c(  6,8) #PKC 
  if(yj==10) edges = c( 4 ,8  ) #  p38 
  if(yj==11) edges = c(3 , 6) #  pjnk
  return(edges)
}
 

#2: ICAM-2
# 3: AKT INH
# 4:  G0076, PKC activity
# 5: Psitectorigenin, Pip2 abundance
# 6: U0126: Mek activity
# 7: LY294002: Pip2,3 mechanism change
# 8: PMA: PKC activity
# 9.: beta2CAMP: PKA activity
jci_direct_interventions_Y = function(yj){
  if(yj==1) edges = c( ) #praf
  if(yj==2) edges = c( 6,8) #pmek
  if(yj==3) edges = c(4  ) #plcg
  if(yj==4) edges = c( 5) #PIP2
  if(yj==5) edges = c ( ) #PIP3
  if(yj==6) edges = c( 6) # p44/Erk
  if(yj==7) edges = c( 4) #Akt
  if(yj==8) edges = c( 4) #PKA
  if(yj==9) edges = c( 6) #PKC 
  if(yj==10) edges = c( 4) #  p38 
  if(yj==11) edges = c( ) #  pjnk
  return(edges)
}
jci_indirect_interventions_Y = function(yj){
  if(yj==1) edges = c(6,8 ) #praf
  if(yj==2) edges = c( 6,8) #pmek
  if(yj==3) edges = c(4  ) #plcg
  if(yj==4) edges = c(4,  5) #PIP2
  if(yj==5) edges = c ( ) #PIP3
  if(yj==6) edges = c( 4, 6) # p44/Erk
  if(yj==7) edges = c( 4) #Akt
  if(yj==8) edges = c( 4) #PKA
  if(yj==9) edges = c( 4, 6) #PKC 
  if(yj==10) edges = c( 4) #  p38 
  if(yj==11) edges = c( 4) #  pjnk
  return(edges)
}

