ICP_ex1 = function(){
  ##########################################
  ####### 1st example:
  ####### Simulate data with interventions
  set.seed(1)
  ## sample size n
  n <- 4000
  ## 5 predictor variables
  p <- 5
  ## simulate as independent Gaussian variables
  X <- matrix(rnorm(n*p),nrow=n)
  ## divide data into observational (ExpInd=1) and interventional (ExpInd=2)
  ExpInd <- c(rep(1,n/2),rep(2,n/2))
  ## for interventional data (ExpInd==2): change distribution
  X[ExpInd==2,] <- sweep(X[ExpInd==2,],2, 5*rnorm(p) ,FUN="*")
  ## first two variables are the causal predictors of Y
  beta <- c(1,1,rep(0,p-2))
  ## response variable Y
  Y <- as.numeric(X%*%beta + rnorm(n))
  ## optinal: make last variable a child of Y (so last variable is non-causal for Y)
  X[,p] <- 0.3*Y + rnorm(n)
  ####### Compute "Invariant Causal Prediction" Confidence Intervals
  icp <- ICP(X,Y,ExpInd)
  ###### Print/plot/show summary of output
  print(icp)
  plot(icp)
  #### compare with linear model
  cat("\n compare with linear model \n")
  print(summary(lm(Y~X)))
  
  return(list(X=X, Y=Y, icp=icp, ExpInd=ExpInd))
}
Vario_ex1 = function(){
  D = ICP_ex1()
  
  ICP(D$X ,D$Y, D$ExpInd, alpha=0.1) 
  #hiddenICP(D$X,D$Y,D$ExpInd,alpha=0.01)
  
  #result_Vario_X1 = main_search(transform_ICP_data(t(as_mat(D$X[,1])), D$Y, D$ExpInd)$D_data,
   #                             list(1), init_config_analysis())
 # result_Vario_X12 = main_search(transform_ICP_data(cbind(D$X[,1], D$X[,2]), D$Y, D$ExpInd)$D_data,
  #                               list(1), init_config_analysis())
  
  #result_Vario_X12$htest_invariant
  Vario_data = transform_ICP_data(cbind(D$X[,1], D$X[,2], D$X[,3], D$X[,4], D$X[,5]), D$Y, D$ExpInd)
  names(Vario_data$X_names) = Vario_data$X_names
  
 Vario_subset_search(Vario_data$D_data, 
                     Vario_data$X_names)
}

ICP_counterex1 = function(){
  n <- 4000
  p <- 5
  X <- matrix(rnorm(n*p),nrow=n)
  ExpInd <- c(rep(1,n/5),rep(2,n/5), rep(3,n/5), rep(4,n/5), rep(5,n/5))
  
  X[ExpInd==2,] <- sweep(X[ExpInd==2,],2, 5*rnorm(p) ,FUN="*")
  X[ExpInd==3,] <- sweep(X[ExpInd==3,],2, 2*rnorm(p) ,FUN="*")
  X[ExpInd==4,] <- sweep(X[ExpInd==4,],2, 6*rnorm(p) ,FUN="*")
  X[ExpInd==5,] <- sweep(X[ExpInd==5,],2, 3*rnorm(p) ,FUN="*")
  
  beta <- c(1,1,rep(0,p-2))
  gamma <- c(.5,.5,rep(0,p-2))
  Y <- as.numeric(X%*%beta + rnorm(n))
  #This time, change in strength in Environment 5
  Y[ExpInd==5] <- as.numeric(X[ExpInd==5,] %*%gamma + rnorm(n/5))
  X[,p] <- 0.3*Y + rnorm(n)
  
  icp <- ICP(X,Y,ExpInd)
  ###### Print/plot/show summary of output
  print(icp)
  plot(icp)
  #### compare with linear model
  cat("\n compare with linear model \n")
  print(summary(lm(Y~X)))
  D = list(X=X, Y=Y, icp=icp, ExpInd=ExpInd)
  Vario_data = transform_ICP_data(cbind(D$X[,1], D$X[,2], D$X[,3], D$X[,4], D$X[,5]), D$Y, D$ExpInd)
  names(Vario_data$X_names) = Vario_data$X_names
  
  xh = Vario_subset_search(Vario_data$D_data, Vario_data$X_names)
  
}

hiddenICP_ex1 = function(){

##########################################
####### 1st example:
####### Simulate data with interventions
set.seed(1)
## sample size n
n <- 2000
## 4 predictor variables
p <- 4
## simulate as independent Gaussian variables
X <- matrix(rnorm(n*p),nrow=n)
## divide data into observational (ExpInd=1) and interventional (ExpInd=2)
ExpInd <- c(rep(1,n/2),rep(2,n/2))
## for interventional data (ExpInd==2): change distribution
nI <- sum(ExpInd==2)
X[ExpInd==2,] <- X[ExpInd==2,] + matrix( 5*rt( nI*p,df=3),ncol=p)
## add hidden variables
W <- rnorm(n) * 5
X <- X + outer(W, rep(1,4))
## first two variables are the causal predictors of Y
beta <- c(1,1,0,0)
## response variable Y
Y <- as.numeric(X%*%beta - 2*W + rnorm(n))
####### Compute "hidden Invariant Causal Prediction" Confidence Intervals
icp <- hiddenICP(X,Y,ExpInd,alpha=0.01)
print(icp)
###### Print point estimates and points in the confidence interval closest to 0
print(icp$betahat)
print(icp$maximinCoefficients)
cat("true coefficients are:", beta)
#### compare with coefficients from a linear model
cat("coefficients from linear model:")
print(summary(lm(Y ~ X-1)))
print(summary(lm(Y ~ X)))

return(list(X=X, Y=Y, icp=icp, ExpInd=ExpInd))
}

Vario_counterex1 = function(){
  D = hiddenICP_ex1()

  ICP(D$X ,D$Y, D$ExpInd, alpha=0.1) # rejected
  hiddenICP(D$X,D$Y,D$ExpInd,alpha=0.01)
  
  
  result_Vario_X1 = Vario_Pi_search(transform_ICP_data(t(as_mat(D$X[,1])), D$Y, D$ExpInd)$D_data,
                               list(1), init_config_analysis())
  result_Vario_X12 = Vario_Pi_search(transform_ICP_data(cbind(D$X[,1], D$X[,2]), D$Y, D$ExpInd)$D_data,
                                list(1), init_config_analysis())
  
  result_Vario_X12$htest_invariant
}


hiddenICP_ex2 = function(){
  ##########################################
  ####### 2nd example:
  ####### Simulate model X -> Y -> Z with hidden variables, trying to
  ###### estimate causal effects from (X,Z) on Y
  set.seed(1)
  ## sample size n
  n <- 10000
  ## simulate as independent Gaussian variables
  W <- rnorm(n)
  noiseX <- rnorm(n)

  noiseY <- rnorm(n)
  noiseZ <- rnorm(n)
  ## divide data into observational (ExpInd=1) and interventional (ExpInd=2)
  ExpInd <- c(rep(1,n/2),rep(2,n/2))
  noiseX[ which(ExpInd==2)] <- noiseX[ which(ExpInd==2)] * 5
  noiseZ[ which(ExpInd==2)] <- noiseZ[ which(ExpInd==2)] * 3
  ## simulate equilibrium data
  beta <- -0.5
  alpha <- 0.9
  X <- noiseX + 3*W
  Y <- beta* X + noiseY + 3*W
  Z <- alpha*Y + noiseZ
  ####### Compute "Invariant Causal Prediction" Confidence Intervals
  icp <- hiddenICP(cbind(X,Z),Y,ExpInd,alpha=0.1)
  print(icp)
  ###### Print/plot/show summary of output (truth here is (beta,0))
  print(signif(icp$betahat,3))
  print(signif(icp$maximinCoefficients,3))
  cat("true coefficients are:", beta,0)
  #### compare with coefficients from a linear model
  cat("coefficients from linear model:")
  print(summary(lm(Y ~ X + Z -1)))
  return(list(X=X, Y=Y, Z=Z, ExpInd=ExpInd))
}

#Does not work as hidden-ICP (on 2 envs), but ICP fails as well
Vario_counterex2 = function(){
  data_ICP = hiddenICP_ex2()
  data_Vario_X = transform_ICP_data(data_ICP$X, data_ICP$Y, data_ICP$ExpInd)
  data_Vario_Z = transform_ICP_data(data_ICP$Z, data_ICP$Y, data_ICP$ExpInd)
  

  result_Vario_X = Vario_Pi_search(data_ICP$X, list(1), init_config_analysis())
  result_Vario_Z = Vario_Pi_search(data_ICP$Y, list(1), init_config_analysis())
  
  result_ICP = ICP(cbind(data_ICP$X, data_ICP$Z), data_ICP$Y, data_ICP$ExpInd, alpha=0.1)
  result_hidden =  hiddenICP(cbind(data_ICP$X, data_ICP$Z),data_ICP$Y,data_ICP$ExpInd,alpha=0.1)
}




ICP_ex3 = function(){
  ## Not run:
  ##########################################
  ####### 3rd example:
  ####### College Distance data
  library(AER)
  data("CollegeDistance")
  CD <- CollegeDistance
  ## define two experimental settings by
  ## distance to closest 4-year college
  ExpInd <- list()
  ExpInd[[1]] <- which(CD$distance < quantile(CD$distance,0.5))
  ExpInd[[2]] <- which(CD$distance >= quantile(CD$distance,0.5))
  ## target variable is binary (did education lead at least to BA degree?)
  Y <- as.factor(CD$education>=16)
  ## use these predictors
  X <- CD[,c("gender","ethnicity","score","fcollege","mcollege","home",
             "urban","unemp","wage","tuition","income","region")]
  ## searching all subsets (use selection="lasso" or selection="stability"
  ## to select a subset of subsets to search)
  ## with selection="all" the function will take several minutes
  icp <- ICP(X,Y,ExpInd,selection="lasso",#selection="all",
             alpha=0.1)
  return(list(X=X, Y=Y, icp=icp, ExpInd=ExpInd))
}

Vario_ex3 = function(){
  ICP_data = ICP_ex3()
  D = transform_ICP_data(ICP_data$X, ICP_data$Y, ICP_data$ExpInd)
  
  #TODO
  
}




