
reproduce_find_Pi = function(vb = TRUE, subdir_nm="run_pi/", max_it=50){
 
  
  it_pre=50
  E_n=5
  D_n=1000  
  
  confints_basic = list(conf_pione=sample_conf_pi_one(E_n, it_pre, vb=FALSE, version_mdl=FALSE),
                        conf_nogroups= sample_conf_nogroups(E_n, it_pre, vb=FALSE, version_mdl=FALSE),
                        conf_best=sample_conf_pi_best(E_n, it_pre, vb=FALSE, version_mdl=FALSE))
  
  confints_mdl = list(conf_pione=sample_conf_pi_one(E_n, it_pre, vb=FALSE, version_mdl=TRUE),
                      conf_nogroups= sample_conf_nogroups(E_n, it_pre, vb=FALSE, version_mdl=TRUE),
                      conf_best=sample_conf_pi_best(E_n, it_pre, vb=FALSE, version_mdl=TRUE))
  
  outer_it = 1 # there is an inner iteration for each pi which counts the seed up. so that graphs can still be identified by their seed (=> DAG, pi) and yi (=> node in DAG)
  seed = 100
  out_createdir("~/Vario/out/")
  
  #For JCI consider the simpler setting where one group is observational and one interventional, s.t. edges from context variables to Y can capture this.
  # The more general case of explanatory groups we do in Figure 1.
  
  for(outer_it in 1:60){ 
    # each corresponds to one DAG with 5-10 targets, 4 partitions(no permut. considered and only 2-group). 
    #= 1,5k expected repetitions
     
    node_n = 5 #seed also identifies node number
    if(outer_it>10) node_n = 6
    if(outer_it>20) node_n = 7
    if(outer_it>30) node_n = 8 
    if(outer_it>40) node_n = 9
    if(outer_it>50) node_n = 10
    if(outer_it> max_it) break
    
    
    if(node_n<8) deg_p=0.3 #ow very connected?
    else deg_p = 0.5
    # the actual node number Varios runs on depend on the size of Markov Blankets for eaxch target Y
    
    res = synth_DAG_param(E_n=E_n, 
                          D_n=D_n,
                          subset_search=FALSE,
                          #Setting A.
                          interventional_vs_observational = FALSE,
                          perfect_interventions = FALSE,
                          #Setting B.
                          one_intervention_per_context=FALSE,
                          #
                          X_max_n=4, 
                          intervene_pa_G=c(3,5), 
                          node_n=node_n,  
                          deg_p=deg_p,
                          it_pre=50, 
                          confints_basic=confints_basic,
                          confints_mdl=confints_mdl,
                          subdir_nm=subdir_nm,
                          show_context_coefficients = FALSE,
                          seed=seed,
                          vb=vb)
    
    seed = seed + 1
    cat("\n\nSeed:",seed,"\n\n")
  }
  #Show results by group number, here up to 5.
  # Group number 0 is a special case: 
  # - correspond to either data from the no-group partition targets with no parents. partitions need to have no groups (because partitions are not meaningful when Y has no parents.)
  # -interventions count 1TP or 1FN depending on whether no groups are correctly predicted (because counting interventions per context is not meaningful here, if Y has no parents.)
  
  infiles = list(paste0("~/Vario/out/check_pi/", subdir_nm, "pi_5_2.txt"), 
                 paste0("~/Vario/out/check_pi/", subdir_nm, "pi_5_3.txt"), 
                 paste0("~/Vario/out/check_pi/", subdir_nm, "pi_5_4.txt"), 
                 paste0("~/Vario/out/check_pi/", subdir_nm, "pi_5_0.txt"))
  decision_rate_pi_causal(infiles, cut = 1)
}







reproduce_find_Pi_and_Causal = function(small=FALSE,
                                        vb = TRUE,
                                        subdir_nm="Example_diagonal/", max_it=40){
  #max_it 40 for Example_medium

  #Generate for each seed, a DAG, then for each node Y, for each partition Pi: generate data for the DAG where Y has partition Pi.
  # Run Varios subset search 
  # -- Data generation outputs: stores the true graphs (Markov Blankets with re-ordered indices for each target)  and data for them in "~/Vario/results_jci" for later, and each dataset in "~/jci/data_vario" with name "100-y1-12345{-graph.csv, -graph.dot, .json, data.csv}
  # -- Vario outputs: relevant counts in "~/Vario/out/check_causal",
  # -- each decision has format : Pi (TP), Parents (TP FP FN), Varios confidence, Intervention on Y in a context (TP FP FN TN),  Varios score,...others
  # Then, output decision rate plots (Accuracy Pi or only Confident Pi; F1 causal parents or only confident ones; F1 Interventions)
  
  
it_pre=50
E_n=5
D_n=1000  
  
confints_basic = list(conf_pione=sample_conf_pi_one(E_n, it_pre, vb=FALSE, version_mdl=FALSE),
                        conf_nogroups= sample_conf_nogroups(E_n, it_pre, vb=FALSE, version_mdl=FALSE),
                        conf_best=sample_conf_pi_best(E_n, it_pre, vb=FALSE, version_mdl=FALSE))

confints_mdl = list(conf_pione=sample_conf_pi_one(E_n, it_pre, vb=FALSE, version_mdl=TRUE),
                      conf_nogroups= sample_conf_nogroups(E_n, it_pre, vb=FALSE, version_mdl=TRUE),
                      conf_best=sample_conf_pi_best(E_n, it_pre, vb=FALSE, version_mdl=TRUE))

outer_it = 1 # there is an inner iteration for each pi which counts the seed up. so that graphs can still be identified by their seed (=> DAG, pi) and yi (=> node in DAG)
seed = 100
out_createdir("~/Vario/out/")

#For JCI consider the simpler setting where one group is observational and one interventional, s.t. edges from context variables to Y can capture this.
# The more general case of explanatory groups we do in Figure 1.

for(outer_it in 1:60){ 
  # each corresponds to one DAG with 5-10 targets, 4 partitions(no permut. considered and only 2-group). 
  #= 1,5k expected repetitions

  if(small & outer_it >= 5) break
   node_n = 5 #seed also i dentifies node number
  if(outer_it>10) node_n = 6
  if(outer_it>20) node_n = 7
  if(outer_it>30) node_n = 8 
  if(outer_it>40) node_n = 9
  if(outer_it>50) node_n = 10
 if(outer_it> max_it) break
  
  
  if(node_n<8) deg_p=0.3 #ow very connected?
  else deg_p = 0.5
  # the actual node number Varios runs on depend on the size of Markov Blankets for eaxch target Y
  
  res = synth_DAG_param(E_n=E_n, 
                        D_n=D_n,
                        subset_search=TRUE,
                        #Setting A.
                        interventional_vs_observational = FALSE,
                        perfect_interventions = FALSE,
                        #Setting B.
                        one_intervention_per_context=FALSE,
                        #
                        X_max_n=4, 
                        intervene_pa_G=c(3,5), 
                        node_n=node_n,  
                        deg_p=deg_p,
                        it_pre=50, 
                        confints_basic=confints_basic,
                        confints_mdl=confints_mdl,
                        subdir_nm=subdir_nm,
                        show_context_coefficients = FALSE,
                        seed=seed,
                        vb=vb)

  seed = seed + 1
  cat("\n\nSeed:",seed,"\n\n")
}
  #Show results by group number, here up to 5.
  # Group number 0 is a special case: 
  # - correspond to either data from the no-group partition targets with no parents. partitions need to have no groups (because partitions are not meaningful when Y has no parents.)
  # -interventions count 1TP or 1FN depending on whether no groups are correctly predicted (because counting interventions per context is not meaningful here, if Y has no parents.)
  
  infiles = list(paste0("~/Vario/out/check_causal/", subdir_nm, "pi_5_2.txt"), 
                 paste0("~/Vario/out/check_causal/", subdir_nm, "pi_5_3.txt"), 
                 paste0("~/Vario/out/check_causal/", subdir_nm, "pi_5_4.txt"), 
                 paste0("~/Vario/out/check_causal/", subdir_nm, "pi_5_0.txt"))
  decision_rate_pi_causal(infiles)
}



