require(pcalg)
require(graph)

gen_rand_dag = function(node_n=5, 
                        interv_type="soft",
                        out_p=0.3,
                        D_n=1000,
                        C_n=5,
                        seed=1,
                        vb=TRUE){
  set.seed(seed)
  if(!(interv_type %in% list("perfect","soft","shift"))) stop("Unknown intervention type")
  
  dag = randomDAG(node_n, prob = out_p, lB=0.25, uB=1)
  
  wgt = wgtMatrix(dag,transpose = FALSE)
  if(vb) cat("\nInitial weight mat\n")
  if (vb) show(wgt) 
  
  # the first C_n nodes are intervened in these contexts: (if there are more than C_n nodes, skip those)
  targets = sample(C_n,size=node_n,replace=(C_n > node_n))  # permn(1:C_n)[[sample(1:length(permn(1:C_n)), 1)]]
    
  D1_Dn = list()
  for (c_i in 1:C_n){
    interv_targets_i = c() 
    for (node_i in 1:node_n){
      if (targets[node_i] == c_i)
        interv_targets_i = c(interv_targets_i, node_i) 
    } 
    
  
    data = rmvDAG_interventional(D_n, dag, 
                                 interv_targets=interv_targets_i,
                                 interv_type=interv_type,
                                 vb = vb,
                                 errDist="normal")

    
    nm = paste0("D", c_i)
    D1_Dn[[nm]] = data.frame(data)
  }
  return(list(D1_Dn=D1_Dn, wgt = wgt, dag= dag, targets_node_to_context= targets))
}


run_vario_rand_dags = function(runs,interv_type="soft"){
  tp = 0
  fp = 0
  fn = 0
  considered_targets =  0
  confints = list(conf_pione=sample_conf_pi_one(5, 50, vb=FALSE, version_mdl=TRUE),
                    conf_nogroups= sample_conf_nogroups(5, 50, vb=FALSE, version_mdl=TRUE),
                    conf_best=sample_conf_pi_best(5, 50, vb=FALSE, version_mdl=TRUE))
  for (seed in 1:runs){
    DAG = gen_rand_dag(interv_type=interv_type,
                       out_p=0.3,seed = seed, vb =FALSE)
    res = run_vario_rand_dag(DAG$D1_Dn, DAG$dag, DAG$targets_node_to_context, confints)
 
    tp = tp + res$counts[1]
    fp = fp + res$counts[2]
    fn = fn + res$counts[3]
    considered_targets  = considered_targets +  res$counts[4]
  }
  if(tp + 1/2*(fp+fn)> 0)
    f1 = tp / (tp + 1/2*(fp+fn))
  else 
    f1 = 0
  return(c(tp,fp,fn,f1, considered_targets))
  
}

run_vario_rand_dag = function(D1_Dn, dag,targets_node_to_context, confints = NULL, X_max_n = 4){

  
  wgt = wgtMatrix(dag, transpose = FALSE)#as(dag, "matrix")
  node_n = ncol(wgt)
  
  if(!length(confints))
    confints = list(conf_pione=sample_conf_pi_one(E_n, it_pre, vb=FALSE, version_mdl=TRUE),
                    conf_nogroups= sample_conf_nogroups(E_n, it_pre, vb=FALSE, version_mdl=TRUE),
                    conf_best=sample_conf_pi_best(E_n, it_pre, vb=FALSE, version_mdl=TRUE))
  
res = Vario_G_search(D1_Dn, paste0("X", 1:node_n), max(node_n,X_max_n), vb =FALSE) 
 
#only consider targets with at least one node
tp = 0
fp = 0
tn = 0
fn = 0
considered_targets = 0
found = matrix(0,node_n, node_n)
for (e in res$edges_pruned)
{
  if(sum(DAG$wgt[, e$node_i]>0)){
  if(DAG$wgt[e$node_j , e$node_i]>0 ) tp = tp + 1
  else  fp = fp + 1
}
  found[e$node_j, e$node_i] = 1
}
for (i in 1:5)  {
  if(sum(DAG$wgt[, e$node_i]>0)) 
    considered_targets = considered_targets + 1
   for (j in 1:5)  
     if(found[j,i]==0 & DAG$wgt[j,i] >0)
       fn = fn +1
}
return(list(counts = c(tp, fp, fn,considered_targets), found =  found, true = wgt, edgenames =  names(res$edges_pruned), res=res))

}


# Adapted from pcalg package: 
rmvDAG_interventional <-
  function(n, dag,
           interv_targets,
           interv_type, #("perfect", "soft", "shift"),
           vb,
           interv_weights=NULL,
           errDist = c("normal", "cauchy", "t4", "mix", "mixt3", "mixN100"),
           mix = 0.1, errMat = NULL, back.compatible = FALSE,
           use.node.names = !back.compatible)
  {
    ## Purpose: Generate data according to a given DAG (with weights) and
    ## given node distribution (rows: number of samples; cols: node values in
    ## topological order)
    ## ----------------------------------------------------------------------
    ## Arguments:
    ## - n     : Number of samples
    ## - dag   : Graph object containing the DAG and weights
    ## - errDist: "normal" or "mix" for pure standard normal node distribution
    ##           or mixing with standard cauchy
    ## - mix   : Percentage of points sampled from standard cauchy
    ## ----------------------------------------------------------------------
    ## Author: Markus Kalisch, Date: 26 Jan 2006;  Martin Maechler
    
    ## check input &  initialize variables
    stopifnot(is(dag, "graph"),
              (p <- length(nodes(dag))) >= 2)
    
    ##  as(.,"matrix") now {for some versions of 'graph' pkg} is 0/1
    ## weightMatrix <- t(as(dag,"matrix"))
    weightMatrix <- if(back.compatible) wgtMatrix.0(dag) else wgtMatrix(dag)

    ## check if top. sorted
    nonZeros <- which(weightMatrix != 0, arr.ind = TRUE)
    if (nrow(nonZeros) > 0) {
      if (any(nonZeros[,1] - nonZeros[,2] < 0) || any(diag(weightMatrix) != 0))
        stop("Input DAG must be topologically ordered!")
    }
    
    errDist <- match.arg(errDist)
    if(grepl("^mix", errDist))
      eMat <- function(outs) { # (n,p)
        X <- c(rnorm(n*p - length(outs)), outs)
        matrix(sample(X), nrow = n)
      }
    if(is.null(errMat)) {
      ## generate errors e_i
      errMat <-
        switch(errDist,
               "normal" = matrix(rnorm  (n*p),  nrow = n),
               "cauchy" = matrix(rcauchy(n*p),  nrow = n),
               "t4" =     matrix(rt(n*p, df = 4), nrow = n),
               "mix"    = eMat(rcauchy(round(mix*n*p))),
               "mixt3"  = eMat(     rt(round(mix*n*p), df = 3)),
               "mixN100"= eMat(  rnorm(round(mix*n*p), sd = 10)))
    }
    else { ## check & use 'errMat' argument:
      stopifnot(!is.null(dim.eM <- dim(errMat)),
                dim.eM == c(n,p), is.numeric(errMat))
    }
    if(use.node.names)
      colnames(errMat) <- nodes(dag) # == colnames(weightMatrix)
    
    ## Interventions 
    if(interv_type=="perfect"){
      for (node_i in interv_targets)
        weightMatrix[node_i,] = rep(0,ncol(weightMatrix))
      if(vb) cat("\nUpdating weight mat -- PERFECT on ", paste0(interv_targets), "\n")
      if(vb) show(t(weightMatrix))
    }
    if(interv_type=="soft"){
      stopifnot(ncol(interv_weights) == ncol(weightMatrix))
      stopifnot(nrow(interv_weights) == nrow(weightMatrix))
      
      for (node_i in interv_targets)
        for (j in 1:ncol(weightMatrix)){
          if(weightMatrix[node_i,j]!=0){
          if(weightMatrix[node_i,j]>0.5)
            weightMatrix[node_i,j] = weightMatrix[node_i,j] +0.4  
          else weightMatrix[node_i,j] = weightMatrix[node_i,j] - 0.4
        }
        }
      if(vb) cat("\nUpdating weight mat -- SOFT on ",paste0(interv_targets), "\n")
      if(vb) show(t(weightMatrix))
    }
    
    if(interv_type=="shift"){ 
      errBef = errMat
      for (node_i in interv_targets)
        errMat[,node_i] = errMat[,node_i]  + 1
      if(vb) {
        cat("\nUpdating err mat -- SHIFT on ", paste0(interv_targets), "\n")
        
        cat("\n First 5 rows of previous err mat \n")
        show(errBef[1:5,])
        cat("\n Updated to \n")
        show(errMat[1:5,])
      }
    }
    ## compute X matrix X_i
    if (sum(weightMatrix != 0) > 0) {
      X <- errMat
      for (j in 2:p) { ## uses X[*, 1:(j-1)] -- "recursively" !
        ij <- 1:(j-1)
        X[,j] <- X[,j] + X[, ij, drop = FALSE] %*% weightMatrix[j, ij]
      }
      X
    }
    else
      errMat
  }
