#' Search over all Partitions Pi for one coefficient Varphi.
#' @details Search builds a tree with depth \code{K=|E|} (number of contexts)
#' where each level corresponds to the partitions with \code{k} groups, 
#' generates partitions along the tree and evaluates score(Pi, Varphi)
#' @param Ranking cls_Ranking of partitions, updated along the way
#' @param C_history cls_Phi class for varphi, updated along the way
#' @export
tree_search  = function(Ranking, 
                        C_history, 
                        vb=FALSE,
                        debug=FALSE){
  
  if(vb) tic("(vario)       Tree ")
  c_i = C_history$ci
  C_cf = C_history$C_cf #ordered by cf value
  E_n = C_history$E_n
  E_ids = c(1:E_n) 
  
  # Candidates: splits into two groups
  E_paired = as_mat(combn(E_ids, 2))
  if(E_n == 2 ) cands = t(as_mat(E_paired))
  else{ 
    E_neighbors = adjacent_pairs(E_ids, C_cf, E_paired)
    cands = rbind(E_paired[1,E_neighbors], 
                E_paired[2,E_neighbors])
 }
  node = list(cands=cands,
              C_part=rep(1,E_n),
              Ranking=Ranking,
              C_history=C_history)
  # Recurse on candidates ------
  tree = traverse_nodes(node, debug)
  
  #One-group partition ------
  #mdl_one = C_history$mdl_one
  #cat(mdl_one)
 #  tree$Ranking$Pi_one$scores_X_C[[xi]][ci,] <<- mdl_one
#  tree$Ranking$Pi_one$score_sum <<- tree$Ranking$Pi_one$score_sum + mdl_one
  if(debug) print("Done")
  if(vb) toc() 
  return(tree)
}



traverse_nodes = function(node, debug){
  E_n = length(node$C_history$C_cf) 
  level = 1

  node = expand_node (node, debug)
  
  k = node$C_history$search_level
  while(k < E_n) { 
    
    assert(k==level+1) 
    assert(length(node$cands)>0)

    if(debug) print(paste("Best pi", out_br(clld(node$C_part, d=0))))
    #if(debug & !node$F_history$precmp) sapply(2:k, function(k) print(paste(paste("Best pi_", k, sep=""), out_br(clld(node$C_history$K_best[[paste(k)]]$C_part, d=0)))))
    if(debug) print(paste0("Done k=", paste0(k, paste0(", Node k=", k+1))))
    
    node = expand_node(node, debug) 
    k = node$C_history$search_level
    level = level+1
  }
  
  node$C_history = finalize_history_C (node$C_history)
  ##TODO node$F_history$cf_done_check(node$C_history)
  return(node)
}



expand_node  = function (node, debug)   { 
  if(debug) print(paste("    ", paste("Cur pi ",out_br(clld(node$C_part, d=0))),sep=""))
  
  cand_nodes = list()
  cands = node$cands
  #continuously updated:
  C_history = node$C_history
  Ranking = node$Ranking

  for (cand_i in 1:ncol(cands)){
    node_up = do_candidate (cands[,cand_i], node, C_history, Ranking,
                            depth="      ", debug)
    cand_nodes[[cand_i]] = list(delta=node_up$delta, C_part=node_up$C_part, cand_id=node_up$cand_id)
    C_history = node_up$C_history
    Ranking = node_up$Ranking
  }

 # cand_nodes = apply(cands, 2, function(cand_id) 
  #  do_candidate (cand_id, node, depth=paste(depth, "   "), debug))
  
  assert(length(cand_nodes)==ncol(cands), ncol(cands))  #prv cands                    
  cand_nodes = list.sort(cand_nodes, delta) 
  
  best_node_k = cand_nodes[[1]] 
  #greedy_k=F: guaranteed best at level k.
  ##if(screw) best = cand_nodes[[length(cand_nodes)]] to rev greedys order
  
  E_n = C_history$E_n
  #sanity---
  best_part_k = best_node_k$C_part
  k_up = C_history$search_level + 1
  best_part_k_check = C_history$K_best[[paste(k_up)]]$C_part
  assert(length(unique(best_part_k))==k_up)
  sapply(1:E_n, function(i) assert(best_part_k_check[i]==best_part_k[i]))

  
  k_prv = C_history$search_level
  C_history = next_lvl_history_C(C_history)
  k = C_history$search_level
  assert(k==k_prv+1) #search level: k that is done
  k_up = k+1
  #---
  
  cands = up_candidates(cands, #the previous candidates.
                        best_node_k$cand_id, #w entry cand_id for split that was actually done
                        Ranking$greedy_k, k_up, E_n)
  return(list(cands = cands,
              C_part = best_node_k$C_part,
              C_history = C_history, 
              Ranking = Ranking))
}

do_candidate = function (cand_split, node, C_history, Ranking, depth, debug) {

  k = node$C_history$search_level
  k_up = k+1
  
  E_n = C_history$E_n
  part = node$C_part
  split_existing_groups = (Ranking$greedy_k | k==1)
   
  if(split_existing_groups) #split between the datasets cand_split=c(Dm, D{m+1}) (actually only need the left one). k=1: same as all pssb part with k+1=2 groups. 
    part_up = split_group_at (part, cand_split[1])
  else 
    part_up = part_k_groups (k_up, E_n, cand_split)

  assert(length(unique(part))==k)
  assert(length(unique(part_up))==k_up)

  #Add a new Pi -----
  #Pi_history = new("cls_Pi") #next_history_K(part_up, node$C_history, E_history$precmp)
  #Pi_history$init(part_up, node$C_history, E_history$precmp)
  #-> handled by Ranking
  
  E_part = reord_C_E_part(part_up, node$C_history)
  #print(paste(depth, paste("Next", out_br(clld(part_up, d=0)))))
  #print(paste(depth, paste("Reord", out_br(clld(E_part, d=0)))))
  
  Pi_history = Ranking$edit(E_part, part_up, node$C_history)
  score = Pi_history$scores_X_C[[C_history$xi]][C_history$ci]
  
  node$C_history = up_history_C(node$C_history, part_up, Pi_history)
  node$C_history = up_history_Kbest(node$C_history, part_up, score, Pi_history)
  #TODO don't need to store in C_hist anymore.
 
  #checks------
  C_part = part_up#Pi_history$C_part
  E_part = Pi_history$E_part
  
  assert (length(unique(C_part)==k_up))
  assert (length(unique(E_part)==k_up))
  
  E_part_check = part_up[C_history$E_ord]
  E_part_check = sort_classes_lex(E_part_check)
  
  sapply(1:length(C_part), function(id)
    (part_up[id]==C_part[id]) & (E_part_check[id]==E_part[id]))
   #---
  #print(paste("pi k", out_br(clld(part_up, d=0))))
  

  if(debug) {
    print(paste("  ", paste(paste("Cand", out_b(cand_split, d=0)),out_br(clld(C_part, d=0)))))
    print(paste(depth, paste("Score", paste(paste(out_br(clld(part_up, d=0)))), fmtd(score,d=3) )))
  }

  #E_history$edit(C_history,K_history)

  
  return(list(delta=score, 
              cand_id=cand_split, 
              C_part=C_part,
              C_history=C_history, 
              Ranking=Ranking))
}



# Help: Candidates ----
up_candidates = function(cands, cand_id, greedy_k, k, N){
  
  if(greedy_k) cands_greedy(cands, cand_id)
  else cands_k_groups(k, N)
}

cands_greedy = function(cands, cand_id){
  
  #all cands resulting from splitting another group in the partition, same as: all cands not seen yet.
  cand_done = c(intersect(which(cands[1,] == cand_id[1]), 
                          which(cands[2,] == cand_id[2])))
  if(ncol(cands)==2){
    cands_up = cands[,-cand_done]
    cands_up = as.matrix(cands_up)
  } else cands_up = cands[,-cand_done]
  
  ##print(cands_up)
  return(cands_up)
}

cands_k_groups = function (k, N) {
  if(k>N) return()
  if(N==k) return(matrix(1:N, nrow=N, ncol=1))
  if(k==1) return(matrix(1, nrow=1, ncol=1))
  
  # all cands with k groups. -> all ordered k-combs of 1:N that start w 1 (= start points)
  kples = combn(1:N, k)
  cands_up = kples[, which(kples[1,]==1)]
 #print(cands_up)
  return(cands_up)
}


part_k_groups  = function(k_up, N, cand_split){
  assert(length(cand_split)==k_up)
  sapply(cand_split, function(cls) assert(cls>=1 & cls<=N))
  classes = 1:k_up
  
  sapply(1:N,function(id){
    cls_plus1 = which(cand_split>id)
    if(length(cls_plus1)) return(classes[(cls_plus1[1]-1)]) 
    else return(classes[length(cand_split)])
  })
}

split_group_at = function(part, d1){
  d2 = d1+1
  assert(d1>=1)
  assert(d2<=length(part)) 
  if(part[d1] != part[d2]) cat(part, d1, d2)
  assert(part[d1]==part[d2])
  
  part_up = sapply (1:length(part), function(i)
  { if(i<=d1) return(part[i]) else return(part[i]+1)})
  assert(length(unique(part))==length(unique(part_up))-1)
  #assert(part[1:d1] = part_up[1:d1])
  return(part_up)
}



