
#TODO only used in thesis

#' Generate synthetic data collection
#' \code{{D1, ... Dn}} with \code{{D: Y=alpha * X1+beta * X1^2 ..+ X1_m + e}}

#' @param E_n environments
#' @param D_n dataset size
#' @param X_n covariates
#' @param Xirrel_sz noise Xis not in equation Y=...
#' @param X_degrees \code{list(degX1, ... degXm)}
#' @param X_ranges \code{list(X1=list(alpha=range1,..))} where \code{range= list(c(numD_i, left_i, right_i))} st. \code{sum_(numD_i)=E_n}
#' @details  \code{X_ranges = list(list(list(c(1, -2, -1),c(4, 10, 20))))} means X1 is linear with alpha in 1 D from [-2, -1], in 4 Ds from [10, 20]
#' @return \code{D = {D_data = {D1=(x,y)_n, D2=...}, D_names={"D1",..}, X_names={"x1",..}}}
#'
generate_Collection = function (E_n,
                                D_n,
                                X_n,
                                NX_n,
                                X_degrees,
                                X_ranges,
                                perfect = FALSE,
                                out_dir = "",
                                debug = FALSE) { 
  coefs_X = sample_coefficients (X_ranges, E_n, perfect)
  
  coefs_D = assign_coefficients(coefs_X, E_n)
  
  D = gen_datasets_mv(
    E_n,
    X_n,
    D_n,
    X_degrees,
    coefs_D,
    out_f = paste(out_dir, "coefficients.txt", sep = ""),
    NXizero = NX_n,
    debug = debug
  )
  assert(length(D$D_data))
  assert(length(D$D_null)) 
  return(D)
}

## Example:
#generate_Collection(5, 1000, 1, 0, list(2), list( list( list(c(3, 1,2),c(2, 1,2 )), list( c(3, 0.1, 0.2), c(2, 0.2, 0.3)) ) ))
#X_ranges = sample_group_cfs (c(5), list(1,1), c(0,1), c(0,1),TRUE)
#generate_Collection(5,1000,2,0,list(1,1),X_ranges)







# TODO rmv-----
 
model_by_degs = function (degs_relevant,
                          degs_irrelevant,
                          X_sz,
                          Xirrelevant_sz) {
  # if(length(degs_relevant)!= X_sz |
  #   Xirrelevant_sz>0 & length(degs_irrelevant)!= Xirrelevant_sz ) stop("(model_by_degs)")
  res = list()
  for (i in 1:X_sz) {
    res[[i]] = degs_relevant[[i]]
  }
  if (Xirrelevant_sz > 0)
    for (j in 1:Xirrelevant_sz) {
      res[[X_sz + j]] = degs_irrelevant[[j]]
    }
  res
}

model_relevant = function  (degs_relevant,
                            deg_irrelevant,
                            X_sz,
                            Xirrelevant_sz) {
  degs_irrelevant = list()
  if (Xirrelevant_sz > 0)
    degs_irrelevant = rep (list(deg_irrelevant), Xirrelevant_sz)
  model_by_degs (degs_relevant, degs_irrelevant, X_sz, Xirrelevant_sz)
}
