#' Given results for each variable Y in turn, generates Varios output graph
#' Each edge X->Y is associated with partition Pi & score
#' -Pi shows intervention locations on variable Y
#' -Pi shows groups with stable mechanism (not the no-group partition), score reflects how stable it is (informative models Pi = prefer small group numbers) 


# For display
list_edges = function(graph){
  
  for (yid in 1:ncol(graph$adj)){
    by_score = list()
    
    col = graph$adj[,yid]
    ynm = graph$variables[yid]
    for(xid in 1:length(col)){
      if(col[xid]==0) next
      xnm = graph$variables[xid]
      
      yin = graph$edges_in[[yid]]
      for (i in 1:length(yin)){
        
        id = names(graph$edges_in[[yid]])[i]
        cand_id = as.integer( stringi::stri_split(stringi::stri_split(id, regex=",")[[1]][1], regex="\\(")[[1]][2])
        pi = stringi::stri_split(stringi::stri_split(id, regex="\\(")[[1]][3], regex="\\)")[[1]][1]
        if(cand_id == xid){
        score = as.numeric(yin[i], collapse="")
        by_score[[length(by_score)+1]] = list(str=
        c("(", xid, yid, ")", Vario:::renameErkAkt(xnm), "->", Vario:::renameErkAkt(ynm), "\t(", pi, ")\t",Vario:::fmtd(score, d=2), "\n"), score = score)
        }
      }
    
    }
    by_score = list.sort(by_score, score)
    for ( x in by_score) cat(x$str)
    cat("\n")
  }
  
}

update_result = function(res){
   res$Ranking_updated = list()
  Pi_cands = order_Y_by_intervention(res)
  Pi_selected = Pi_cands[[1]]$Pi
  cat(paste(Pi_selected, collapse = ""),"\n")
  for (i in 1:length(res$RESULT$Ranking)){
    entry= res$RESULT$Ranking[[i]]
    nm = names(res$RESULT$Ranking)[i]
    Pi = entry$E_part
    one_group = (length(unique(Pi)) == 1) 
    max_group = (length(unique(Pi))==length(Pi))
    if(one_group | max_group)  
      break
    
    #Require that Pi as intervention wherever Piselected has
    agrees = TRUE
    for (id in 1:length(Pi)) {
      is_isolated = (length(which(Pi_selected==Pi_selected[id]))==1)
      is_isolated_pi = (Pi[id]!=1) #(length(which(Pi==id))==1)
    if(is_isolated & !is_isolated_pi)
      agrees = FALSE
  }
  cat("   ", paste(Pi , collapse = ""), " ", agrees,"\n")
  if(!agrees) next
  res$Ranking_updated[[nm]] = entry
  
  }
  return(res)
}

order_Y_by_intervention = function(Result){
  yid = Result$y_target
  variables = Result$x_named
  Pi_cands = list()
  Ranking = Result$RESULT$Ranking
  
  #Collect Partitions
  for (entry in Ranking){
    Pi = entry$E_part
    Pi_nm = paste(Pi, collapse="")
    # Up to last relevant edge 
    one_group = (length(unique(Pi)) == 1) 
    max_group = (length(unique(Pi))==length(Pi))
    if(one_group | max_group)  
      break
  
    # For each Pi, sum scores of all edges X->Y that support Pi (Pi' at edge has superset of interventions)
    if(!(Pi_nm %in% names(Pi_cands)))
      Pi_cands[[Pi_nm]] = list(Pi = Pi, score_sum = 0, edges = list(), xnms =list())  
  }
  
  # Having collected all unique partitions, add scores of partitions supporting each one
  for (i in 1:length(Ranking)){
    entry = Ranking[[i]]
    entry_nm = names(Ranking)[i]
    Pi = entry$E_part
    Pi_nm = paste(Pi, collapse="")
    xnm = entry$xnm
    xseen =list()
  
    # Up to last relevant edge 
    one_group = (length(unique(Pi)) == 1) 
    max_group = (length(unique(Pi))==length(Pi))
    if(one_group | max_group)  
      break
     
    for (Piprime_nm in names(Pi_cands)) {
      
      
      Piprime = Pi_cands[[Piprime_nm]]$Pi  
      agrees= TRUE
      for (id in 1:length(Piprime)){
        is_isolated = (length(which(Piprime==Piprime[id]))==1)
        is_isolated_pi = (Pi[id]!=1) #(length(which(Pi==id))==1)
        if(is_isolated & !is_isolated_pi)
          agrees = FALSE
      }
      if(!agrees) next
      
      
      X_seen = (xnm %in% xseen)
      if(X_seen) next
      
      xseen[[length(xseen)+1]] = xnm
      Pi_cands[[Piprime_nm]]$score_sum = Pi_cands[[Piprime_nm]]$score_sum + entry$score_norm
      Pi_cands[[Piprime_nm]]$edges[[(length(Pi_cands[[Piprime_nm]]$edges)+1)]] = entry_nm
      Pi_cands[[Piprime_nm]]$xnms[[(length(Pi_cands[[Piprime_nm]]$xnms)+1)]] = xnm
    }}   
  
  Pi_cands = list.sort(Pi_cands, score_sum)
  return(Pi_cands)
}

renameErkAkt = function(nm)
{
  if(nm=="p44/42")   return("Erk")
  if(nm=="pakts473")   return("Akt")
  return(nm)
  
}

check_ancestral = function(graph){
  ances_Gvario = 0 # a false positive with one (or more) path(s) over one intermediate node in Gvario
  ances_Gtruth = 0 # ancestral in the true graph
  paths_fp = list()
  paths_tp = list()
  #match with Gtruth: 
  tp = 0
  tp_ances_vario = 0 #those tp that are also ancestral in Gvario (pruning by path would remove them)
  fp_ances_vario = 0 #those fp that are also ancestral in Gvario  (with propagated errors)
  fp_ances_true = 0 #.. Gtruth (not as bad as fp)

  variables = graph$variables
  Gvario  = graph$adj
  Gtruth = Vario:::consensus_graph_Yall(variables)
  
  for (yid in 1:ncol(Gvario)){
    by_score = list()
    paths_fp_y = list()
    paths_tp_y = list()
    col = Gvario[,yid]
    ynm = graph$variables[yid]
    for(xid in 1:length(col)){
      
      #set to false here
      is_ances_truth = FALSE # at least one path f at least one of the edges X->Y (edges X->Y differ only in partition and score)
      is_ances_vario = FALSE
      
      
      if(col[xid]==0) next
      xnm = graph$variables[xid]
      
      yin = graph$edges_in[[yid]]
      for (i in 1:length(yin)){
        
        id = names(graph$edges_in[[yid]])[i]
        cand_id = as.integer( stringi::stri_split(stringi::stri_split(id, regex=",")[[1]][1], regex="\\(")[[1]][2])
        pi = stringi::stri_split(stringi::stri_split(id, regex="\\(")[[1]][3], regex="\\)")[[1]][1]
        if(cand_id == xid){
          # Case: Positive edge xid -> yid, at i in edge list (different in case mult edges w differnt pi) (tp or fp)
          
          is_tp = (Gtruth[xid, yid] == 1)
          if(is_tp) tp = tp + 1
          
          scoreXZ = 0
          scoreZY = 0
          z_selected = 0
          
          for (zid in 1:length(variables) ){
            if(Gtruth[zid, yid] == 1 & Gtruth[xid, zid] == 1 ) is_ances_truth = TRUE
            if(Gvario[zid, yid] == 1 & Gvario[xid, zid] == 1 ) is_ances_vario = TRUE

            
            if(Gvario[zid, yid] == 1 & Gvario[xid, zid] == 1 ) {
              # look for associated scores here. if theres multiple options, may override by best score.
              
              scoreXZ_cand = 0
              scoreZY_cand = 0
              
              zin = graph$edges_in[[zid]]
              for (ii in 1:length(zin)){
                id = names(zin)[ii]
                xcand_id = as.integer( stringi::stri_split(stringi::stri_split(id, regex=",")[[1]][1], regex="\\(")[[1]][2])
                cand_pi1 = stringi::stri_split(stringi::stri_split(id, regex="\\(")[[1]][3], regex="\\)")[[1]][1]
                s1 = as.numeric(zin[ii], collapse="")
                if(xcand_id == xid & s1 < scoreXZ_cand) {
                  scoreXZ_cand = s1
                  pi1 = cand_pi1
                  }
         
              
              
              yin = graph$edges_in[[yid]]
              for (ij in 1:length(yin)){
                id = names(yin)[ij]
                cand_id = as.integer( stringi::stri_split(stringi::stri_split(id, regex=",")[[1]][1], regex="\\(")[[1]][2])
                cand_pi2 = stringi::stri_split(stringi::stri_split(id, regex="\\(")[[1]][3], regex="\\)")[[1]][1]
                s2 = as.numeric(yin[ij], collapse="")
                if(cand_id == zid & s2 < scoreZY_cand) {
                  scoreZY_cand = s2
                  pi2 = cand_pi2
                }
                if(cand_id == zid & xcand_id==xid){
                  path = paste(c(Vario:::renameErkAkt(variables[xid]),"->", Vario:::renameErkAkt(variables[zid]), "-> ", Vario:::renameErkAkt(variables[yid]), cand_pi1, cand_pi2,   pi,
                                 Vario:::fmtd((s1+s2) /2,d=2), "score_direct =",  Vario:::fmtd(as.numeric(yin[i]), d=2)), collapse=" ")
                  
                if(is_tp) 
                  paths_tp_y[[length(paths_tp_y)+1]] = path
                else
                  paths_fp_y[[length(paths_fp_y)+1]] = path
                
              }}
              
              }
              
              
              if(scoreZY_cand + scoreXZ_cand < scoreXZ + scoreZY ){
                scoreZY = scoreZY_cand
                scoreXZ =scoreXZ_cand
                z_selected = zid
              }
              
              ### end 
            }
          }
          
          if(tp & is_ances_vario) tp_ances_vario = tp_ances_vario + 1
          if((!tp)  & is_ances_vario) fp_ances_vario = fp_ances_vario + 1
          if((!tp) & is_ances_truth) fp_ances_true = fp_ances_true + 1
          score = as.numeric(yin[i], collapse="")
          if(is_ances_vario) ances_path = paste(c("X-Z->Y ",Vario:::renameErkAkt(variables[z_selected]), "\t(", Vario:::fmtd(scoreXZ,d=2), ",",Vario:::fmtd(scoreZY,d=2), ")"), collapse="")
           else ances_path = ""
          by_score[[length(by_score)+1]] = list(str=
                                                  c("(", xid, yid, ")", Vario:::renameErkAkt(xnm), "->", Vario:::renameErkAkt(ynm), "\t(", pi, ")\t",Vario:::fmtd(score, d=2), 
                                                    "\t", "(",ok(is_tp), ok(is_ances_vario), ")", ances_path, 
                                                    "\n"), score = score)
        }
      }
      
    }
    by_score = list.sort(by_score, score)
    for ( x in by_score) cat(x$str)
    cat("\n")
    paths_fp[[yid]] = paths_fp_y
    paths_tp[[yid]] = paths_tp_y
  }
  # todo fix f double edges : cat("TP:", tp,"TP ances Vario:", tp_ances_vario,"FP ances Vario:", fp_ances_vario, "FP ances true:", fp_ances_true)
  return(list(fp=paths_fp,tp=paths_tp))
}

ok = function(bool) {
  if(bool) return ("✓")
   else return("-")
}
