
# invariance hypoth testing -------


#' Test Coefficients for invariance 
#' @param data_E1_En Datasets, list: \code{E->(X_e, Y_e)}
#' @param regress_E1_En Regression coefficients in contexts, list: \code{E->varphi(X_e->Y_e)}
#' @param X_degs  X->polynomial degree in regression
#' @param X_nms  X->name
#' @details - Chow test on pairs (ftest)
#'- testing coefficients directly (pairs, one-to-mean)
#' - permutation testing;
#' Adjusting alpha levels using  bonferroni multiplicity correction
#
hypoth_test_invariance = function(data_E1_En,
                                  regress_E1_En,
                                  X_degs, X_nms,
                                  alpha=0.05){
  
  # Pairs of contexts to test 
  E12 = as.matrix(combn(1:length(data_E1_En), 2))
  E12_nms = apply(E12, 2, function(i) paste(i,collapse=""))
  
  pair_n = ncol(E12)
  pair_alpha = p.adjust(rep(alpha, pair_n))
  
  #Hypoth testing: coefficients of pairs equal
  #TODO: also on mean coef. vs each one dataset
  htests = apply(E12, 2, function(pair) {
    E1 = pair[1]
    E2 = pair[2] 
    n1 = nrow(data_E1_En[[E1]])
    n2 = nrow(data_E1_En[[E2]])
    
    data_12 = pool_D1_D2(data_E1_En, E1, E2)
    R12 = regress_Ei (data_12 , X_degs, X_nms)$rsq 
    R1 = regress_E1_En[[E1]]$rsq 
    R2 = regress_E1_En[[E2]]$rsq 
    
    ftest = ftest(R1, R2, R12, n1, n2, 
                  regress_df(X_degs),
                  pair_alpha)
    
    M1 = regress_E1_En[[E1]]$m
    cfM2 =  regress_E1_En[[E2]]$m$coefficients
    
    htest = htest (M1, cfM2, pair_alpha) 
    return(c(ftest=ftest, htest_pval=htest))
  })
  
  #stop()
  #TODO pooled mean or cf mean?
  cf_means = regress_Ei (pool_collection(data_E1_En),
                         X_degs, X_nms)$m$coefficients
  
  htest = sapply(1:length(data_E1_En), function(Ei){
    Mi = regress_E1_En[[Ei]]$m
    htest = htest (Mi, cf_means)
  })
  
  
  test_result = list(chow_pairs=htests[1,], 
                     htest_pairs=htests[2,],
                     htest_coefs=htest ) 
  return(test_result)
}


#help ------
pool_D1_D2 = function(D1_DN, i1, i2){
  C = D1_DN[[i1]]
  Cr = D1_DN[[i2]] 
  C = rbind(C, Cr)
  return(C)
}
pool_collection = function(D1_DN){
  C = D1_DN[[1]]
  Cr = D1_DN[2:length(D1_DN)]
  for (D in Cr)
    C = rbind(C, D)
  return(C)
}

ftest = function(r1, r2, r12, n1, n2, p, alpha=0.05){
  chow_value = ((r12-r1-r2)/ p)/((r1+r2)/(n1+n2-2*p))
  #print(chow_value)
  return(chow_value <= critf(p, n1+n2-2*p, alpha))
}
critf = function(df1, df2, alpha ) { 
  return(qf(alpha, df1, df2, lower.tail = FALSE))
}

#Hypothesis: equal coefficients
htest = function(m, cfs, alpha=0.05){
 
  cfs=cfs##[-1] intercept removed  
  cf_nms= names(coef(m))##[-1]
  assert(length(cfs)==length(cf_nms))
  return(FALSE) #TODO f server as car n working. the other invariance tests suffice anyway.
  
  h = linearHypothesis(m, cf_nms, cfs) 
  return(h$`Pr(>F)`[2]>=alpha) #not rejected
}

