#Regression formula

# Y ~ poly(X1, c1) + poly(X2, c2) ... -----

#' Multivariate regression 
#' @param data (X=(x1,..xm),Y)
#' @param X list: names X in formula
#' @param degs list: degrees X
#' @return lm(y~x1+I(x1^2)+...)
#' @export 
lm_mv = function (data, X, degs) {
  formula = reformulate(termlabels = term_lables(X, degs), response = "y")
  #out(term_lables(X, degs), f="out/CollectionXY/check-regrformula.txt")
 for (col in 1:ncol(data))
   if(all(is.na(col))) stop() 
  m = lm(formula, data = data)

  
  return (list("m"=m, "form"=formula))
}

#' Bayesian Multivariate regression 
#' @param data (X=(x1,..xm),Y)
#' @param X list: names X
#' @param degs list: degrees X
#' @return stan_glm(y~x1+I(x1^2)+...)
#' @export 
blm_mv = function (data, X, degs, prior = NULL) {
  formula = reformulate(termlabels = term_lables(X, degs), response = "y")
  if (length(prior))
    blm = stan_glm(formula, data = D,
                   family = gaussian(link = "identity"),
                   prior = prior,
                   seed = 12345)
  else blm = stan_glm(formula, data = D,
                      family = gaussian(link = "identity"),
                      seed = 12345)
  return (blm)
}


#' Extract p value from lm
#' @param model lm 
#' @return p value
#' @export
lm_p = function (model) {
  if (class(model) != "lm") stop()
  #fstatistic = summary(model)$fstatistic
  #pvalue = pf(fstatistic[1], fstatistic[2], fstatistic[3], lower.tail=F)
  #attributes(pvalue) = NULL
  pvalue = summary(model)$coefficient[,"Pr(>|t|)"]
  return(pvalue)
}



#' Paste together regression equation,
#' by #https://stackoverflow.com/questions/4951442/formula-with-dynamic-number-of-variables
#' @param X X names, eg. list("x1", "x2")
#' @param X X degs, eg. list(q, lin)
#' @return equation eg. y ~ x1 + x1^2 + x2 
#' @export
term_lables = function (X, degs,
                        include_intercept = TRUE){
  nms = c()
  for (i in 1:length(X)){
    Xi_nm = X[[i]]
    Xi_deg = degs[[i]]
    if(Xi_deg>10) stop(Xi_deg)
    nms = c(nms,Xi_nm)
    deg = 2
    while(deg <= Xi_deg){
      nms = c(nms, sc("I(",Xi_nm, "^", deg, ")"))
      deg = deg + 1
    }
  }
  if(include_intercept) nms = c(nms, "0")
  return(nms)
}


# 
# term_labels_altern = function(X, degs){
#   assert(!(10 %in% degs))
#   assert(length(X)==length(degs))
# 
#   form = c()
#   for (i in 1:length(X)){
#     nm_i = X[[i]]
#     deg_i = degs[[i]]
# 
#     form_i = sapply(1:deg_i, function(deg){
#       xi_to_deg = sapply(1:deg, function(d) {
#         if(d==1) return(nm_i) else return(paste("*", nm_i))})
#       return(paste(xi_to_deg, collapse = " "))
#     })
#     form = c(form, form_i)
# 
# 
#   }
#   return(form)
# }
# 
# 
# term_lables_upto_cubic = function (X, degs){
#   nms = c()
#   for (i in 1:length(X)){
#     Xi_nm = X[[i]]
#     Xi_deg = degs[[i]]
#     nms = c(nms,Xi_nm)
#     if(Xi_deg!="lin") {
#       nms = c(nms,sc("I(",Xi_nm, "^2)"))
#     }
#     if(Xi_deg=="cub") {
#       nms = c(nms,sc("I(",Xi_nm, "^3)"))
#     }
#     
#   }
#   return(nms)
# }



# # h------
# test_ranges = function(i){l = list()
#  for (j in 1:i){
#    l[[length(l)+1]] = list(c(5, 100/(10^j), 200/(10^j))) }
#  return(list(l))
# }
# 
# 
 i_to_coefnm = function (i) { 
   res = ifelse(i==1, "alpha", ifelse(i==2,  "beta", 
                                      ifelse(i==3, "gamma", fmtd(i,0)) ))
   return (res)
}
# 
# i_to_deg = function (i) { #used?
#   res = ifelse (i==1, "lin", ifelse(i==2, "q", ifelse (i==3, "cub",  fmtd(i,0))) )
#   return (res)
# }
# 
# 
# # i_to_coefnms = function (deg){
# #   switch(deg, 
# #          lin = { todo = list("alpha") },
# #          q = { todo = list("alpha", "beta") },
# #          cub = { todo = list("alpha", "beta", "gamma") },)
# #   assertlen(todo, "(i_to_coefnms) not lin,q,cub")
# #   return (todo)
# # }
# 
# deg_to_i = function (deg){
#   res = ifelse (deg =="lin", 1, ifelse( deg=="q", 2, ifelse (deg == "cub", 3, as.numeric(deg))))
#   return (res)
# }
# 
