#OLS Regression -----

#' OLS regression
#' @param data_E1_En list: \code{E -> (X_e, Y_e)}
#' @param X_degs c (\code{X -> }regression degree)
#' @export
regression_E1_En = function (data_E1_En, 
                             X_degs,
                             skip_invariance=FALSE){ 
  E_n = length(data_E1_En)
  X_n = length(X_degs)

  # for lm formula
  X_nms = lapply(1:length(X_degs), function(i) 
    return(paste0("x", i))) 
   
 
  regress_E1_En = lapply(1:E_n,  function (i)  
    regress_Ei (data_E1_En[[i]], X_degs, X_nms))
   
  
  # Assign coefficient (and lconf, rconf) matrix to each Xi
  cfs_X_E1_En = list()
  lconf  = list()  
  rconf  = list() 
  pval = list()
  offset = 0 # index of cfs relevant for cur Xi
  
  
  for(xi in 1:X_n){
    x = X_nms[[xi]]
    result = regress_X(xi, offset,
                       regress_E1_En,
                       X_degs, X_nms)
    cfs_X_E1_En[[x]] = result$cfs
    lconf[[x]] = result$lconf
    rconf[[x]] = result$rconf
    pval[[x]] = result$pval
    offset = result$offset
  } 
  names(regress_E1_En) = paste0 ("D", 1:length(regress_E1_En))
  regress_E1_En<<- regress_E1_En
  d1_dn <<- data_E1_En
 
 
  # Test for perfect invariance here
 if(!skip_invariance) htest_invariant = hypoth_test_invariance(data_E1_En,
                          regress_E1_En,
                          X_degs, X_nms)
  else htest_invariant=NULL
    
  return(list(regress_E1_En=regress_E1_En,
              htest_invariant=htest_invariant,
              cf_mats=list(cfs=cfs_X_E1_En,
                           lconf=lconf, 
                           rconf=rconf,
                           pval=pval)))
  
}

# Regression  Parameters -----
#' Relevant regression parameters for one dataset/environment in collection
#' @param data_Ei \code{(X1_i, ...Xm_i, Y_i)} in context E_i
#' @param X_degs degrees of  \code{X1,...Xm}
regress_Ei = function (data_Ei, X_degs, X_nms) { 
  m = lm_mv(data_Ei, X_nms,X_degs)$m 
  
  # no intercept:
  return(list("cfs"= m$coefficients, #[-1],
              "conf"=as_mat(confint(m)),  # [-1,]),
              "rsq"=sum(m$residuals^2),
              "p"=lm_p(m), #[-1],
              "m"=m))
}

#' Relevant regression coefficients for one Xi 
regress_X = function (xi, offset,
                      regress_E1_En,
                      X_degs, X_nms){
  xnm = X_nms[[xi]]
  xdeg = X_degs[[xi]]
  D_n = length(regress_E1_En)
  
  #remove deg if insignificant in all environments and for all X 
  # TODO ? if(!deg_by_lasso) 
  xdeg = xdeg#regress_rmv_insig(regress_E1_En)
  #print(xdeg)
  init_mat = matrix(0, nrow = D_n, ncol = xdeg)
  cfs = init_mat
  lconf = init_mat
  rconf = init_mat
  pval = init_mat
  
  for(e in 1:D_n){
    for(d in 1:xdeg) {
      cfs[e,d] = regress_E1_En[[e]]$cfs[d+offset]

      lconf[e,d] = regress_E1_En[[e]]$conf[d+offset,1]
      rconf[e,d] = regress_E1_En[[e]]$conf[d+offset,2]
      pval[e,d] = regress_E1_En[[e]]$p[d+offset]
    }}
  offset = offset + xdeg
 
  return(list(offset=offset,
              cfs = t(cfs),
              lconf=t(lconf),
              rconf=t(rconf),
              pval=t(pval)))
}

regress_df = function(X_degs)
  return(sum(X_degs)+1) #todo insig. remove?

regress_rmv_insig = function(regress_E1_En){
  mx_sig= 1
  for(regress_Ei in regress_E1_En){
    if(length(which(regress_Ei$p<0.05)))
      mx_sig=max(mx_sig, max(which(regress_Ei$p<0.05)))
  }
  return(mx_sig)
}
