#OLS Regression -----

#' OLS regression
#' @param data_E1_En list: \code{E -> (X_e, Y_e)}
#' @param X_degs c (\code{X -> }regression degree)
#' @export
regression_piecewise_E1_En = function (data_E1_En, 
                                       X_degs){ 
  E_n = length(data_E1_En)
  X_n = length(X_degs)
  
  # for lm formula
  X_nms = lapply(1:length(X_degs), function(i) 
    return(paste0("x", i))) 
  
  # For spline regression: Select uniform knots which will be the same in all context-regressions, f.e. Y-quantiles overall
  #data_pooled = pool_collection(data_E1_En)
  #dy = data_pooled[,ncol(data_pooled)]
  #qy = quantile(dy)
 # med = mean(sapply( 1:E_n, function(ei) 
 #   return(median(data_E1_En[[ei]][,ncol(data_E1_En[[ei]])]))
 # ))
  SUBS<<- list()
  regress_E1_En_pieces = lapply( 1:E_n, function(ei){
    #data_pooled = pool_collection(data_E1_En)
    data_i = data_E1_En[[ei]]
 dy = data_i[,ncol(data_i)]
 med = median(dy)
    #qy = quantile(dy,p=c(0.25,0.5,0.75))
    d_sub_1 = data_i[which(data_i$y <=med ),]
     d_sub_2 = data_i[  which(data_i$y >=med  ) ,]  
       
   # d_sub_3 = data_i[which(data_i$y >qy[2]),]
    #d_sub_3 = data_i[intersect(which(data_i$y <=qy[3]), which(data_i$y >qy[2])),] 
    #d_sub_4 = data_i[which(data_i$y >qy[4]), ] 
    SUBS[[ei]] <<- list(d_sub_1, d_sub_2)
    #, d_sub_3, d_sub_4)
    m_1 = regress_Ei(d_sub_1, X_degs, X_nms)
    m_2 = regress_Ei(d_sub_2, X_degs, X_nms)
   # m_3 = regress_Ei(d_sub_3, X_degs, X_nms)
   # m_4 = regress_Ei(d_sub_4, X_degs, X_nms) 
    return(list(m_1, m_2)) #, m_3, m_4))
  })
    # Assign coefficient (and lconf, rconf) matrix to each Xi
    # Coefficients at all knots concatenated
    cfs_X_E1_En = list()
    lconf  = list()  
    rconf  = list() 
    pval = list()
    offset = 0  
    
    # Extract coefficients of X
    for(xi in 1:X_n){
      x = X_nms[[xi]]
      result = regress_piecewise_X(xi, offset,
                                   regress_E1_En_pieces,
                         X_degs, X_nms)
      cfs_X_E1_En[[x]] = result$cfs
      lconf[[x]] = result$lconf
      rconf[[x]] = result$rconf
      pval[[x]] = result$pval
      offset = result$offset
    } 
    for (piece in regress_E1_En_pieces)
      names(piece) = paste0("Q", 1:length(piece))
    names(regress_E1_En_pieces) = paste0 ("D", 1:length(regress_E1_En_pieces))     
    
  
  
  # Test for perfect invariance here
 # htest_invariant = hypoth_test_invariance(data_E1_En,
         #                                  m_1,
                  #                         X_degs, X_nms)
  #TODO
  htest_invariant = list(chow_pairs=list(FALSE), 
                         htest_pairs=list(FALSE),
                         htest_coefs=list(FALSE)) 
  
  return(list(regress_E1_En=regress_E1_En_pieces,
              htest_invariant=htest_invariant,
              cf_mats=list(cfs=cfs_X_E1_En,
                           lconf=lconf, 
                           rconf=rconf,
                           pval=pval)))
  
}
 

#' Relevant regression coefficients for one Xi 
regress_piecewise_X = function (xi, offset,
                                regress_E1_En_pieces,
                                X_degs, X_nms){
  xnm = X_nms[[xi]]
  xdeg = X_degs[[xi]]
  pieces =  length(regress_E1_En_pieces[[1]])
  D_n = length(regress_E1_En_pieces)
   
  init_mat = matrix(0, nrow = D_n, ncol = xdeg*pieces)
  cfs = init_mat
  lconf = init_mat
  rconf = init_mat
  pval = init_mat
  
  for(e in 1:D_n){
    col_ct = 1
  for(m in regress_E1_En_pieces[[e]]){
        for(d in 1:xdeg) {
      cfs[e,col_ct] = m$cfs[d+offset]
      
      lconf[e,col_ct] = m$conf[d+offset,1]
      rconf[e,col_ct] = m$conf[d+offset,2]
      pval[e,col_ct] = m$p[d+offset]
        }
    col_ct = col_ct + 1
  }
    col_ct = col_ct + 1
    }
  offset = offset + xdeg
  
  return(list(offset=offset,
              cfs = t(cfs),
              lconf=t(lconf),
              rconf=t(rconf),
              pval=t(pval)))
}