
# TODO unclutter; gen_collection only used in thesis

# Samples everything needed for gen-collection

#' Sample coefficients
#' @param X_c_ranges = \code{list_Xi(list_cj(list_dn(entry(Xi,cj,gn))))},
#' \code{entry(Xi,cj,gn)=c(nd, l, r)} where \code{nd} number datasets in group \code{gn}, \code{[l,r]} interval group cfs
#' @param E_n environments
#' @param perfect for precomputing
#' @export
sample_coefficients = function (X_c_ranges, E_n,
                                perfect = FALSE) {
  
  lapply (X_c_ranges, function (c_ranges) { #length(c_ranges)=degree of current X
    lapply (c_ranges, function (ranges) {
      if (sum(sapply(ranges, function(range)
        return(range[1]))) != E_n) stop("range")
      
      unlist(lapply(ranges, function(range) {
        nD = range[1]
        r = range[-1]
        
        if(E_n == nD | perfect) # Pi one 
          return(rep(mean(r), nD))
        #return(rnorm(nD, mean = mean(r), sd = 1))
        
        #unif sampling
        return(sample_real (r, nD))
        #gaussian, works too, sd??
        # rnorm(nD, mean = mean(r), sd = sd(c(r[2]/2,r[1]/2))) #TODO does sd=intra range make sense?
        
      }))
    })
  })
}

assign_coefficients = function (X_c_coefficients, D_n) {
  lapply(1:D_n, function(di) {
    lapply(X_c_coefficients, function(c_coefficients) {
      unlist(lapply(c_coefficients, function(coefficients) {
        as.numeric(coefficients[di])
      }))
    })
  })
  
}



#' Sample X: degree, groups, ranges 
sample_X = function(config, shape) {
  degs = c(config$deg_mn, config$deg_mx)
  eachX = 1:(config$X_n) 
  
  X_degs = lapply(eachX, function (xi)
    sample(degs, size = 1)) 
  ##inter = rescale_dist_params (config$inter, length(unique(shape)))
  inter = config$inter #TODO
  #Each group's coefficient ranges
  X_group_cfs = sample_group_cfs (shape,
                                  X_degs,
                                  inter,
                                  config$intra,
                                  #, X_group_rngs)
                                  clean = FALSE,
                                  config = config) 
  return(list(degrees = X_degs,
              group_cfs = X_group_cfs))
}


#Sample group for pi --------

sample_group_cfs = function(shape, X_degrees, inter, intra, clean=FALSE, config=NULL){
  
  eachX = seq_along(X_degrees)
  
  lapply(eachX, function(xi) {
    deg = X_degrees[[xi]]
    if(deg>10) stop(deg)
    
    lapply(1:deg, function(ci){ 
      return(sample_group_cf(ci, shape, inter, intra, clean, config))})
  })
  
} #result: ranges = list(list(rangeX1_cf1.., rangeX1_cf2..), list(rangeX2...))

#same format as sample group cfs
sample_pi_one_cfs = function(E_n, rng){
  return(list(list(list(c(E_n, rng)))))
}
#Random range: sample_group_cfs(c(E_n), c(1), c(5,20), c(0,1), config=list(bad_cfs=0 , bad_envs=0))
#Given range: sample_pi_one_cfs(E_n, rng)

sample_group_cf = function (ci, shape, inter, intra, clean, config=NULL){ 
  
  if(!clean) { #only reference data is clean
    screw= ((config$bad_cfs>0 & ci<=config$bad_cfs) | config$bad_envs>0)
    if(screw){
      mess = mess_with_cfs(ci, config, shape, inter, intra)
      shape = mess$shape
      inter = mess$inter
      intra = mess$intra
      ###print(paste("     ", paste(fmtd(ci,0), paste("shape", out_br(clld(shape, d=0))))))
    }
  }
  
  
  
  cf_rng = c(1,1000)
  cf = i_to_coefnm(ci) 
  z = sample.int(9, size=1)
  k = sample.int(2, size=1)-3 #to range from -3 to 3
  
  
  #scale does not matter f my score, but for regr errors -> do not want to scale inter in one iteration to the same as intra in another iteration
  distinguish_5_1=TRUE
  if(!distinguish_5_1)
    scale = z*(10^k)
  else {
    scale = 10^(-(sample.int(5, size=1)-2)) # 10^(-(ci-2)) 
    cf_rng = c(10,100)#*scale
  }
  
  
  cf_rng_i = cf_rng * scale
  start = sample_real(cf_rng_i, 1)
  
  group_rngs = sample_group_dists(shape, inter, intra)
  
  
  #n gaps between members of previous groups (f each group, sz-1 gaps)
  ei = 0
  each_group = 1:length(shape)
  
  cf_groups = list()# lapply(each_group, function (gi) { 
  for(gi in each_group){
    group_rng = 
      range_next_group(shape, group_rngs, start, gi, ei, scale) # -> format: c(num_el_group, min_possb_cf, max_pssb_cf)
    ei = ei + shape[gi]-1 # gaps
    
    cf_groups[[length(cf_groups)+1]] = group_rng
  }
  return(cf_groups)
} 


# Simplified -----
sample_group_cfs_same_range = function(shape, #X_shapes, 
                                       X_degrees, X_group_rngs){
  
  eachX = 1:length(X_degrees)
  lapply(eachX, function(xi) {
    #shape = X_shapes[[xi]]
    deg = X_degrees[[xi]]
    group_rngs = X_group_rngs[[xi]]
    
    if(deg>10) stop(deg)
    
    lapply(1:deg, function(ci){ 
      return(sample_group_cf(ci, shape, group_rngs))})
  })
  
} #result: ranges = list(list(rangeX1_cf1.., rangeX1_cf2..), list(rangeX2...))


sample_group_cf_same_range = function (ci, shape, group_rngs){
  cf_rng = c(1,1000)
  cf = i_to_coefnm(ci) 
  scale = 10^(-(ci-1))
  cf_rng_i = cf_rng * scale
  #start does not matter much
  start = sample_real(cf_rng_i, 1) 
  #number of gaps between in-group elements seen in previous groups (for each group, sz-1 gaps)
  ei = 0
  each_group = 1:length(shape)
  
  cf_groups = list()# lapply(each_group, function (gi) {  #samples each group in partition
  for(gi in each_group){
    group_rng = 
      range_next_group(shape, group_rngs, start, gi, ei, scale) # -> format: c(num_el_group, min_possb_cf, max_pssb_cf)
    ei = ei + shape[gi]-1 # gaps
    
    cf_groups[[length(cf_groups)+1]] = group_rng
  }
  return(cf_groups)
} 


#' #Noisy  -----
#' 
#' mess_with_cfs = function(ci, config, shape, inter, intra){
#'   
#'   if(config$bad_cfs>0){
#'     #Either just a noisy group:
#'     if(config$bad_cfs_uncertainty)
#'       inter = c(0,2)
#'     
#'     #Entirely different partition:
#'     if(config$bad_cfs_random)
#'       shape = random_different_shape (shape)
#'     
#'     return(list(shape=shape, 
#'                 inter=inter,
#'                 intra=intra))
#'   }
#'   
#'   if(config$bad_envs>0){
#'     if(config$bad_envs>1) stop("n implem")
#'     #One env is noisy (in given shape, env is isolated (-> just choose the first one wo group). for cf i, put it into ith group)
#'     #In this case, only use shapes that have an isolated set!!
#'     
#'     if(config$bad_envs_allaround) shapes = move_env_around_groups (shape)
#'     else shapes = move_env_between_groups (shape)
#'     
#'     #select one confusing shape, does it work then??
#'     shapes = shapes[[sample(1:length(shapes), 1)]]
#'     
#'     #include the shape itself to avoid special case that greedy fails- implement cmopletely exhaustive version!
#'     #shapes[[length(shapes)+1]] = shape
#'     shapes = list(shapes, shape)
#'     circle_through_shapes = ci%%length(shapes) + 1
#'     shape = shapes[[circle_through_shapes]]
#'     
#'     return(list(shape=shape, 
#'                 inter=inter,
#'                 intra=intra))
#'   }
#'   #Alternative to artificial partition: change dataset size or noise, TODO.
#'   
#'   stop()
#' }
#' 
#' 
#' 
#' 
#' 
#' 
#' 

# Distances --------
sample_n = function(range, i){
  offset = range[1]-1
  return(sample.int(range[2]-offset, i) + offset)
}
#' Sample cf distances from one dataset Di to the next
#' @param shape cf groups: \code{x_i | i=1..n} means \code{n} groups, num elements \code{x_i}
#' @param inter_rng allowed dist between groups
#' @param intra_rng allowed dist between elements
#' @details \code{sample_group_dists(c(2, 3, 1), c(10,20), c(.1,.9)) = {"inter"=c(12, 19, 14), "intra"=c(0.3, 0.4, 0.7)} =
#' = {"inter"=c(start-G1, G1-G2, G2-G3), "intra"=c(G1.1-G1.2, G2.1-G2.2, G2.2-G2.3)}}
#'
sample_group_dists = function(shape, inter_rng, intra_rng) {
  inter_dist = c()
  intra_dist = c()
  for (s in shape) {
    inter_dist = c(inter_dist, sample_n(inter_rng, 1))
    if(s > 1) intra_dist = c(intra_dist, sample_real(intra_rng, (s-1)))
  }
  return(list("inter"=inter_dist,
              "intra"=intra_dist))
}


#' Actual cf ranges per group, given neighbor distances
#' @param shape cf groups: \code{{x_i | i=1..n}} means \code{n} groups w \code{x_i} elements
#' @param ranges cf distances from one dataset Di to the next from (\code{\link{sample_group_dists}})
#' @param start smallest
#' @param gi group num (from smallest)
#' @param ei between-element sum so far (counts gaps btw elements from groups so far, = sum(sz prev groups) - no. prev groups)
#' @param scale \code{10^i}, i: deg of coefficient
#' @return \code{c(group_sz, left, right) }
#' @details \code{c(group_sz, left, right) } format can be used by \code{\link{generate_Collection}}
range_next_group = function(shape, ranges, start, g_i, e_i, scale, debug=FALSE){ #fn samples coefficients for all D, for ONE group (in partition, per coef, per Xi)

  # how many distances/gaps btw elements of prev groups: e_i (can be zero if singleton groups before that)
  intra_group_sofar = ifelse(e_i == 0, 0, sum(ranges$intra[1:e_i]))
  if(debug) print(paste("- sofar in: ", intra_group_sofar))

  # gaps beween groups so far: g_i. (start<->G1 counts as the first)
  inter_group_sofar = sum(ranges$inter[1:g_i])
  if(debug) print(paste("- sofar btw: ", inter_group_sofar))

  # gaps between elements we need for this group
  e_n = shape[g_i] - 1
  g_n = shape[g_i]


  inter_group_now = ifelse(e_n==0, 0,
                           ifelse(e_n==1, ranges$intra[(e_i+1)],
                                  sum(ranges$intra[(e_i+1):(e_i + e_n)])))
  if(debug) print(paste("- elem btw: ", inter_group_now))
  L = start + inter_group_sofar + intra_group_sofar
  R =  L + inter_group_now
  range_group = c(L, R)
  c(g_n, range_group * scale)
}


test_sample_group_dists = function(shape=c(1,1,4,2,1,3) , inter=c(20,100), intra = c(0,10), debug=FALSE) {
  range = sample_group_dists(shape, inter, intra)
  if(debug) print(paste("- inter: ", range[1]))
  if(debug) print(paste("- intra: ", range[2]))
  return(test_range_next_group(shape, range, debug=debug))
}


test_range_next_group = function(shape, ranges, gi=0, ei=0, start=0, debug=FALSE){
  res=list()
  for (s in shape){
    gi = gi + 1
    res[[length(res)+1]] = range_next_group (shape, ranges, start, gi, ei, 1, debug=debug)
    ei = s + ei - 1
  }
  return(res)}

