


# TODO unclutter

# Confidence ----
#' Samples collections for Pi with no groups 
#' -> score confidence at each k compared to random
#' @param E_n environments
#' @param it_n runs of Vario
#' @return score confidence intervals 2:ks
sample_conf_nogroups = function(E_n, it_n, vb, version_mdl) {
  scores_k = matrix(ncol = E_n-1,
                    nrow = it_n)
  #print("Confints No Groups")
  
  if(vb) tic(paste0(paste0(paste0("(vario) ", it_n), " many runs")))
  
  for (it in 1:it_n){
    
    R_kmax = simulate_pi_nogroups(E_n, version_mdl)
    for(k in 2:E_n){
      assert(paste(k) %in% ls(R_kmax$Pi_kbest))
      R_k = R_kmax$Pi_kbest[[paste(k)]]
      scores_k[it,k-1] = R_k$score_sum
    }
  }
  
  conf_k = apply(scores_k, 2, cint)
  #print(conf_k) 
  if(vb) toc()
  return(conf_k)
}


# One run
simulate_pi_nogroups = function(E_n, version_mdl){
  
  # Sample unif coefficients
  rng = c(1, sample_real(c(1,100), 1))
  cf_range = sample_pi_zero_cfs(E_n, rng)
  deg = c(1) #later cmp to sccore per coefficient
  
  # Data collection
  D = generate_Collection_null(E_n, cf_range)
  
  # Vario result
  R_nogroup = Vario_Pi_search(D$D_data, deg, init_config_precmp(version_mdl=version_mdl))
  
  # Cfs: print(R_nogroup$C_histories$`1-1`$E_cf)
  return(R_nogroup)
}



# returns eg. for rng=1:6, En = 5: 1st cf in (1,2), ... 5th in (5,6).
sample_pi_zero_cfs = function(E_n, rng){
  splt = (max(rng)-min(rng))/E_n
  res = list()
  res[[1]] =list()
  for(e in 1:E_n){
    
    res[[1]][[e]] = c(1, (min(rng)+((e-1)*splt)), (min(rng)+e*splt))
  }
  
  return(list(res))
}

# Distance plot ----
# sample coefficients f any partitions and find distances
 

sample_interdist = function(K, k, it_n, nice_samples, version_mdl){

  D_n = 1000
  E_n = K
  X_n = 1 
  NX_n = 0
  deg = 1  
  rng = c(1, sample_real(c(1,100), 1))
  cf_range = sample_pi_zero_cfs(E_n, rng)
  scores_k = matrix(ncol = E_n-1,
                    nrow = it_n)
  inter_dist_k = matrix(ncol = E_n-1,
                        nrow = it_n)
  config = list(deg_mn  =deg, deg_mx = deg, X_n  = X_n, NX_n = NX_n, intra = c(0,1), inter= c(10, 10),
                bad_cfs =0 , bad_envs = 0, verbose = FALSE, Pi_K =k, E_n =E_n, Pi_k1=k,Pi_max_per_k=100)
  shapes = enumerate_pi(config)
  
  for (it in 1:it_n){
    
    shape = shapes[sample.int(length(shapes), size = 1)][[1]]  
  X = sample_X(config, shape) 
  
  # Generate data 
  if(nice_samples)
  D = generate_Collection(E_n, D_n, 
                          X_n, NX_n,
                          X$degrees, 
                          X$group_cfs) 
  
  else 
    D = generate_Collection_null(E_n, cf_range)
  
  R_vario = Vario_Pi_search(D$D_data, deg, init_config_precmp(version_mdl))
  
  # Consider best pi at each k
  for(k in 2:E_n){
    assert(paste(k) %in% ls(R_vario$Pi_kbest))
    R_k =  R_vario$Pi_kbest[[paste(k)]]
    scores_k[it,k-1] = R_k$score_sum
    inter_dist_k[it,k-1] = logg(R_k$inter_dist)
  } 
  }
  
  conf_scores_k = apply(scores_k, 2, cint)
  conf_dist_k =  apply(inter_dist_k, 2, cint)
  
  return(list(scores_k=scores_k,
              conf_scores_k=conf_scores_k,
              inter_dist_k=inter_dist_k,
              conf_dist_k=conf_dist_k))
}
