
#' Pistar vs. Pi returned by method, accuracies on environments, groups, pi
#' @param shape group sizes in pistar 
#' @param pi partition returned by method
#' @return pi match, TP,FP,TN,FN, cluscomp; pi matches at each k and at each pi
#' 
match_pi_to_shape = function (shape, pi) {
  assert(sum(shape)==length(pi))
  pi_star = shape_to_part(shape)
  
  ###decisions_pi_k = match_pi_k(pi_star, pi) #partitions exact match
  decisions = pair_contingency_shape_pi(pi_star, pi) #pairs of envs correctly assigned
  pi_match =  decisions$pi_match
  
  #ari =  adjustedRandIndex(pi_star, pi)
  
  clus = list()#clustComp(pi_star,pi)
  
  decisions = c(decisions, it=1) 

  #add these up later to avg accuracies for all runs / for specific k etc.
  return(list(pi_match=pi_match,
              decisions = decisions,
              clus = clus))
}



match_clus = function(shape, pi){ # omit clustering accuracy if E_n <=3.
  tryCatch(
    expr = {
      res = clustComp(pi_star,pi)
    },
    error = function(e){ 
      res = list()
      # (Optional)
      # Do this if an error is caught...
    },
    warning = function(w){
      # (Optional)
      # Do this if an warning is caught...
      res = (clustComp(pi_star,pi))
    },
    finally = {
     return(res)
    }
  )
}




# Accuracy Measures -------
#' Different accuracy measures
#' @param res match(0/1), tp, fp, tn, fn (pair_contingency function summed over mult. runs)
#' @param beta for Fscore
#' @return result avgd. over multiple runs
#' 
accuracies = function(returns, v, beta=1) {
  
  counter =returns$counter[[v]]
  counter_clus =returns$counter_clus[[v]]
  seen_k = returns$seen_k[[v]]
  matches_k = returns$matches_k[[v]]
  randindex_k = returns$ri_k[[v]]
  
  K = length(matches_k)
  acc_k = rep(0, K)
  ri_k = rep(0, K)
  for(i in 1:K)
    if(seen_k[[i]]>0) {
      acc_k[i] = matches_k[[i]]/seen_k[[i]]
      ri_k[i] = randindex_k[[i]]/seen_k[[i]]
      }

  
  acc_km1 = acc_k[-1]
  acc_kmK = acc_k[-length(acc_k)]
  acc_kmK1 = acc_kmK[-1]
  
  
  tp= counter$tp
  fp= counter$fp
  tn= counter$tn
  fn= counter$fn
  it = counter$it
  pi_match = counter$pi_match
  #adj_ri= res$ri/res$it
  
  ri = (tp + tn) / (tp+fp+tn+fn)
  prec = tp / (tp+fp)
  rec = tp / (tp+fn)
  f = ((beta^2+1)*prec*rec)/(beta^2*prec + rec)
  
  counter_clus = lapply(counter_clus, function(sum_over_runs)
    return (sum_over_runs/it))
  
  
  #Sanity 
  if(v == "v1"| v=="v2"| v=="v3"){  
  Confusion_mat = returns$confus_vario
  if(v == "v2")  
    Confusion_mat = returns$confus_vario_n
  if(v == "v3")  
    Confusion_mat = returns$confus_vario_noone
  matches = diag(Confusion_mat)

  sapply(seq_along(matches), function(mi)
    assert(matches[mi]==matches_k[[mi]]))
  }
  result = list(rand_index=ri, 
                precision=prec, 
                recall=rec, 
                fscore=f, 
                beta=beta, 
                it=it,
                pi_matches=pi_match, 
                pi_acc_imbalanced=pi_match/it,
                pi_acc_k = acc_k,
                rand_index_k = ri_k,
                pi_acc = sum(acc_k)/length(acc_k), #not mixing k, but avgd over accuracies at each k(at k, avg over runs.)
                pi_acc_omit1=sum(acc_km1)/length(acc_km1), #also balanced, omit 1
                pi_acc_omitK=sum(acc_kmK)/length(acc_kmK), #also balanced, omit 5
                pi_acc_omitK1=sum(acc_kmK1)/length(acc_kmK1) #also balanced, omit both
  )
  return(c(counter_clus, 
           result))
           
}




#' Pistar vs. Pi - pair (dis)similarity
#' @param shape group sizes in pistar 
#' @param pi partition returned by method
#' @return match (0/1), Rand_index, TP,FP,TN,FN
#' 
pair_contingency_shape_pi = function(pi_star, pi,
                                     vb = FALSE){
  g_star = unique(pi_star)
  if(vb) print(paste(pi_star,collapse=""))
  if(vb) print(paste(pi,collapse=""))
  
  # TP = sim pairs of environments in same group
  tp = 0 
  tfp = 0 
  tn = 0
  tfn = 0

  at_g = 1
  
  #True groups
  for (g in g_star){

    
    # TP, FP within group -----
    i_star = which(pi_star==g) 
    pi_cls = pi[i_star] # Guess 
    if(vb) print(paste("Group with", paste(pi_cls,collapse="")))
    
    # TP from sim group elems
    for(cls in unique(pi_cls)){
      if(length(which(pi_cls==cls))>=2) 
        tp = tp + choose(length(which(pi_cls==cls)),2)
      if(vb) print(paste("TP pairs ", length(which(pi_cls==cls))))
    }
    # TP+FP from all group elems
    if(length(i_star)>=2) 
      tfp = tfp + choose(length(i_star),2)

    
    
    # TN, FN between all pairs of groups -----
    if(at_g+1<=length(g_star))
    for (g2 in g_star[(at_g+1):length(g_star)]){
      pi_cls_g2 = pi[which(pi_star==g2)] 
      if(vb) print(paste("   Groups ", paste(g, g2)))
      
      classes = unique(union(pi_cls, pi_cls_g2))
      at_c = 1
      for(cls in classes){        
        if(at_c+1<=length(classes)){
        for(cls2 in classes[(at_c+1):length(classes)]){
          if(vb) print(paste("      Classes ", paste(cls, cls2)))
          
          # TN from diff classes in diff groups
          occ = length(which(pi_cls==cls)) + length(which(pi_cls_g2==cls2))
          if(occ>=2) 
            tn = tn + choose(occ,2)
          if(vb) print(paste("      TN pairs ", occ))
          
        }
        }

        at_c = at_c + 1
      }
      # TN from any classes in diff groups
      occ = length(pi_cls) +length(pi_cls_g2)
      if(occ>=2) 
        tfn = tfn + choose(occ,2)
      if(vb) print(paste("      TFN pairs ", occ))
      
    }
    at_g = at_g+1

  }
 # assert(tfp>=tp)
#  assert(tfn>=tn)


  pi_match = !(FALSE %in% sapply(seq_along(pi_star), function(i) 
    pi_star[i]==pi[i]))
  if(pi_match) m = 1
  else m  = 0
  
  fn =  tfn-tn
  fp = tfp-tp
  
  ri = (tp + tn) / (tp+fp+tn+fn)

  return(list(pi_match=m, tp=tp, fp=fp, tn=tn, fn=fn, 
              ri = ri))#,it = 1))
}



#ää Not used right now
#' Pistar vs. Pi - accuracy if assign each group (true) to majority class (guess)
#' @param shape group sizes in pistar 
#' @param pi partition returned by method
#' @return match (0/1), Rand_index, TP,FP,TN,FN
#' 
accuracy_shape_pi = function (shape, pi) {
  #pi = pi
  assertlen(pi)
  assert(sum(shape) == length(pi))
  
  pistar_k = length(shape)
  pi_k = length(unique(pi))

  c_g = 0 #TP Groups
  m_g = 0 #FP+FN
  c_d = 0 #TP Environment
  m_d = 0
  pi_match = 1 # TP Partition
  
  partial_match = list() #overlap
  matched_cls = c() #cls in pi already matched to a group_sz (ensures mapping unique)
  at = 1
  
  
  #  1.Exact matches

  for (group_size in shape){ 
    
    # One group
    elems = pi[at:(at+group_size-1)] 
    cls = unique(elems)[1]
    
    contained = length(unique(elems))==1 # nec: all are (contained) in the same class
    covering = length(pi[which(pi==cls)]) == group_size #suf: no other sets in group 
    is_cor = contained & covering 
    
    if(is_cor) {
      c_g = c_g + 1
      c_d = c_d + group_size
      matched_cls = c(matched_cls, cls)
    }
    else{
      m_g = m_g + 1
      pi_match = 0
      #Special case for datasets: allow smaller groups/partial overlap 
      if(group_size==1)
        m_d = m_d + 1
      else 
        partial_match[[length(partial_match)+1]] = list("sz"=group_size,"at"=at)
    }
    at = at + group_size
  }
  
  
  # 2. Majority class for partially correct groups 
  
  if(length(partial_match)){
    
    # Find Mapping: groups -> classes, st. max. overlap 
    
    partial_match = list.sort(partial_match, ("sz")) 
    
    for (group in partial_match){
      group_sz = group$sz
      at = group$at
      
      group_rng = at:(at+group_sz-1)
      sub = pi[group_rng]
      rest_cls = pi[-group_rng]
      diff_cls = sort(table(sub), decreasing = TRUE) #candidate classes for this group
      
      matched = FALSE
      for (main_cls in names(diff_cls)){

        if(main_cls %in% matched_cls | #alr assigned 
           main_cls %in% rest_cls) next #not a strictly smaller overlap
        
        matched = TRUE
        overlap = length(which(sub==main_cls))
        c_d = c_d + overlap
        m_d = m_d + group_sz - overlap
        matched_cls = c(matched_cls, main_cls)
        break
      }
      if(!matched) m_d = m_d + group_sz
    }
  }
  assert(c_g+m_g==length(shape))
  assert(c_d+m_d==length(pi))
  
  match_result = c(pi_match, 1-pi_match, 
                   c_g, m_g,
                   c_d, m_d)
          #0, 0, 0, 0) #these cols are f model statistics later
  return(match_result)
}
