#' Main Algorithm:
#' Given data \code{\{X_e, Y_e\}},  in each context, 
#' find causal parents and partition in \code{Y = f_{PI}(pa(Y))+N}  
#' @details calls \code{Vario_Pi_search}; creates & updates \code{cls_subset_search}
#' @param data_E1_En list: \code{E -> \{X_e, Y_e\}} 
#' @param X_max_n max. length of subsets of X to consider. default is 4.
#' @param it_n alt. to max. number of subsets, restrict iterations
#' @param it_pre rounds of precomputing to find confidence intervals 
#' @param confints (opt.) if precomputed already
#' @param X_names (for display) covariates X names
#' @param target_nm (for display) target Y name  
#' @param vb verbose
#' @param greedy_k greedy pi search
#' @return \code{resultVario} class summarizing results
#' @export
Vario_subset_search = function(D_E1_En, 
                               X_names,
                               X_max_n=4,
                               it_pre=50, 
                               it_n = NULL,
                               greedy_k=FALSE, vb=TRUE,
                               target_nm="y",
                               confints = NULL,
                               version_mdl = TRUE,
                               version_simp = TRUE
                               ){ 
  X_max_n=min(X_max_n, length(X_names))
  E_n = length(D_E1_En)
  X_n = length(X_names)
  assert_Data_E1_En (D_E1_En, X_n) 
  
  if(!length(it_n))
    it_n = sum(sapply(1:X_max_n, function(i) return(choose(X_n, i))))
  
  #Confidence intervals
  # by generating reference data, meaning expected scores of variables X s.t. Y=varphiX+N but varphi do NOT have a partition in environments
  if(!length(confints)){
  if(vb) cat("\n(vario) init ... ")
  if(vb) tic(paste0("\n(vario) ", it_pre, " calls", sep=""))
  confints  = list(conf_pione=sample_conf_pi_one(E_n, it_pre, FALSE,version_mdl = version_mdl),  
                   conf_nogroups= sample_conf_nogroups(E_n, it_pre, FALSE,version_mdl = version_mdl),
                   conf_best=sample_conf_pi_best(E_n, it_pre, FALSE,version_mdl = version_mdl)) 
  }
  config = init_config_analysis(confints, greedy_k, version_mdl=version_mdl, version_simp=version_simp)
   
  if(vb) toc()  
  
  if(vb) cat(paste0("(vario) Iterations: ",it_n))
  #Result
  cls_subset_search = new("cls_subset_search")
  cls_subset_search$init(X_n, X_max_n, 
                         D_E1_En, config,
                         target_nm=target_nm, 
                         covariates_nm=X_names)
  it = 0 
  X_n = length(X_names)
  
  
  # Each subset size Xs
  for (Xs_n in 1:X_max_n){
    if(it > it_n) break
    if(vb) tic(paste0("(vario) |X| = ",Xs_n))
    
    # Each subset Xs of size Xs_n 
    subset_ids = as.matrix(combn(1:X_n, Xs_n))
    for (i in 1:ncol(subset_ids)){
      it = it + 1 
      if(it > it_n) break
      if(it%%10==0)
              if(vb)
                cat(paste0(", ", it))
      
      subset_id = subset_ids[,i]
      Xs_name = names(X_names)[subset_id]
      if(!length(Xs_name)) Xs_name = paste0(subset_id, collapse="")
      
       
      # Edit in Xs, updating seen partitions and so on
      cls_subset_search$edit(subset_id, Xs_name, it) 
    }
    if(vb) toc()
  }  
  
  #Postprocess, intersecting subsets to obtain X_Pi and averaging scores
  
  cls_subset_search$finish()  
  return(cls_subset_search)
  
  
}
