 #' Main Partition search Algorithm: 
#' for one subset \code{X={X1,...Xm}}, find Pi in \code{Y = f_{PI}(pa(Y))+N},
#' called by \code{Vario_subset_search}
#' @param data_E1_En list: \code{E -> \{X_e, Y_e\}}.
#' @param X_degrees regression degree of each covariate
#' @param config list including flags greedy_k, meta_computing  
#' @export
Vario_Pi_search = function(data_E1_En, X_degrees, config){
  X_n = length(X_degrees)
  check_main_search(data_E1_En, X_degrees, config)
  
   
  if(config$verbose) tic("(vario)       Total, one run")
 
  #Regression in each context ----- 
  #result has local coefficients of each X in E1_En      
  regr_result = regression_E1_En(data_E1_En,  
                                 unlist(X_degrees)) 
  
  #Stepwise regression/ Regression Spline with fixed knots
  #else regr_result = regression_piecewise_E1_En(data_E1_En,unlist(X_degrees)) 

  Pi_search_cls = new("cls_pi_search")
  Pi_search_cls$init(data_E1_En,
                     regr_result,
                     config) 
  #Pi search f each X -----
  # over cfs of X in E1_En
  for(xi in 1:X_n)
    Pi_search_cls = pi_search(Pi_search_cls, xi, config)
  Pi_search_cls$finish_pis() 
  
  if(config$verbose) toc()
  return(Pi_search_cls)
}



#' Pi search given X  
#' @param Ranking Pi_search_cls
#' @param xi curret X
#' @param vb verbose
#' @export
pi_search = function (Ranking, xi, config){ 
  
  Cf_mat = Ranking$Cf_X_E1_En[[xi]]
  lconf_mat = Ranking$lconf[[xi]]
  rconf_mat = Ranking$rconf[[xi]]
  # mat:   D1      D2  
  # [1,]  alpha1 alpha2
  # [2,]  beta1
   
  #Pi search for each coefficient ----
  toeach_cf = 1:nrow(Cf_mat)
  
  lapply(toeach_cf, function(ci){
    C_history = init_history_C(xi, ci, 
                               Cf_mat, 
                               lconf_mat, 
                               rconf_mat,
                               version_simp=config$version_simp) 
    tree = tree_search(Ranking, C_history, config$verbose)

    tree$Ranking$check_C(tree$C_history)
    Ranking = tree$Ranking  
  }) 
  return(Ranking)
  
}
 
# sanity
check_main_search = function(data_E1_En, X_degrees, config) {
 lapply(data_E1_En, function(E){ 
  assert ("y" %in% names(E))
  sapply(1:length(X_degrees), function(xi)
    assert(paste0("x", xi) %in% names(E)))
})  
}
