setwd("~/Vario/R")
library(devtools) 
load_all() 



cat("Edges for Vario-MDL") 

# some preprocessing
X_max_n = 5
D = load_Sachs_data()
names(D$E8)[4]="PIP2"
names(D$E8)[5]="PIP3"
names(D)= sapply(1:length(D), function(i) 
  return(paste0("E", i)))  
X_names = names(D[[1]])
 

#To save time, load result of Vario and run VarioG from .Rdata files
cat("(varioG) Loading all edges from Vario ")

load("~/Vario/reproduce_sachs/reproduced_edges_Vario_setsize5.Rdata")

cat("(varioG) Pruning step in VarioG ")

res = get_edges_nonancestral (edges_sachs_mdl_nopruning, consider_3anc = FALSE, X_names, consider_conf_afterwards = FALSE)
edges_sachs_mdl_pruning  = list.sort (as.list( res[[1]]), gain)  


#Consensus
adj_truth = consensus_augmented_graph_Sachs(X_names)
adjs = consensus_ancestral_graph(adj_truth)
adj_anc = adjs[[1]]
adj_anc3 = adjs[[2]]

cat("(varioG) Edges by decreasing gain values: ")

get_counts_ground_truth(edges_sachs_mdl_pruning, adj_anc, adj_anc3, adj_truth, X_names)
 
cat("(varioG) Finished  ")

 

#To reproduce without loading .Rdata files
reproduce_vario_sachs_inefficient = function(){
   
  X_max_n = 5
  D =  load_Sachs_data()
  names(D$E8)[4]="PIP2"
  names(D$E8)[5]="PIP3"
  names(D)= sapply(1:length(D), function(i) 
    return(paste0("E", i)))  
  X_names = names(D[[1]])
  E_n = length(D)
  
  cat("\n(varioG) init ... ")
  tic(paste0("\n(varioG) ", it_pre, " calls", sep=""))
  it_pre = 50
  confints = list(conf_pione =sample_conf_pi_one(E_n, it_pre, FALSE,version_mdl = TRUE), 
                  conf_nogroups= sample_conf_nogroups(E_n, it_pre, FALSE,version_mdl = TRUE),
                  conf_best= sample_conf_pi_best(E_n, it_pre, FALSE,version_mdl = TRUE)) 
  toc()  
  
  
  edges_per_Y = list()
  results_per_Y = list()
  cat("(varioG) Edge search for each target Y ")
  
  for (y_i in list(1,2,3,4,5,6,7,8,9,10,11)){
    y_nm = X_names[y_i]
    cat("\n(varioG) Y =", y_i, ":" , y_nm, "\n")
    xs = (1:length(D[[1]]))[-y_i]
    Dy = transform_xls_data(D, xs, c(y_i))
    D1_Dn = Dy$D_data
    x_nms = Dy$X_names[which(Dy$X_names != "y")]  
    
    
    RESULT = Vario_subset_search(D1_Dn, x_nms,  
                                 X_max_n=X_max_n,
                                 confints = confints,
                                 vb=vb, version_mdl=TRUE,
                                 version_simp=TRUE,
                                 greedy_k=greedy_k)
    results_per_Y[[y_i]] = RESULT
    edges_per_Y[[y_i]] = RESULT$X_Pi_ranking
  }
  
  
  
  
  cat("(varioG) Order edges per node (and cmp MDL gain)")
  names(edges_per_Y) =X_names
  edges_ordered = get_edge_list(edges_per_Y, results_per_Y, X_names,confints_8, consider_conf = FALSE) #order by target: list node-> ingoing edges
  edges_sachs_mdl_nopruning =  list.sort (as.list(edges_ordered), gain)

  save(edges_sachs_mdl_nopruning,file="~/Vario/reprouce_sachs_mdl_simp/reproduced_edges_Vario_setsize5.Rdata")
  
  
  cat("(varioG) Finished: all edges from Vario ")
  
  
  cat("(varioG) Pruning step in VarioG ")
  
  res =  get_edges_nonancestral (edges_sachs_mdl_nopruning, consider_3anc = FALSE, X_names, consider_conf_afterwards = FALSE)
  edges_sachs_mdl_pruning  = list.sort (as.list( res[[1]]), gain)  
  
  adj_truth = consensus_augmented_graph_Sachs(X_names)
  get_counts_ground_truth(res[[1]], adj_truth, X_names)
  
  
  cat("(varioG) Finished: all edges from VarioG ")
  save(edges_sachs_mdl_pruning,file="~/Vario/reprouce_sachs_mdl_simp/reproduced_edges_VarioG_setsize5.Rdata")
  
  return(edges_per_Y, results_per_Y)
}

# Output
# Edge praf      ->       pmek    (+)
# Edge praf      ->       PKA     (-)
# Edge pmek      ->       praf    (+)
# Edge plcg      ->       PIP2    (+)
# Edge plcg      ->       PIP3    (+)
# Edge plcg      ->       PKC     (+)
# Edge plcg      ->       P38     (a)
# Edge plcg      ->       pjnk    (a)
# Edge PIP2      ->       pmek    (a)
# Edge PIP2      ->       plcg    (+)
# Edge PIP3      ->       PKA     (aa)
# Edge p44/42    ->       pakts473        (+)
# Edge p44/42    ->       PKA     (+)
# Edge pakts473  ->       p44/42          (+)
# Edge pakts473  ->       PKA     (-)
# Edge PKA       ->       praf    (+)
# Edge PKA       ->       pmek    (+)
# Edge PKA       ->       plcg    (a)
# Edge PKA       ->       PIP2    (a)
# Edge PKA       ->       p44/42          (+)
# Edge PKA       ->       pakts473        (+)
# Edge PKA       ->       PKC     (+)
# Edge PKA       ->       P38     (+)
# Edge PKA       ->       pjnk    (+)
# Edge PKC       ->       pmek    (+)
# Edge PKC       ->       P38     (+)
# Edge P38       ->       praf    
# Edge P38       ->       pmek    
# Edge P38       ->       PIP2    
# Edge P38       ->       pakts473        
# Edge P38       ->       PKC     (-)
# Edge pjnk      ->       PIP2    
# Edge pjnk      ->       PKA     (-)
# Results: 
#TP   / FP (thereof: A / A3 / R )  / missed = f1 
#18 /  15 ( 5 1 4 ) / 13 = 0.5625 

#Meaning: A = ancestral but not directly causal or anticausal 
#A3 = ancestral over 2 nodes but not directly ancestral,causal,anticausal 
#R = anticausal (and not bidirectional)


# Compared to Vario without pruning: 
#Results: 
#TP   / FP (thereof: A / A3 / R )  / missed = f1 
#26 /  38 ( 14 8 8 ) / 5 = 0.5473684 


