setwd("~/Vario/R")
library(devtools)
library(readr)
load_all() 


#---- TODO: Customize the target directories here --------------
 
maindir_nm = "~/Vario/out"

# Data and ground truth graphs are here
variodir_nm = "~/Vario/out/synthetic_graphs_vario/"
jcidir_nm = "~/Vario/out/synthetic_graphs_jci/"
# Varios Results are here
outdir_nm = "~/Vario/out/synthetic_out_vario/"
outdir2_nm = "~/Vario/out/synthetic_out_jci_icp/"
outdir3_nm = "~/Vario/out/synthetic_out_utigsp/"

out_createdir("out/")
out_createdir(outdir_nm)
out_createdir(outdir2_nm)
out_createdir(outdir3_nm)
out_createdir(variodir_nm) 
out_createdir(jcidir_nm) 

subdir_nms = list("A_diagonal_shift/", 
                  "B_groups_shift/", 
                  "C_groups_changes/",  
                  "D_2groups_changes/",
                  "E_mgroups_changes/",
                  "F_hard_interventions/",
                  "G_2hard_interventions/")
for(subdir_nm in subdir_nms){
  out_createdir(paste0(outdir_nm, subdir_nm))
  out_createdir(paste0(outdir2_nm, subdir_nm))
  out_createdir(paste0(outdir3_nm, subdir_nm))
  out_createdir(paste0(jcidir_nm, subdir_nm))
  out_createdir(paste0(variodir_nm, subdir_nm))
}

#---- Precompute --------------
N_contexts = 5
confints_basic = list(conf_pione=sample_conf_pi_one(N_contexts, 50, FALSE, FALSE),
                      conf_nogroups= sample_conf_nogroups(N_contexts, 50, FALSE, FALSE),
                      conf_best=sample_conf_pi_best(N_contexts, 50, FALSE, FALSE))

# --- Simulate data --------------
# R code in R/exp_synthetic/generate_DAG_linear.R
N_variables = 5
N_regimes =N_contexts -1

for (seed in 1:50) {
  subdir_nm = "A_diagonal_shift/"
  simul_linear_stochastic_adapted(paste0(variodir_nm,subdir_nm),
                                  paste0(jcidir_nm,subdir_nm),
                                  N_variables,N_regimes, 1,1,1000,1,0,
                                  seed=seed, diagonal_setting = FALSE, 
                                  shift_interventions = TRUE,
                                  hard_interventions = FALSE,
                                  n_context_to_target = 1)
  #A: Interventions on Y: can be read from  "targets", 1st col of 
  
  subdir_nm = "B_groups_shift/"
  simul_linear_stochastic_adapted(paste0(variodir_nm,subdir_nm),
                                  paste0(jcidir_nm,subdir_nm),
                                  N_variables,N_regimes, 1,1,1000,1,0,
                                  seed=seed, diagonal_setting = FALSE, 
                                  shift_interventions = TRUE,
                                  hard_interventions = FALSE,
                                  n_context_to_target = 2)
  #B: 1st and 2nd col 
  
  
  #CHANGES --------------
  subdir_nm = "C_groups_changes/"
  simul_linear_stochastic_adapted(paste0(variodir_nm,subdir_nm),
                                  paste0(jcidir_nm,subdir_nm),
                                  N_variables,N_regimes, 1,1,1000,1,0,
                                  seed=seed, diagonal_setting = FALSE, 
                                  shift_interventions = FALSE,
                                  mechanism_changes = TRUE,
                                  hard_interventions = FALSE,
                                  n_context_to_target = 1,
                                  multiple_changes = FALSE)
  #C: 1st col 
  
  subdir_nm = "D_2groups_changes/"
  simul_linear_stochastic_adapted(paste0(variodir_nm,subdir_nm),
                                  paste0(jcidir_nm,subdir_nm),
                                  N_variables,N_regimes, 1,1,1000,1,0,
                                  seed=seed, diagonal_setting = FALSE, 
                                  shift_interventions = FALSE,
                                  mechanism_changes = TRUE,
                                  hard_interventions = FALSE,
                                  n_context_to_target = 2,
                                  multiple_changes = FALSE)
  #D:  1st and 3rd col  
  subdir_nm = "E_mgroups_changes/"
  simul_linear_stochastic_adapted(paste0(variodir_nm,subdir_nm),
                                  paste0(jcidir_nm,subdir_nm),
                                  N_variables,N_regimes, 1,1,1000,1,0,
                                  seed=seed, diagonal_setting = FALSE, 
                                  shift_interventions = FALSE,
                                  mechanism_changes = TRUE,
                                  hard_interventions = FALSE,
                                  n_context_to_target = 1,
                                  multiple_changes = TRUE)
 #E:   1st, 4th, 5th col 
  
  
  #HARD INTERVENTIONS ------------
  subdir_nm = "F_hard_interventions/"
  simul_linear_stochastic_adapted(paste0(variodir_nm,subdir_nm),
                                  paste0(jcidir_nm,subdir_nm),
                                  N_variables,N_regimes, 1,1,1000,1,0,
                                  seed=seed, diagonal_setting = FALSE, 
                                  shift_interventions = FALSE,
                                  hard_interventions = TRUE,
                                  n_context_to_target = 1)
  #F: targets: 1st col
  
  
  
  
  subdir_nm = "G_2hard_interventions/"
  simul_linear_stochastic_adapted(paste0(variodir_nm,subdir_nm),
                                                      paste0(jcidir_nm,subdir_nm),
                                                      N_variables,N_regimes, 1,1,1000,1,0,
                                                      seed=seed, diagonal_setting = FALSE, 
                                                      shift_interventions = FALSE,
                                                      hard_interventions = TRUE,
                                                      n_context_to_target = 2)
  #G: targets: 1st col,3rd col
}
