setwd("~/Vario/R")
library(devtools)
library(readr)
load_all() 


#---- TODO: Customize the target directories here --------------
 
maindir_nm = "~/Vario/out"

# Data and ground truth graphs are here
variodir_nm = "~/Vario/out/synthetic_graphs_vario/"
jcidir_nm = "~/Vario/out/synthetic_graphs_jci/"
# Varios Results are here
outdir_nm = "~/Vario/out/synthetic_out_vario/"

out_createdir("out/")
out_createdir(outdir_nm)
out_createdir(variodir_nm) 
out_createdir(jcidir_nm) 

subdir_nms = list("A_diagonal_shift/", 
                  "B_groups_shift/", 
                  "C_groups_changes/",  
                  "D_2groups_changes/",
                  "E_mgroups_changes/",
                  "F_hard_interventions/",
                  "G_2hard_interventions/")
for(subdir_nm in subdir_nms){
  out_createdir(paste0(outdir_nm, subdir_nm))
  out_createdir(paste0(jcidir_nm, subdir_nm))
  out_createdir(paste0(variodir_nm, subdir_nm))
}


cat("Make sure that Data is in out/synthetic_graphs_vario/ by running generate_DAG_linear.R first." )

N_variables = 5
N_regimes =N_contexts -1


#---- Precompute --------------
N_contexts = 5
confints_basic = list(conf_pione=sample_conf_pi_one(N_contexts, 50, FALSE, FALSE),
                      conf_nogroups= sample_conf_nogroups(N_contexts, 50, FALSE, FALSE),
                      conf_best=sample_conf_pi_best(N_contexts, 50, FALSE, FALSE))


# --- Run Vario --------------
# R code in R/exp_synthetic/check_DAG_linear.R
for(subdir_nm in subdir_nms){
for(version_mdl in list(TRUE,FALSE)){
  if(version_mdl) version_nm = "MDL/"
  else version_nm  = "Emp/"
  
  out_createdir(paste0(outdir_nm, subdir_nm,version_nm))
  x = Vario_on_DAG_linear(N_contexts,
                      confints=confints_basic, 
                      dir_nm = variodir_nm,
                      outdir_nm = outdir_nm,
                      subdir_nm = subdir_nm,
                      blanket_only = TRUE,
                      version_mdl = version_mdl)  
  
  # -> Files "pi_Ncontexts_Ngroups.txt" for Ngroups in [0,Ncontexts]

}
}
        
# --- Show Results --------------
# R code in R/exp_synthetic/check_results.R
#
cat("-------------\nRESULTS\n-------------","\nVarioMDL:", version_mdl, "\nSetting:", subdir_nm)
#decision_rate_pi_causal(list(paste0(outdir_nm,subdir_nm,"MDL/", "pi_5_2.txt"))) 
  
  #PLOT 1 Shift, diagonal#
  subdir_nm = "A_diagonal_shift/"
  cat("\n---------------\nVARIO MDL on", subdir_nm,  "\n---------------\n")   
  decision_rate_pi_causal(list(paste0(outdir_nm,subdir_nm,"MDL/", "pi_5_2.txt"))) 
  
  cat("\n---------------\nVARIO Emp on ", subdir_nm,  "\n---------------\n")
  decision_rate_pi_causal(list(paste0(outdir_nm,subdir_nm,"Emp/", "pi_5_2.txt"))) 
  
  
  #PLOT 2 Shift, groups
  subdir_nm = "B_groups_shift/"
  decision_rate_pi_causal(list(paste0(outdir_nm,subdir_nm,"MDL/", "pi_5_2.txt"),paste0(outdir_nm,subdir_nm,"MDL/", "pi_5_3.txt"))) 
  decision_rate_pi_causal(list(paste0(outdir_nm,subdir_nm,"Emp/", "pi_5_2.txt"),paste0(outdir_nm,subdir_nm,"MDL/", "pi_5_3.txt"))) 
  
  # PLOT3  Hard interventions
  subdir_nm = "G_2hard_interventions/"
  decision_rate_pi_causal(list(paste0(outdir_nm,subdir_nm,"MDL/", "pi_5_2.txt"),paste0(outdir_nm,subdir_nm,"MDL/", "pi_5_3.txt"))) 
  decision_rate_pi_causal(list(paste0(outdir_nm,subdir_nm,"Emp/", "pi_5_2.txt"),paste0(outdir_nm,subdir_nm,"MDL/", "pi_5_3.txt"))) 
  
  # PLOT4 Mechanism Changes
  subdir_nm = "E_mgroups_changes/" # results for D are similar
  decision_rate_pi_causal(list(paste0(outdir_nm,subdir_nm,"MDL/", "pi_5_2.txt"),paste0(outdir_nm,subdir_nm,"MDL/", "pi_5_3.txt"))) 
  decision_rate_pi_causal(list(paste0(outdir_nm,subdir_nm,"Emp/", "pi_5_2.txt"),paste0(outdir_nm,subdir_nm,"MDL/", "pi_5_3.txt"))) 
  
   
  
 
  