# Test Varios ability to find a partition for a variable Y with known causal parents  -------
# Example: 
# X1 -> Y -> X2 in 5 contexts
# a mechanism change in context 5, i.e. causal coefficient of f:X1->Y changes
# correspoding true partition: [[1,2,3,4],[5]], Vario needs to find this exact partition
# here, generate such partitions at random, and data for Y and its parents in a random DAG 
# order decisions by confidence and show, for each %value of decisions considered, what the accuracy for the best x% partitions is 
setwd("~/Vario/R")
library(devtools) 
load_all() 


#Precompute confidence intervals (needed only for the empirical score) -------

confints_3 = gen_confidence_intervals(C_n = 3)
confints_5 = gen_confidence_intervals(C_n = 5)
confints_8 = gen_confidence_intervals(C_n = 8)
confints_10 = gen_confidence_intervals(C_n = 10)

todo = list(3, 5, 8, 10)
todo_confints = list(confints_3, confints_5, confints_8, confints_10)

# Partition discovery for context_n=3,5,8,10 --------------------

out_createdir("out/")
out_createdir("out/check_pi/")

RUNS = 50

# Test both the MDL-score (presented in the main paper) 
# and the empirical score based on the elbow criterion (included in the appendix)
for (version_mdl in list(TRUE, FALSE)) {
  for (i in 1:length(todo)){  
    
    context_n = todo[[i]]
    confints = todo_confints[[i]]
  
    for (seed in 1:RUNS)
      
      # Test decisions on a partition: for each partition of contexts, 
      # - generate a target Y in a random linear DAG G that has mechanism changes according to the partition
      # - let Vario predict a partition given Y and its causal parents in G
      # - show decision rate plots, i.e. order predicted partitions by score and show accuracy for different cutoff values
    synth_DAG_param(
      context_n,
      1000,
      FALSE,
      FALSE,
      FALSE,
      subdir_nm = "run/",
      confints_basic = confints,
      confints_mdl = confints,
      noise = c(0 , 0), # -> similarity of coefficients within group
      inter = c(1, 1.001), #-> difference/min. offset of coefficients between groups (relative difference because coefficients are scaled afterwards)
      intervene_pa_G = c(3, 3), #-> not relevant here
      version_mdl = version_mdl
    )
    
  } 
  #This will read the results in "out/check_pi/"
  if(version_mdl){
    cat("\nShowing decisionrates for MDL,", context_n, "contexts")
    show_decision_rates_mdl(context_n, 0, "run/")
  }
  else{
    cat("\nShowing decisionrates for EMP,", context_n, "contexts")
    show_decision_rates_basic(context_n, 0, "run/")
    
  }
}


  